UNIT DAS1601;

{ ---------------------------------------------------------------------------- }
{ |                       DAS-1601 Interface                                 | }
{ ---------------------------------------------------------------------------- }

INTERFACE

PROCEDURE DAS1601Init;
procedure DAS1601analogOut(Channel: byte; Value: word ); {channel: 0..1; value:0..4095}
Function DAS1601analogIn(channel:byte):word;             {channel: 0..7; value:0..4095}
Procedure DAS1601digitalOut(channel:byte;Value:byte);    {channel: 0..3}
Function DAS1601digitalIn(bitNR:byte):boolean;           {bitNr: 0..3}
Function DAS1601FastanalogIn:word;

IMPLEMENTATION

CONST
  {DAS-1601 registers:}
  adr=$310;
  ad0=adr+$0;
  ad1=adr+$1;
  mux=adr+$2;
  dio=adr+$3;
  da0lb=adr+$4;
  da0hb=adr+$5;
  da1lb=adr+$6;
  da1hb=adr+$7;
  statusa=adr+$8;
  ctrl=adr+$9;
  cnten=adr+$A;
  gain=adr+$B;
  timer0=adr+$C;
  timer1=adr+$D;
  timer2=adr+$E;
  timerctl=adr+$F;
  piopa=adr+$400;
  piopb=adr+$401;
  piopc=adr+$402;
  pioctl=adr+$403;
  disarm=adr+$404;
  bmden=adr+$405;
  mden1600=adr+$406;
  statusb=adr+$407;

PROCEDURE DAS1601Init;
var n:byte;
BEGIN
  Port[pioctl]:=$80;   {Configure Ports A, B, and C, all as output}
  Port[pioctl]:=$0;
  Port[mden1600]:=$40; { $00: disable, $40: enable 1600 mode}
  Port[bmden]:=$00;    { $00: disable, $40: enable burst mode}
  Port[disarm]:=$00;   { $00: enable, $40: disable conversions}
  Port[cnten]:=$03;    {high nibble: burst length, bit1: ctr0, bit0: ctr1/2}
  Port[ctrl]:=$00;     {bit7:INTE(0=disabld),6-4:int.Lvl.,2:DMAE,1-0:trig source};
  Port[gain]:=$00;     {Gain is bits0 and1, 00:gain1,01:gain10,10:gain100,11: gain500}
  for n:=0 to 2 do Port[PIOPA+n]:=0;
  Port[DIO]:=0;
END;

procedure DAS1601analogOut(Channel: byte; Value: word); {channel: 0..5; valeur:0..4095}
var dal,dah:byte;
begin
  if (channel<2) then begin
    if (Value>4095) then writeln('DAS1601 : analogOut : wrong value ',value);
    dal:=(Value shl 4) and $00f0;

    DAL:=0; {This only, because our card is not working correctly}

    dah:=(Value shr 4) and $00FF;
    port[da0lb+channel*2]:=dal;
    port[da0hb+channel*2]:=dah;
  end else writeln('DAS1601 : analogOut : wrong channel ',channel);
end;

Function DAS1601analogIn(channel:byte):word;
var n:word;
begin
  if (channel>7) then writeln('DAS1601 : analogIn : wrong channel ',channel);
  Port[mux]:=(channel or (channel shl 4)) and $77;
  for n:=0 to 100 do;
  Port[ad0]:=0;  {Start the conversion}
  REPEAT UNTIL (Port[statusa] AND $80)=0; {wait until conversion is finished}
  DAS1601analogIn:=PortW[ad0] shr 4;
end;

Function DAS1601FastanalogIn:word;
begin
  Port[ad0]:=0;  {Start the conversion}
  REPEAT UNTIL (Port[statusa] AND $80)=0; {wait until conversion is finished}
  DAS1601FastanalogIn:=PortW[ad0] shr 4;
end;

Procedure DAS1601digitalOut(channel:byte;Value:byte);
begin
  if (channel<4) then begin
    if (channel=3) then Port[dio]:=Value    {write to primary IO}
    else Port[piopa+channel]:=Value;        {write to secondary IO}
  end else writeln('DAS1601 : digitalOut : wrong channel ',channel);
end;

Function DAS1601digitalIn(bitNR:byte):boolean;
begin
  if (bitNR>3) then writeln('DAS1601 : digitalIn : wrong bitNR ',bitNr);
  port[piopa]:=255;
  port[piopb]:=255;
  port[piopc]:=255;
  DAS1601digitalIn:=(port[dio] and (1 shl bitNR))>0;
end;

BEGIN
  DAS1601Init;
END.