unit Exp;
{
  This unit contains all routines to command the experiment.
}
interface
uses unimess;

Type TDet=set of byte;

procedure TestSequence;
procedure PrepareShutters;
procedure MOTUpdate;
procedure SlowerUpdate;
procedure DetectionUpdate;
procedure AndorUpdate;
procedure MatroxUpdate;
procedure SetAndorCameraParameters;
procedure SetMatroxCameraParameters;
procedure Blink;
procedure SetAOMfrequencies;
procedure InitExp(initSRS:boolean);
procedure SweepDetuning(det:TDet);
procedure ChargeDetectTrap;
procedure ChargeRecaptureTrap;
procedure CalibrateVCO;
procedure Beep;
procedure TakeAbsorptionPicture;
procedure FilmAbsorptionPicture;
procedure TestInjection(isotope:integer);
procedure TestExpInjection(isotope:integer);
procedure TestSerialLink;
procedure TestShutters;
procedure InitCards;
procedure TakeProbePicture;
procedure TestNumber;
procedure OpenShutters;
procedure CloseShutters;
procedure SetMOTDetuning;
procedure Set7PrincipalDetuning(detuning:double);
procedure Set7RepumpDetuning(detuning:double);
procedure Set6PrincipalDetuning(detuning:double);
procedure Set6RepumpDetuning(detuning:double);
procedure SetOpticalPumpingDetuning6Li(OpticalPumping6repumpDetuning:double);
procedure SetOpticalPumpingDetuning7Li(OpticalPumping7repumpDetuning:double);
procedure SetDopplerCoolPrincipalFrequency(frequ:double);
procedure SetDopplerCoolRepumpFrequency(frequ:double);
procedure SetProbeDetuning(Isotop:integer);
procedure TestAnalogOutputs;
procedure TestAnalogInputs;
procedure TestDigitalInputs;
procedure SweepAnalogOutputs;
procedure TestDigitalOutputs;
procedure SweepDigitalOutputs;
procedure DoUniMess(mess:PUniMess);
procedure SendIEEE;
procedure WatchSRQ;
procedure TriggerCamera;
procedure TriggerCameraAndExposure;
function Detuning:string;
procedure CalibrateFluorescence;
procedure FluorescenceOszilloskop;
procedure CalibrateOszilloskop;
procedure FastOszilloskop;
procedure DancingMOT;
procedure Queue(Mess:string);
procedure QueueUniMess(mess:PUniMess);
procedure QueueTakeAbsorptionPicture;
procedure ExecuteQueue(EraseQueue:boolean);
procedure IterateQueue;
procedure ClearQueue;
procedure TestDualFlou;
procedure ShowMemory;
function IdleProc(action:boolean):boolean;

implementation
uses crt,dos,IO,ExpIO,Params,Timing,Vision,MixData,DAS1601,DDA06,MultiOut,
     Serial,IEEE,menu,menusys,MagTrap,Debug,SigGen,Sequence,NdYAG,PulsGen;
var oldexitproc:pointer;
type TDetuning=(MOTDetuning,CMOTDetuning,MolassesDetuning,DetectionDetuning,OpticalPumpingDetuning);
const AktDetuning:TDetuning=(MOTDetuning);

const Lithium7HyperfineSplitting=812.7;
const Lithium6HyperfineSplitting=228.2;
const TitleQuiet:boolean=false;
const NormPictureCounter:integer=0;
const NormPictureCounterMax=50;

procedure Beep;
var n:integer;
begin
  for n:=50 to 100 do begin
    sound(n*10);
    delay(1);
 end;
  nosound;
end;

procedure ErrorBeep;
var n,m:integer;
begin
  for m:=0 to 3 do begin
    for n:=50 to 100 do begin
      sound(n*10);
      delay(1);
      sound(n*30);
      delay(1);
      sound(n*40);
      delay(1);
    end;
    for n:=0 to 30 do begin
      sound(100*10);
      delay(1);
      sound(100*30);
      delay(1);
      sound(100*40);
      delay(1);
    end;
  end;
  nosound;
end;

procedure PotBeep;
var m,n:integer;
begin
  for m:=0 to 2 do begin
    for n:=50 to 75 do begin
      sound(n*10);
      delay(1);
    end;
    for n:=75 to 50 do begin
      sound(n*10);
      delay(1);
    end;
  end;
  nosound;
end;

function Detuning:string;
begin
  case Aktdetuning of
    MOTDetuning:Detuning:='MOT';
    CMOTDetuning:Detuning:='CMOT';
    MolassesDetuning:Detuning:='Molasses';
    DetectionDetuning:Detuning:='Detection';
    OpticalPumpingDetuning:Detuning:='Optical Pumping';
  end;
end;

procedure MOTUpdate;
begin
  SetAOM20Intensity(MOTIntensity);
  SetMOTCurrent(MOTMOTCurrent);
  SetTransfer1Voltage(0);
  IOUpdate;
  SetTransfer1Current(MaxTransfer1Current);
  SetTransfer2Voltage(MOTTransfer2Voltage);
  SetMOTVoltage(MOTMOTVoltage);
  SetAOMfrequencies;
end;

procedure SlowerUpdate;
begin
  SetAOM20Intensity(MOTIntensity);
  SetMOTCurrent(MOTMOTCurrent);
  SetTransfer1Voltage(0);
  SetFinalSlowerCurrent(SlowerCurrent);
  SwitchFinalSlowerVoltage(on);
  IOUpdate;
  SetTransfer1Current(MaxTransfer1Current);
  SetTransfer2Voltage(MOTTransfer2Voltage);
  SetMOTVoltage(MOTMOTVoltage);
  SetAOMfrequencies;
end;

procedure PrepareProbeBeam;
begin
  if TwoPictures and (CameraPosition=1) then begin
    if ExpansionTime6<ExpansionTime7 then DetectIsotop:=6 else DetectIsotop:=7;
    if (not AndorFKSTriggerMode) then begin
      if ExpansionTime6<ExpansionTime7 then begin
        ExpansionTime7:=ExpansionTime6+AndorFKSExposureTime;
      end else begin
        ExpansionTime6:=ExpansionTime7+AndorFKSExposureTime;
      end;
      AtomCleaningTime:=AndorFKSExposureTime;
    end;
  end;
  SelectProbeBeam(DetectIsotop=7);
  case CameraPosition of
    0:if DetectIsotop=7 then SetProbeConfiguration(Li6ProbeToIoffe) else SetProbeConfiguration(Li6ProbeToMOT);
    1:SetProbeConfiguration(ProbeMixed);
    {2:case FinalPosition of
        LowerTrap:SelectProbeBeam(LowerTrap7Li);
        HigherTrap:SelectProbeBeam(HigherTrap7Li);
      end;
    end;}
  end;
  ExposureTrigger(off);
  IOUpdate;
end;

procedure DetectionUpdate;
begin
  Set6LiProbeIntensity(Probe6MOTIntensity);
  Set7LiProbeIntensity(Probe7MOTIntensity);
  SetAOMfrequencies;
  PrepareShutters;
  PrepareProbeBeam;
  SetProbeConfiguration(ProbeMixed);
  IOUpdate;
end;

procedure AndorUpdate;
begin
  AndorBiningX:=1 shl round(ln(AndorBiningX)/ln(2));
  AndorBiningY:=1 shl round(ln(AndorBiningY)/ln(2));
  SetAndorCameraParameters;
end;

procedure MatroxUpdate;
begin
  SetMatroxCameraParameters;
end;

procedure SetMatroxCameraParameters;
begin
  VisionSetCameraParameters(0,MatroxBiningX,MatroxBiningY,MatroxStartX,
    MatroxStartY,MatroxWidth,MatroxHeight,MatroxEichX,MatroxEichY,
    MatroxMode,0,0,false,0,0,0,0,false,0,0,0,0,false,MatroxGain,MatroxOffset);
end;

procedure SetAndorCameraParameters;
begin
  VisionSetCameraParameters(1,AndorBiningX,AndorBiningY,AndorStartX,
    AndorStartY,AndorWidth,AndorHeight,AndorEichX,AndorEichY,
    0,AndorTemperature,AndorHorizontalSpeed,AndorFastKineticSeries,
    AndorFKSExposedRows,AndorFKSImages,AndorFKSShiftSpeed,AndorFKSExposureTime,
    PictureSplitting,AndorFKSPictureDistance,AndorFKSPictureOffset,
    AndorFKSPictureHeight,AndorFKSSoftBining,AndorFKSTriggerMode,0,0);
end;

procedure Set7PrincipalDetuning(detuning:double);
begin
  AOM1Frequency:=0.5*(Detuning
                      -0.5*Lithium7HyperfineSplitting
                      +0.5*AOM0Frequency
                      -AOM20MOTFrequency);
  SetAOMfrequency(1,AOM1frequency);
end;

procedure Set7RepumpDetuning(detuning:double);
begin
  AOM2Frequency:=0.5*(Detuning
                      +0.5*Lithium7HyperfineSplitting
                      +0.5*AOM0Frequency
                      -AOM20MOTFrequency);
  SetAOMfrequency(2,AOM2frequency);
end;

procedure Set6PrincipalDetuning(detuning:double);
begin
  AOM11Frequency:=0.5*(Detuning
                       -0.5*Lithium6HyperfineSplitting
                       +0.5*AOM10Frequency
                       -AOM20MOTFrequency);
  SetAOMfrequency(11,AOM11frequency);
end;

procedure Set6RepumpDetuning(detuning:double);
begin
  AOM13Frequency:=0.5*(Detuning
                       +Lithium6HyperfineSplitting
                       -0.5*AOM12Frequency
                       -AOM20MOTFrequency);
  SetAOMfrequency(13,AOM13frequency);
end;

procedure SetSlowerDetuning;
begin
  AOM3Frequency:=0.5*(Slower7PrincipalDetuning
                      -0.5*Lithium7HyperfineSplitting
                      +0.5*AOM0Frequency
                      -2*AOM1Frequency);
  AOM4Frequency:=0.5*(Slower7RepumpDetuning
                      +0.5*Lithium7HyperfineSplitting
                      +0.5*AOM0Frequency
                      -2*AOM2Frequency);
  SetAOMfrequency(3,AOM3frequency);
  SetAOMfrequency(4,AOM4frequency);
  AOM14Frequency:=0.5*(Slower6PrincipalDetuning
                       -0.5*Lithium6HyperfineSplitting
                       +0.5*AOM10Frequency
                       -2*AOM11Frequency);
  AOM15Frequency:=0.5*(Slower6RepumpDetuning
                       +Lithium6HyperfineSplitting
                       -0.5*AOM12Frequency
                       -2*AOM13Frequency);
  SetAOMfrequency(14,AOM14frequency);
  SetAOMfrequency(15,AOM15frequency);
end;

procedure SetMOTDetuning;
begin
  Set7RepumpDetuning(MOT7RepumpDetuning);
  Set7PrincipalDetuning(MOT7PrincipalDetuning);
  Set6RepumpDetuning(MOT6RepumpDetuning);
  Set6PrincipalDetuning(MOT6PrincipalDetuning);
  AktDetuning:=MOTDetuning;
  SetSlowerDetuning;
end;

procedure SetDopplerCoolPrincipalFrequency(frequ:double);
begin
  Set7PrincipalDetuning(frequ-2*AOM20DopplerCoolFrequency+AOM20MOTFrequency);
end;

procedure SetDopplerCoolRepumpFrequency(frequ:double);
begin
  Set7RepumpDetuning(frequ-Lithium7HyperfineSplitting-2*AOM20DopplerCoolFrequency+AOM20MOTFrequency);
end;

procedure SetOpticalPumpingDetuning6Li(OpticalPumping6repumpDetuning:double);
begin
  AOM13Frequency:=0.5*(OpticalPumping6repumpDetuning
                       +Lithium6HyperfineSplitting
                       -0.5*AOM12Frequency
                       -AOM22Frequency);
  SetAOMfrequency(13,AOM13frequency);
  AktDetuning:=OpticalPumpingDetuning;
end;

procedure SetOpticalPumpingDetuning7Li(OpticalPumping7repumpDetuning:double);
begin
  AOM2Frequency:=0.5*(OpticalPumping7repumpDetuning
                      +0.5*Lithium7HyperfineSplitting
                      +0.5*AOM0Frequency
                      -AOM22Frequency);
  SetAOMfrequency(2,AOM2frequency);
  AktDetuning:=OpticalPumpingDetuning;
end;

procedure SetCMOTDetuning;
begin
  Set7RepumpDetuning(CMOT7RepumpDetuning);
  Set7PrincipalDetuning(CMOT7PrincipalDetuning);
  Set6RepumpDetuning(CMOT6RepumpDetuning);
  Set6PrincipalDetuning(CMOT6PrincipalDetuning);
  AktDetuning:=CMOTDetuning;
end;

procedure SetMolassesDetuning;
begin
  Set7RepumpDetuning(Molasses7RepumpDetuning);
  Set7PrincipalDetuning(Molasses7PrincipalDetuning);
  Set6RepumpDetuning(Molasses6RepumpDetuning);
  Set6PrincipalDetuning(Molasses6PrincipalDetuning);
  AktDetuning:=MolassesDetuning;
end;

procedure SetProbeDetuning(Isotop:integer);
begin
  if (Isotop=7) or TwoPictures then begin
    AOM1Frequency:=0.5*((Probe7PrincipalDetuning+Probe7Offset)
                        -0.5*Lithium7HyperfineSplitting
                        +0.5*AOM0Frequency
                        -AOM21Frequency);
    AOM2Frequency:=0.5*((Probe7RepumpDetuning+Probe7Offset)
                        +0.5*Lithium7HyperfineSplitting
                        +0.5*AOM0Frequency
                        -AOM22Frequency);
    SetAOMfrequency(1,AOM1frequency);
    SetAOMfrequency(2,AOM2frequency);
  end;
  if (Isotop=6) or TwoPictures then begin
    AOM11Frequency:=0.5*((Probe6PrincipalDetuning+Probe6Offset)
                         -0.5*Lithium6HyperfineSplitting
                         +0.5*AOM10Frequency
                         -AOM21Frequency);
    AOM13Frequency:=0.5*((Probe6RepumpDetuning+Probe6Offset)
                         +0.5*Lithium6HyperfineSplitting
                         +0.5*AOM10Frequency
                         -AOM12Frequency
                         -AOM22Frequency);
    SetAOMfrequency(11,AOM11frequency);
    SetAOMfrequency(13,AOM13frequency);
  end;
  AktDetuning:=DetectionDetuning;
end;

procedure SetHideProbeDetuning(Isotop:integer);
begin
  if (Isotop=7) or TwoPictures then begin
    AOM1Frequency:=0.5*((HideProbe7PrincipalDetuning+Probe7Offset)
                        -0.5*Lithium7HyperfineSplitting
                        +0.5*AOM0Frequency
                        -AOM21Frequency);

    SetAOMfrequency(1,AOM1frequency);
  end;
  if (Isotop=6) or TwoPictures then begin
    AOM11Frequency:=0.5*((HideProbe6PrincipalDetuning+Probe6Offset)
                         -0.5*Lithium6HyperfineSplitting
                         +0.5*AOM10Frequency
                         -AOM21Frequency);
    SetAOMfrequency(11,AOM11frequency);
  end;
  AktDetuning:=DetectionDetuning;
end;


procedure SetAOMFrequencies;
var hAOM4frequency:double;
    PD,RD,D:double;
begin
  SetMOTDetuning;
  SwitchAOM20Frequency(off);
  SwitchAOM20(on);
  SetAOM20Intensity(MOTIntensity);
  SetMOTCurrent(MOTMOTCurrent);
  SetMOTVoltage(MOTMOTVoltage);
  SetTransfer1Voltage(0);
  IOUpdate;
  SetTransfer1Current(MaxTransfer1Current);
  SetTransfer2Voltage(MOTTransfer2Voltage);
  IOUpdate;
end;

procedure Title(t:string);
begin
  if not TitleQuiet then begin
    textBackground(blue);
    textcolor(yellow);
    clrscr;
    writeln('* ',t,' *');
    writeln;
  end;
end;

procedure BlackTitle(t:string);
begin
  if not TitleQuiet then begin
    textBackground(black);
    textcolor(yellow);
    clrscr;
    writeln('* ',t,' *');
    writeln;
  end;
end;

  procedure Block;far;
  var h:byte;
  begin
    SequenceStartBlockNoArg(Block);
      for h:=1 to 10 do begin
        SwitchOszilloskopTrigger(on);
        Wait(1*ID+5);
        SwitchOszilloskopTrigger(off);
        Wait(2*ID+5);
      end;
    SequenceEndBlock;
  end;


procedure TestSequence;
var TotalWait,TimeSinceLastWait,SequenceTime:double;
    n,m:byte;
begin
  Title('TestSequence');
  repeat
    StartCounter;
    StartWaitSum;
    SwitchOszilloskopTrigger(on);
    IOUpdate;
    InitPortList;
    StartSequenceDuringWait(3*ID+100);
    Wait(4*ID+100);
    SequenceStartPortListDuringWait(5*ID+100);
    for m:=1 to 5 do begin
      for n:=1 to 10 do begin
        SwitchOszilloskopTrigger(off);
        IOUpdate;
        SwitchOszilloskopTrigger(on);
        IOUpdate;
      end;
      Wait(6*ID+10);
    end;
    SequenceStopPortList;
    SwitchOszilloskopTrigger(off);
    Block;
    TotalWait:=GetTotalWait;
    TimeSinceLastWait:=GetTimeSinceLastWait;
    SequenceTime:=GetSequenceTime;
    IOUpdate;
    StopSequence;
    Wait(7*ID+100);
    writeln('TotalWait:',TotalWait:2:3,' SequenceTotalWait:',SequenceTime:2:3);
    writeln('TimeSinceLastWait:',TimeSinceLastWait:2:3, ' Total=',TotalWait-TimeSinceLastWait:2:3);
  until keypressed;
end;

var OldFluo6Li,OldFluo7Li:double;
procedure InitMeasureTwoIsotopeFlourescence;
begin
  OldFluo6Li:=0;
  OldFluo7Li:=0;
end;

procedure MeasureTwoIsotopeFlourescence(var Li7Fluorescence:double;var Li6Fluorescence:double;Cycles:word;OldWeight:double);
var Fluo6Li,Fluo7Li:double;
    n:word;
begin
  Fluo6Li:=0;
  Fluo7Li:=0;
  for n:=1 to Cycles do begin
    SwitchMaster6principal(on);
    SwitchMaster6repump(on);
    SwitchMaster7principal(off);
    SwitchMaster7repump(off);
    Wait(0.5);
    MeasureFluorescence(0);
    SwitchMaster7principal(on);
    SwitchMaster7repump(on);
    SwitchMaster6principal(off);
    SwitchMaster6repump(off);
    Wait(0.5);
    MeasureFluorescence(1);
    SwitchMaster6principal(on);
    SwitchMaster6repump(on);
    SwitchMaster7principal(on);
    SwitchMaster7repump(on);
    Wait(5);
    Fluo6Li:=Fluo6Li+GetFLuorescence(0);
    Fluo7Li:=Fluo7Li+GetFLuorescence(1);
  end;
  Fluo7Li:=Fluo7Li/Cycles;
  Fluo6Li:=Fluo6Li/Cycles;
  if OldWeight>0 then begin
    OldFluo6Li:=(Fluo6Li+OldFluo6Li*OldWeight)/(OldWeight+1);
    OldFluo7Li:=(Fluo7Li+OldFluo7Li*OldWeight)/(OldWeight+1);
    Li7Fluorescence:=OldFluo7Li;
    Li6Fluorescence:=OldFluo6Li;
  end else begin
    Li7Fluorescence:=Fluo7Li;
    Li6Fluorescence:=Fluo6Li;
  end;
  Wait(1);
end;

procedure FluorescenceOszilloskop;
var out:text;
    t:double;
    name:string;
    dt:double;
begin
  Title('Fluorescence Oszilloskop');
  write('Filename:');
  readln(name);
  write('dt [ms] :');
  readln(dt);
  t:=0;
  assign(out,name);
  rewrite(out);
  StartCounter;
  repeat
    MeasureFluorescence(0);
    writeln(out,t:2:2,' ',GetFluorescence(0)*100:2:2);
    gotoxy(1,6);
    writeln(t:2:2,' ',GetFluorescence(0)*100:2:2,'  ');
    if AkkusticFluorescence then sound(round(10*System.Exp(GetFluorescence(0)*System.Ln(2000))))
    else nosound;
    Wait(8*ID+dt);
    t:=t+dt;
  until keypressed;
  system.close(out);
end;

procedure CalibrateOszilloskop;
var t:double;
    name:string;
    dt:double;
begin
  Title('Calibrate Oszilloskop');
  repeat
    gotoxy(1,6);
    writeln(IOanalogIn(0):15,'     ');
    delay(100);
  until keypressed;
end;

procedure FastOszilloskop;
var out:text;
    t:double;
    name:string;
    dt:double;
    n,maxpoints,npoints:longint;
const MaxBuf=10;
      BufLength=32000;
type TBuffer=array[0..BufLength-1] of word;
     PBuffer=^TBuffer;
var Buf:array[0..MaxBuf] of PBuffer;
    aktbuf:integer;
    ende:boolean;
    thisbuf:word;
    thisentry:word;
CONST
  {DAS-1601 registers:}
  adr=$310;
  ad0=adr+$0;
  ad1=adr+$1;
  mux=adr+$2;
  dio=adr+$3;
  da0lb=adr+$4;
  da0hb=adr+$5;
  da1lb=adr+$6;
  da1hb=adr+$7;
  statusa=adr+$8;

begin
  Title('Fast Oszilloskop');
  write('Filename:');
  readln(name);
  write('dt [ms] :');
  readln(dt);
  write('Number of points :');
  readln(npoints);
  aktbuf:=-1;
  maxpoints:=-1;
  ende:=false;
  while (maxpoints<npoints) and (not ende) and (aktbuf<MaxBuf) do begin
    if MaxAvail>sizeof(TBuffer) then begin
      inc(aktbuf);
      new(Buf[aktbuf]);
      maxpoints:=maxpoints+BufLength;
    end else ende:=true;
  end;
  if npoints>maxpoints then npoints:=maxpoints+1;
  thisbuf:=0;
  thisentry:=0;
  DAS1601analogIn(0);
  StartCounter;
  Wait(9*ID+1);
  for n:=1 to npoints do begin
    Port[ad0]:=0;  {Start the conversion}
    asm cli end; {switch interrupts off}
    REPEAT UNTIL (Port[statusa] AND $80)=0; {wait until conversion is finished}
    asm sti end; {switch interrupts on}
    Buf[thisbuf]^[thisentry]:=PortW[ad0] shr 4;
    Wait(10*ID+dt);
    if thisentry=BufLength-1 then begin
      thisentry:=0;
      inc(thisbuf);
    end else inc(thisentry);
  end;
  thisbuf:=0;
  thisentry:=0;
  t:=0;
  assign(out,name);
  rewrite(out);
  for n:=1 to npoints do begin
    writeln(out,t:2:6,' ',Buf[thisbuf]^[thisentry]);
    t:=t+dt*0.001;
    if thisentry=BufLength-1 then begin
      thisentry:=0;
      inc(thisbuf);
    end else inc(thisentry);
  end;
  system.close(out);
  while (aktbuf>=0) do begin
    dispose(Buf[aktbuf]);
    dec(aktbuf);
  end;
end;

procedure TestSerialLink;
var inp,ein:char;
    port:byte;
    z:byte;
    n:longint;
begin
  Title('Test Serial Link');
  write('COM port (1/2/3/4) : ');
  ein:='*';
  repeat
    repeat until keypressed;
    while keypressed do ein:=readkey;
  until (ein='1') or (ein='2') or (ein='3') or (ein='4');
  writeln('COM'+ein);
  writeln;
  writeln('*: Flush 10 kB, ESC: end');
  case ein of
    '1':port:=1;
    '2':port:=2;
    '3':port:=3;
    '4':port:=4;
  end;

  {InitCOM(port);
  SetProtocol(port,9600,8,1,Pnone,false,true);}

  OpenPort(Port,115200);

  ein:='';
  repeat
    repeat
      {if AskCharNum(port)>0 then write(GetChar(port));}
       if GetChar(port,inp) then write(inp);
    until keypressed;
    while keypressed do ein:=readkey;
    if (ein<>'') and (ein<>#27) then begin

      if ein='*' then begin
        z:=0;
        for n:=0 to 10000 do begin
          PutChar(port,char(z+byte('0')));
          inc(z);
          if z=10 then z:=0;
        end;
      end;
      PutChar(port,ein);
      TextColor(lightgreen);
      write(ein);
      TextColor(Yellow);
      ein:='';
    end;
  until ein=#27;
end;

procedure SelectIsotope(isotope:integer);
begin
  if isotope=7 then begin
    ShutterMOT7principal(open);
    ShutterMOT7repump(open);
    ShutterMOT6principal(close);
    ShutterMOT6repump(close);
    ShutterSlower7principal(open);
    ShutterSlower7repump(open);
    ShutterSlower6principal(close);
    ShutterSlower6repump(close);
  end;
  if isotope=6 then begin
    ShutterMOT7principal(close);
    ShutterMOT7repump(close);
    ShutterMOT6principal(open);
    ShutterMOT6repump(open);
    ShutterSlower7principal(close);
    ShutterSlower7repump(close);
    ShutterSlower6principal(open);
    ShutterSlower6repump(open);
  end;
  SwitchAOM20(on);
  SwitchAOM20Frequency(off);
  ShutterMOT(open);
  ShutterAOM20(open);
end;

procedure TestExpInjection(isotope:integer);
var ein:char;
    startmf,mf:integer;

  procedure Wait;
  var action:boolean;
  begin
    delay(500);
    if keypressed then begin
      while keypressed do ein:=readkey;
      action:=true;
      if ein=' ' then repeat IdleProc(action);action:=false; until keypressed;
      while keypressed do ein:=readkey;
    end;
  end;

begin
  SelectIsotope(isotope);
  repeat
    SetProbeDetuning(Isotope);
    IOUpdate;
    Title('Exposition Detuning');
    Wait;
    SetHideProbeDetuning(Isotope);
    IOUpdate;
    Title('Hide Exposition Detuning');
    Wait;
    if ein=#27 then exit;
    if isotope=7 then begin
      if DopplerCoolBeamIsPrincipal[1] then begin
        SetDopplerCoolPrincipalFrequency(DopplerCoolStartFrequency[1]);
        ShutterMOT7repump(off);
        ShutterMOT7principal(on);
      end else begin
        SetDopplerCoolRepumpFrequency(DopplerCoolStartFrequency[1]);
        ShutterMOT7repump(on);
        ShutterMOT7principal(off);
      end;
      IOUpdate;
      Title('Doppler Cool 1 Start Detuning');
      Wait;
      if ein=#27 then exit;
      if DopplerCoolBeamIsPrincipal[2] then begin
        SetDopplerCoolPrincipalFrequency(DopplerCoolStartFrequency[2]);
        ShutterMOT7repump(off);
        ShutterMOT7principal(on);
      end else begin
        ShutterMOT7repump(on);
        ShutterMOT7principal(off);
        SetDopplerCoolRepumpFrequency(DopplerCoolStartFrequency[2]);
      end;
      IOUpdate;
      Title('Doppler Cool 2 Start Detuning');
      Wait;
      ShutterMOT7repump(on);
      ShutterMOT7principal(on);
    end;
    if ein=#27 then exit;
    SetCMOTDetuning;
    IOUpdate;
    Title('CMOT Detuning');
    Wait;
    if ein=#27 then exit;
    {SetMolassesDetuning;
    IOUpdate;
    Title('Molasse Detuning');
    Wait;
    if ein=#27 then exit;}
    SetMOTDetuning;
    IOUpdate;
    Title('MOT Detuning');
    Wait;
    if ein=#27 then exit;
    SetOpticalPumpingDetuning7Li(OpticalPumping7repumpDetuning);
    SetOpticalPumpingDetuning6Li(OpticalPumping6repumpDetuning);
    IOUpdate;
    Title('Optical Pumping Detuning');
    Wait;
  until ein=#27;
end;

procedure TestInjection(isotope:integer);
var os7pd,os7rd,ot7pd,ot7rd,
    os6pd,os6rd,ot6pd,ot6rd,n,a1,a2:double;
begin
  SelectIsotope(isotope);
  os7pd:=Slower7PrincipalDetuning;
  os7rd:=Slower7RepumpDetuning;
  ot7pd:=MOT7PrincipalDetuning;
  ot7rd:=MOT7RepumpDetuning;
  os6pd:=Slower6PrincipalDetuning;
  os6rd:=Slower6RepumpDetuning;
  ot6pd:=MOT6PrincipalDetuning;
  ot6rd:=MOT6RepumpDetuning;
  n:=0;
  a1:=15;
  a2:=30;
  repeat
    n:=n+0.05;
    MOT7PrincipalDetuning:=ot7pd+a1*sin(n);
    MOT7RepumpDetuning:=ot7rd+a1*sin(2*n);
    Slower7PrincipalDetuning:=os7pd+a2*sin(0.5*n);
    Slower7RepumpDetuning:=os7rd+a2*sin(3*n);
    MOT6PrincipalDetuning:=ot6pd+a1*sin(0.2*n);
    MOT6RepumpDetuning:=ot6rd+a1*sin(2.5*n);
    Slower6PrincipalDetuning:=os6pd+a2*sin(0.25*n);
    Slower6RepumpDetuning:=os6rd+a2*sin(1.5*n);
    TextBackground(blue);
    TextColor(Yellow);
    clrscr;
    writeln('*** Test Injection ***');
    writeln;
    writeln('7Li Slower Principal Detuning = ',Slower7PrincipalDetuning:5:2,' MHz');
    writeln('7Li Slower Repump Detuning    = ',Slower7RepumpDetuning:5:2,' MHz');
    writeln('7Li MOT Principal Detuning    = ',MOT7PrincipalDetuning:5:2,' MHz');
    writeln('7Li MOT Repump Detuning       = ',MOT7RepumpDetuning:5:2,' MHz');
    writeln('6Li Slower Principal Detuning = ',Slower6PrincipalDetuning:5:2,' MHz');
    writeln('6Li Slower Repump Detuning    = ',Slower6RepumpDetuning:5:2,' MHz');
    writeln('6Li MOT Principal Detuning    = ',MOT6PrincipalDetuning:5:2,' MHz');
    writeln('6Li MOT Repump Detuning       = ',MOT6RepumpDetuning:5:2,' MHz');
    SetAOMFrequencies;
    delay(1);
  until keypressed;
  MOT7PrincipalDetuning:=ot7pd;
  MOT7RepumpDetuning:=ot7rd;
  Slower7PrincipalDetuning:=os7pd;
  Slower7RepumpDetuning:=os7rd;
  MOT6PrincipalDetuning:=ot6pd;
  MOT6RepumpDetuning:=ot6rd;
  Slower6PrincipalDetuning:=os6pd;
  Slower6RepumpDetuning:=os6rd;
  SetAOMFrequencies;
end;

procedure SweepDetuning(det:TDet);
var n:double;
    ein:char;
    os7pd,os7rd,ot7pd,ot7rd,
    os6pd,os6rd,ot6pd,ot6rd:double;
    action:boolean;
begin
  os7pd:=Slower7PrincipalDetuning;
  os7rd:=Slower7RepumpDetuning;
  ot7pd:=MOT7PrincipalDetuning;
  ot7rd:=MOT7RepumpDetuning;
  os6pd:=Slower6PrincipalDetuning;
  os6rd:=Slower6RepumpDetuning;
  ot6pd:=MOT6PrincipalDetuning;
  ot6rd:=MOT6RepumpDetuning;
  n:=0;
  repeat
    TextBackground(blue);
    TextColor(Yellow);
    clrscr;
    writeln('Sweep frequencies with +/-');
    writeln;
    writeln('7Li Slower Principal Detuning = ',Slower7PrincipalDetuning:5:2,' MHz');
    writeln('7Li Slower Repump Detuning    = ',Slower7RepumpDetuning:5:2,' MHz');
    writeln('7Li MOT Principal Detuning    = ',MOT7PrincipalDetuning:5:2,' MHz');
    writeln('7Li MOT Repump Detuning       = ',MOT7RepumpDetuning:5:2,' MHz');
    writeln('6Li Slower Principal Detuning = ',Slower6PrincipalDetuning:5:2,' MHz');
    writeln('6Li Slower Repump Detuning    = ',Slower6RepumpDetuning:5:2,' MHz');
    writeln('6Li MOT Principal Detuning    = ',MOT6PrincipalDetuning:5:2,' MHz');
    writeln('6Li MOT Repump Detuning       = ',MOT6RepumpDetuning:5:2,' MHz');
    action:=true;
    repeat IdleProc(action);action:=false; until keypressed;
    while keypressed do ein:=readkey;
    if ein='-' then n:=n-0.5;
    if ein='+' then n:=n+0.5;
    if ein='/' then n:=n-2.5;
    if ein='*' then n:=n+2.5;
    if 1 in det then MOT7PrincipalDetuning:=ot7pd+n;
    if 2 in det then MOT7RepumpDetuning:=ot7rd+n;
    if 3 in det then Slower7PrincipalDetuning:=os7pd+n;
    if 4 in det then Slower7RepumpDetuning:=os7rd+n;
    if 5 in det then MOT6PrincipalDetuning:=ot6pd+n;
    if 6 in det then MOT6RepumpDetuning:=ot6rd+n;
    if 7 in det then Slower6PrincipalDetuning:=os6pd+n;
    if 8 in det then Slower6RepumpDetuning:=os6rd+n;
    SetAOMFrequencies;
  until ein=#27;
end;

procedure Blink;
var delay:double;
    m,n,i:word;
    ein:char;
begin
{  repeat
    InitPortList;
    StartSequence;
    SequenceStartPortListDuringWait(11*ID+20);
    for m:=1 to 3 do begin
      for n:=1 to 2 do begin
        SwitchOszilloskopTrigger2(on);
        IOUpdate;
        SwitchOszilloskopTrigger2(off);
        IOUpdate;
      end;
      Wait(12*ID+5+m);
    end;
    SequenceStopPortList;
    for n:=1 to 4 do begin
      SwitchOszilloskopTrigger2(on);
      Wait(13*ID+1);
      SwitchOszilloskopTrigger2(off);
      Wait(14*ID+1);
    end;
    StartCounter;
    StopSequence;
  until keypressed;
}

{  Delay:=500/Frequency;
  repeat
    SwitchIGBT(6,on);
    witchIGBT(4,on);
    SwitchIGBT(8,on);
    writeln;
    write('on ');
    IOUpdate;
    for n:=1 to 1 do begin
      Wait(15*ID+Delay);
      write('*');
    end;
    SwitchIGBT(6,off);
    SwitchIGBT(4,off);
    SwitchIGBT(8,off);
    n:=n+1;
    if n>8 then n:=1;
    writeln;
    write('off ');
    IOUpdate;
    for n:=1 to 1 do begin
      Wait(16*ID+Delay);
      write('*');
    end;
    while keypressed do ein:=readkey;
  until ein=#27;
end;
{
    IOUpdate;
    for n:=1 to 30 do Wait(17*ID+1000);
    SwitchIGBT(7,off);
    IOUpdate;
      Wait(18*ID+100);

    while keypressed do ein:=readkey;
  until ein=#27;
end;
}
  delay:=500/frequency;
  startCounter;
  repeat
    SwitchAOM20(off);
    SwitchIGBTsOff;
    Write('*');
    Wait(19*ID+delay);
    SwitchAOM20(on);
    SwitchIGBTsToMOT;
    SwitchIGBT(9,on);
    Wait(20*ID+delay);
  until keypressed;

  {startCounter;
  repeat
    SwitchOszilloskopTrigger2(on);
    IOUpdate;
    StartSequenceDuringWait(21*ID+200);
    crt.Delay(50);
    SwitchOszilloskopTrigger2(off);
    IOUpdate;
    for n:=1 to 3 do begin
      SwitchAOM20(off);
      Wait(22*ID+100);
      SwitchAOM20(on);
      Wait(23*ID+100);
    end;
    StopSequence;
  until keypressed;}

  while keypressed do readkey;
end;

procedure ChargeDetectTrap;
var ExposureTime:double;
begin
  case CameraTyp[CameraPosition] of
    MatroxCamera:if DetectIsotop=7 then ExposureTime:=Matrox7LiExposureTime else ExposureTime:=Matrox6LiExposureTime;
    AndorCamera:if DetectIsotop=7 then ExposureTime:=Andor7LiExposureTime else ExposureTime:=Andor6LiExposureTime;
  end;
  PrepareSinglePuls(0,0.5,ExposureTime,0.001,0);
  SwitchAOM20(on);
  StartCounter;
  repeat
    Wait(24*ID+ChargingTime);
    TriggerCamera;
    Wait(25*ID+ExposureTime);
  until keypressed;
end;

procedure ChargeRecaptureTrap;
begin
  repeat
    sound(1000);
    delay(100);
    nosound;
    SwitchAOM20(on);
    IOUpdate;
    delay(3000);
    sound(100);
    delay(100);
    nosound;
    delay(1000);
    StartCounter;
    SwitchAOM20(off);
    Wait(26*ID+0.1);
    SwitchAOM20(on);
    Wait(27*ID+1000);
  until keypressed;
end;

procedure CalibrateVCO;
var nr:word;
    name:string;
    n:word;
    out:text;
begin
  textBackground(blue);
  textcolor(yellow);
  clrscr;
  writeln('* AOM Calibration *');
  writeln;
  write('DAC number : ');
  readln(nr);
  write('Filename : ');
  read(name);
  assign(out,name);
  rewrite(out);
  n:=0;
  repeat
    IOanalogOutDirect(nr,n);
    delay(500);
    writeln(out,GetFrequency:5:0,' ',n:5);
    writeln(n:5,'  ',GetFrequency:5:0);
    n:=n+1000;
  until n>65000;
  system.close(out);
end;

procedure PrepareSRSforTakeAbsPicture;
var ExpansionTime:double;
begin
  if TwoPictures and (CameraTyp[CameraPosition]=AndorCamera) then begin
    PrepareSinglePuls(0,AndorStartExposureSecurityTime,Andor7LiExposureTime,0.001,0);
    Andor6LiExposureTime:=Andor7LiExposureTime;
    SRSWaitTimeBeforePulse:=AndorStartExposureSecurityTime;
    AktExposureTime:=Andor7LiExposureTime;
  end else begin
    case CameraTyp[CameraPosition] of
      MatroxCamera:if DetectIsotop=7 then AktExposureTime:=Matrox7LiExposureTime else AktExposureTime:=Matrox6LiExposureTime;
      AndorCamera:if DetectIsotop=7 then AktExposureTime:=Andor7LiExposureTime else AktExposureTime:=Andor6LiExposureTime;
    end;
    if DetectIsotop=7 then ExpansionTime:=ExpansionTime7
    else ExpansionTime:=ExpansionTime6;
    if (ExpansionTime+AktExposureTime<0.9) then begin
      PrepareSinglePuls(0,ExpansionTime,AktExposureTime,0.001,0);
      if (ExpansionTime<0.02) then writeln('Expansion Time too short!',^G);
      SRSWaitTimeBeforePulse:=ExpansionTime;
    end else begin
      if (0.2+AktExposureTime>0.9) then writeln('Exposure Time too long!',^G);
      PrepareSinglePuls(0,0.2,AktExposureTime,0.001,0);
      SRSWaitTimeBeforePulse:=0.2;
    end;
  end;
end;

procedure TriggerCamera;
begin
  MatroxCameraTrigger(on);
  AndorCameraTrigger(on);
  Wait(28*ID+1);
  MatroxCameraTrigger(off);
  AndorCameraTrigger(off);
  IOUpdate;
  Wait(29*ID+10);
  MatroxCameraTrigger(on);
  AndorCameraTrigger(on);
  Wait(30*ID+1);
  MatroxCameraTrigger(off);
  AndorCameraTrigger(off);
  IOUpdate;
end;

procedure MultiTriggerAndorCamera;
var i:byte;
begin
  wait(100);
  for i:=0 to 10 do begin
    AndorCameraTrigger(on);
    Wait(270*ID+1);
    AndorCameraTrigger(off);
    Wait(271*ID+10);
  end;
end;

procedure TriggerCameraAndExposure;
begin
  MatroxCameraTrigger(on);
  AndorCameraTrigger(on);
  ExposureTrigger(on);
  Wait(31*ID+1);
  MatroxCameraTrigger(off);
  AndorCameraTrigger(off);
  ExposureTrigger(off);
  IOUpdate;
end;

procedure TriggerCameraTwoPictures(Pic1,Pic2:boolean);
var TotalTime,ShortTime,LongTime,AndorTriggerTime:double;

  procedure ExposureSequence(FirstWait:double;first:boolean);
  begin
    if ExpansionTime6<ExpansionTime7 then begin
      SelectProbeBeam(Li6Probe);
      if DisinjectUnusedRepumper then begin
        SwitchMaster7repump(off);
        SwitchMaster6repump(on);
      end;
    end else begin
      SelectProbeBeam(Li7Probe);
      if DisinjectUnusedRepumper then begin
        SwitchMaster6repump(off);
        SwitchMaster7repump(on);
      end;
    end;
    SwitchRepumpAndOpticalPumpingEnable(ProbeWithRepump and (not PictureFirstWithoutThenWithRepump));
    TotalTime:=0;
    Wait(ID*261+FirstWait-AndorStartExposureSecurityTime);  {First ExpansionTime}
    TotalTime:=TotalTime+ShortTime-AndorStartExposureSecurityTime;

    AndorCameraTrigger(on);                          {Start Exposure Window}
    if Pic1 then ExposureTrigger(on);                {First Exposure}
    Wait(ID*262+AndorTriggerTime);
    TotalTime:=TotalTime+AndorTriggerTime;
    AndorCameraTrigger(off);
    if Pic1 then ExposureTrigger(off);

    if not TwoPicturesOfSameIsotop then begin
      if ExpansionTime6<ExpansionTime7 then begin
        SelectProbeBeam(Li7Probe);
        if DisinjectUnusedRepumper then begin
          SwitchMaster6repump(off);
          SwitchMaster7repump(on);
        end;
      end else begin
        SelectProbeBeam(Li6Probe);
        if DisinjectUnusedRepumper then begin
          SwitchMaster7repump(off);
          SwitchMaster6repump(on);
        end;
      end;
    end;
    SwitchRepumpAndOpticalPumpingEnable(ProbeWithRepump);
    SwitchMaster7Repump(ProbeWithRepump);
    SwitchMaster6Repump(ProbeWithRepump);

    if DoHideMagField and (
      ((HideMagFieldAfterPictureNr=3) and (not first)) or
      ((HideMagFieldAfterPictureNr=1) and first) )
    then begin
      SwitchIGBT(4,on);
      SwitchIGBT(5,on);
    end;

    Wait(ID*263+LongTime-TotalTime-AndorStartExposureSecurityTime); {Wait for Transfer and second TOF}

    AndorCameraTrigger(on);                          {Start Exposure Window}
    if Pic2 then ExposureTrigger(on);                {Second Exposure}
    Wait(ID*264+AndorTriggerTime);
    TotalTime:=TotalTime+AndorTriggerTime;
    AndorCameraTrigger(off);
    if Pic2 then ExposureTrigger(off);
  end;

begin
  AndorTriggerTime:=2*SRSWaitTimeBeforePulse+AktExposureTime;
  if ExpansionTime6<ExpansionTime7 then begin
    ShortTime:=ExpansionTime6;
    LongTime:=ExpansionTime7;
  end else begin
    ShortTime:=ExpansionTime7;
    LongTime:=ExpansionTime6;
  end;
  ExposureSequence(ShortTime,true);          {Two Images with atoms}
  if DoHideMagField and (HideMagFieldAfterPictureNr=2) then begin
    SwitchIGBT(4,on);
    SwitchIGBT(5,on);
  end;
  if DoHideDetuning then begin
    SetHideProbeDetuning(6);
    SetHideProbeDetuning(7);
  end;
  ExposureSequence(AtomCleaningTime-SRSWaitTimeBeforePulse-AktExposureTime,false);          {Two Images without atoms}
  IOUpdate;
  if DoHideMagField then begin
    Wait(1);
    SwitchIGBT(4,off);
    SwitchIGBT(5,off);
    Wait(1);
  end;
end;

procedure TriggerCameraAndExposureFKS(one,two:boolean);
begin
  MatroxCameraTrigger(on);
  AndorCameraTrigger(on);
  if one then ExposureTrigger(on);
  Wait(32*ID+1);
  MatroxCameraTrigger(off);
  AndorCameraTrigger(off);
  ExposureTrigger(off);
  Wait(33*ID+PictureSplittingDelay-SRSWaitTimeBeforePulse);
  MatroxCameraTrigger(on);
  AndorCameraTrigger(on);
  if two then ExposureTrigger(on);
  Wait(34*ID+1);
  MatroxCameraTrigger(off);
  AndorCameraTrigger(off);
  ExposureTrigger(off);
  IOUpdate;
end;

procedure TriggerExposure;
begin
  ExposureTrigger(on);
  Wait(35*ID+1);
  ExposureTrigger(off);
  IOUpdate;
end;

procedure PrepareShutters;
begin
  ShutterAndorProbe(on);
  ShutterAOM20(open);
  ShutterMaster7AndMOT7LiPrincipal(on);
  ShutterMOT7principal(TrapLithium7);
  ShutterMOT7repump(TrapLithium7);
  ShutterMOT6principal(TrapLithium6);
  ShutterMOT6repump(TrapLithium6);
  SwitchSlower7(SlowLithium7);
  SwitchSlower6(SlowLithium6);
  ShutterSlowerJet(open);
  ShutterSlower7principal(SlowLithium7);
  ShutterSlower7repump(SlowLithium7);
  ShutterSlower6principal(SlowLithium6);
  ShutterSlower6repump(SlowLithium6);
  ShutterProbe(open);
  ShutterProbeRepumpAndOpticalPumpingBeam(open);
  ShutterMOT(open);
  ShutterAOM20(open);
  SwitchMaster7Repump(on);
  SwitchMaster7Principal(on);
  SwitchMaster6Repump(on);
  SwitchMaster6Principal(on);
end;

const FirstPicture=1;
      SecondPicture=2;
      Finished=0;
const AktPicture:byte=Finished;

procedure CheckFluorescence;
var Timeout:longint;
    Iso:byte;
const Ad:array [6..7,0..5] of double=((2,0.3,2,0.5,1.5,1),
                                      (2,0.3,1,0.5,0.75,1));
begin
  if TrapLithium7 then Iso:=7 else Iso:=6;
  TimeOut:=0;
  MeasureFluorescence(0);
  while (GetFluorescence(0)*100+Ad[Iso,0]>FluoEndLevel) and (TimeOut<1000) do begin
    inc(TimeOut);
    SwitchAOM20(off);
    if GetFluorescence(0)<Ad[Iso,1] then Wait(36*ID+Ad[Iso,2])
    else if GetFluorescence(0)<Ad[Iso,3] then Wait(37*ID+Ad[Iso,4])
    else Wait(38*ID+Ad[Iso,5]);
    SwitchAOM20(on);
    Wait(39*ID+20);
    MeasureFluorescence(0);
  end;
  if (GetFluorescence(0)*100+Ad[Iso,0]>FluoEndLevel) then begin
    SwitchAOM20(off);
    SwitchIGBTsOff;
    Wait(40*ID+300);
    SwitchAOM20(on);
    SwitchIGBTsToMOT;
    IOUpdate;
  end;
end;

procedure Add6Li;
begin
  ShutterMOT6principal(on);
  ShutterMOT6repump(on);
  SwitchSlower6(on);
  ShutterSlower6principal(on);
  ShutterSlower6repump(on);
  SwitchMaster6Repump(on);
  SwitchMaster6Principal(on);
end;

procedure Added6LiCharging;
var n:word;
begin
  if Added6LiChargingTime>0 then begin
    Add6Li;
    n:=round(Added6LiChargingTime/100);
    for n:=1 to n do begin
      Wait(41*ID+100);
      if keypressed then exit;
    end;
    Wait(41*ID+Added6LiChargingTime-n*100);
  end;
end;

procedure DoDeltaFluoStopped6LiCharging;
var StartFluo,hfluo:double;
    timeout:word;
    n:word;
begin
  Add6Li;
  Wait(50);
  StartFluo:=0;
  for n:=1 to 300 do begin
    MeasureFluorescence(0);
    StartFluo:=StartFluo+GetFluorescence(0);
    Wait(42*ID+1);
  end;
  StartFluo:=StartFluo/3;
  timeout:=0;
  hfluo:=StartFluo;
  repeat
    MeasureFluorescence(0);
    Wait(78*ID+1);
    timeout:=timeout+1;
    gotoxy(50,2);
    hfluo:=hfluo*0.997+0.003*GetFluorescence(0)*100;
    write('TiMOTFluo=',hfluo:3:1,'% in ',timeout:5,' ms');
  until (hfluo<=(StartFluo-Fluo6LiEndLevel)) or (timeout>20000) or keypressed;
end;

procedure DoFluoStopped6LiCharging;
var timeout:longint;
    Li7Fluorescence,Li6Fluorescence:double;
    action:boolean;
begin
  Add6Li;
  timeout:=0;
  InitMeasureTwoIsotopeFlourescence;
  repeat
    MeasureTwoIsotopeFlourescence(Li7Fluorescence,Li6Fluorescence,1,50);
    Li7Fluorescence:=Li7Fluorescence*100;
    Li6Fluorescence:=Li6Fluorescence*100;
    inc(timeout);
    if (Fluo7LiEndLevel>0) and (Fluo7LiEndLevel<Li7Fluorescence) and (Li6Fluorescence>Fluo6LiEndLevel) then begin
      SwitchMaster7principal(off);
      delay(1);
      SwitchMaster7principal(on);
      IOUpdate;
      if (Li6Fluorescence>(Fluo6LiEndLevel+2)) then begin
        SwitchMaster6principal(off);
        delay(1);
        SwitchMaster6principal(on);
        IOUpdate;
      end;
    end;
  until ((Li6Fluorescence>Fluo6LiEndLevel) and
    (((Fluo7LiEndLevel>Li7Fluorescence))) or (Fluo7LiEndLevel>0)) or (timeout>1000000);
  End7LiFluorescence:=Li7Fluorescence/100;
  End6LiFluorescence:=Li6Fluorescence/100;
  if (timeout>1000000) then begin
    writeln;
    Writeln('6Li is not charging (press any key)');
    writeln;
    errorbeep;
    VisionMessage('6Li is not charging');
    action:=true;
    repeat IdleProc(action);action:=false; until keypressed;
    while keypressed do readkey;
  end;
end;

procedure RampSlowerField(onoff:boolean;ramptime:double;dechargetime:double);
begin
  if SlowerState<>onoff then begin
    if onoff then begin
      if dechargetime>0 then begin
        SwitchFinalSlowerVoltage(off);
        SetFinalSlowerCurrent(0);
        IOUpdate;
        wait(dechargetime);
      end;
      SetFinalSlowerCurrent(0);
      IOUpdate;
      SwitchFinalSlowerVoltage(on);
      SwitchSlowerField(onoff);
      Ramp(SetFinalSlowerCurrent,0,SlowerCurrent,ramptime,2);
    end else begin
      Ramp(SetFinalSlowerCurrent,SlowerCurrent,0,ramptime,2);
      SetFinalSlowerCurrent(0);
      if dechargetime>0 then begin
        SwitchFinalSlowerVoltage(off);
        wait(dechargetime);
      end;
      SwitchSlowerField(onoff);
    end;
  end else SwitchSlowerField(onoff);
end;

procedure PrepareTakeAbsPicture;
var n:word;
begin
  FinalTrap:='MOT';
  AtomPath:='M';
  if DoDebug then StartDebug(MainDir+DebugFileName,DebugAsOszilloskop,OriginOffset);
  AktPicture:=Finished;
  PrepareSRSforTakeAbsPicture;
  StartCounter;
  if VisionInSerie then begin
    if VisionSerieFirstPicture then VisionSerieFirstPicture:=false else begin
      if ((VisionCalculationTime*1000.0-ChargingTime-SecurityTime)>0) then begin
   {    writeln('Waiting for Vision =',VisionCalculationTime*1000.0-ChargingTime-SecurityTime:5:0);}
        Wait(42*ID+VisionCalculationTime*1000.0-ChargingTime-SecurityTime);
      end;
    end;
  end;
  for n:=0 to 3 do SynthTrigger(n,off);
  SwitchSternGerlachField(off);
  ShutterProbeRepumpAndOpticalPumpingBeam(open);
  SwitchProbeAndRepumpAOMHeating(ProbeAOMsDefaultOn);
  SwitchMOTDecharging(off);
  SwitchTransferDecharging(off);
  SwitchMOTGuidingField(off);
  SwitchIoffeGuidingField(off);
  SwitchRepumpAndOpticalPumpingEnable(on);
  IOUpdate;
  PrepareShutters;
  PrepareProbeBeam;
  SwitchOpticalPumpingBeam(off);
  SwitchSlower6(SlowLithium6);
  SwitchSlower7(SlowLithium7);
  SetAOM20Intensity(MOTIntensity);
  SetMOTCurrent(MOTMOTCurrent);
  SetTransfer1Voltage(0);
  IOUpdate;
  SetTransfer1Current(MaxTransfer1Current);
  SetTransfer2Voltage(MOTTransfer2Voltage);
  SetMOTVoltage(MOTMOTVoltage);
  SetMOTDetuning;
  SwitchFinalSlowerVoltage(on);
  RampSlowerField(on,100,10);
  InitMagTrap;
  SwitchIGBTsToMOT;
  SwitchAOM20Frequency(off);
  SwitchAOM20(on);
  Wait(43*ID+100);
  if DoSmoothSweep[1] then PrepareSmoothMicroWaveSweep(1);
  if not FluoStoppedCharging then begin
    if DeChargeMOT and (ChargingTime>SecurityTime) then begin
      SwitchAOM20(off);
      SwitchIGBTsOff;
      Wait(44*ID+300);
    end;
    SwitchIGBTsToMOT;
    SwitchAOM20(on);
    SwitchOszilloskopTrigger(on);
    if (ChargingTime-SecurityTime>0) then begin
  {    writeln('Waiting and charging =',ChargingTime-SecurityTime:5:0);}
      Wait(49*ID+ChargingTime-SecurityTime);
      Added6LiCharging;
    end;
  end else begin
    CheckFluorescence;
    SwitchOszilloskopTrigger(on);
  end;
end;

function TakeAbsPicture:boolean;forward;

function DoTakeAbsPicture(Calibration:double;PictureTyp:char):boolean;
var ein:char;
    ende:boolean;
    endecharging:boolean;
    ExpansionTime:double;
    n:double;
    i:word;
    m,x:byte;
    CMOT7prin,CMOT7rep,CMOT6prin,CMOT6rep:double;
    PictureWait:double;
    FluoSecurityTime:double;
    hfluo:double;
    timeout,timeout2:longint;
    cx,cy:byte;
    MatroxCameraWait:double;
    action:boolean;
    LastTimingError:double;
    helpoffset:double;
    SendFluo1,SendFluo2:double;
    UsedTimeOfFlight7,UsedTimeOfFlight6,UsedTimeOfFlight:double;

  procedure CheckError;
  var action:boolean;
  begin
    if not NoTemperatureOrWaterError then begin
      writeln;
      Writeln('Temperature or Water error! (press any key)');
      writeln;
      errorbeep;
      VisionMessage('Temperature or Water error!');
      action:=true;
      repeat IdleProc(action);action:=false; until keypressed;
      while keypressed do readkey;
      InitExp(false);
    end;
  end;

  procedure ShiftMOT;
  begin
    if MOTShiftTime>0 then begin
      SetTransfer2Voltage(0);
      SetTransfer1Voltage(0);
      IOUpdate;
      SwitchTransferDecharging(on);
      Wait(50*ID+MOTShiftTime);
      SetTransfer1Current(MaxTransfer1Current);
      SwitchIGBTsToMagTrap;
    end;
  end;

  procedure HeatupCycle;
  begin
    SwitchMOTDecharging(off);
    SwitchTransferDecharging(off);
    SetBarsCurrent(0);
    SetBarsVoltage(MaxBarVoltage);
    SetTransfer1Voltage(MaxTransfer1Voltage);
    SetTransfer2Voltage(0);
    SetTransferCurrent(0);
    SetMOTVoltage(0);
    SwitchIGBT(8,on);
    SwitchIGBT(2,on);
    SwitchIGBT(4,on);
    ChangeIoffeTrap(HeatupBarCurrent,HeatupPinchCurrent,
      HeatupPinchCurrent,0,1000);
    Wait(53*ID+HeatupTime);
    ChangeIoffeTrap(0,0,0,0,1000);
    SwitchIGBTsOff;
  end;

  procedure ReInitialize;
  var n:byte;
  begin
    StartCounter;
    Wait(51*ID+50);
    SwitchMagTrapSlowlyCompletelyOff;
    SetMOTCurrent(0);
    SetMOTVoltage(0);
    SetTransfer1Voltage(0);
    SetTransfer1Current(0);
    SetTransfer2Voltage(0);
    SetNdYAGIntensity(0,ScaleHorizontalNdYagIntensity(0));
    SetNdYAGIntensity(1,ScaleHorizontalNdYagIntensity(0));
    SwitchHorizontalNdYAG(off);
    SwitchVerticalNdYAG(off);
    SwitchPushbeam(off);
    Wait(52*ID+50);
    SwitchMOTDecharging(off);
    SwitchTransferDecharging(off);
    IOUpdate;
    SetBarsVoltage(0);
    SetBarsCurrent(0);
    IOUpdate;
    SwitchIGBT(8,on);
    PrepareShutters;
    PrepareProbeBeam;
    SwitchAOM20Frequency(off);
    SwitchAOM20(on);
    SwitchFinalSlowerVoltage(on);
    RampSlowerField(on,100,10);
    SetMOTDetuning;
    SetAOM20Intensity(MOTIntensity);
    SetMOTCurrent(MOTMOTCurrent);
    SetTransfer1Voltage(0);
    IOUpdate;
    SetTransfer2Voltage(MOTTransfer2Voltage);
    SetTransfer1Current(MaxTransfer1Current);
    SetMOTVoltage(MOTMOTVoltage);
    IOUpdate;
    SwitchIGBTsToMOT;
    SwitchMOTGuidingField(off);
    SwitchIoffeGuidingField(off);
    ShutterProbeRepumpAndOpticalPumpingBeam(open);
    ShutterMaster7AndMOT7LiPrincipal(on);
    SwitchProbeAndRepumpAOMHeating(ProbeAOMsDefaultOn);
    Set6LiProbeIntensity(Probe6MOTIntensity);
    Set7LiProbeIntensity(Probe7MOTIntensity);
    InitMagTrap;
    if RFOn(0) then SwitchRFOff(0);
    if RFOn(1) then begin SwitchRFOff(1);SwitchRF(1,off); end;
    if RFOn(2) then begin SwitchRFOff(2);SwitchRF(3,off); end;
    Wait(54*ID+102);
    UsedFeshbachField:=NotInitialised;
  end;

  procedure SaveSystemMenu;
  var action:boolean;
  begin
    while keypressed do ein:=readkey;
    if ein=' ' then begin
      repeat
        Writeln('*** Pause ***');
        writeln;
        writeln('6) Test Injection 6Li');
        writeln('7) Test Injection 7Li');
        writeln('N) Test Number');
        writeln('R) Repeat Measurement');
        writeln('I) Initialize');
        writeln('M) Main Menu');
        writeln;
        writeln('C) Continue');
        writeln;
        writeln('ESC) End of measurement');
        writeln('Enter) End of charging');
        action:=true;
        repeat IdleProc(action);action:=false; until keypressed;
        while keypressed do ein:=upcase(readkey);
        if ein='6' then TestExpInjection(6);
        if ein='7' then TestExpInjection(7);
        if ein='N' then TestNumber;
        if ein='I' then InitExp(false);
        if ein='M' then begin
          MainMenu;
          textBackground(blue);
          textcolor(yellow);
          clrscr;
        end;
        while keypressed do ein:=upcase(readkey);
      until (ein='C') or (ein='R') or (ein=#27);
      if ein='R' then TakeAbsPicture
      else if ein=#27 then ende:=true
      else if ein=#13 then endecharging:=true;
    end else if ein=#27 then ende:=true
    else if ein=#13 then endecharging:=true;
  end;

  procedure Molasses;
  begin
    SetAOM20Intensity(MolassesIntensity);
    if MolassesIntensity=0 then SwitchAOM20(off);
    if not MolassesWithMOTField then begin
      SwitchIGBTsOff;
      SwitchAOM20(off);
      Wait(55*ID+MolassesFieldOffTime);
      SwitchAOM20(on);
    end;
    SetMolassesDetuning;
    if MolassesWithGuidingField then SwitchOpticalPumpingField(on);
    Wait(56*ID+MolassesTime);
  end;

  procedure HyperfinePumping;
  begin
    if ((Repump7LiOffTime<>0) or (Repump6LiOffTime<>0)) then begin
      SetAOM20Intensity(HyperfinePumpingIntensity);
      if not HyperfinePumpingWithMOTField then SwitchIGBTsOff;
    end;
    if (Repump7LiOffTime<0) then begin
      DisableMOT7Principal;
      Set7RepumpDetuning(HyperfinePumping7LiRepumpDetuning);
      Wait(57*ID+-Repump7LiOffTime);
    end else if (Repump7LiOffTime>0) then begin
      DisableMOT7Repump;
      Set7PrincipalDetuning(HyperfinePumping7LiPrincipalDetuning);
      Wait(58*ID+Repump7LiOffTime);
    end;
    if (Repump6LiOffTime<0) then begin
      DisableMOT6Principal;
      Set6RepumpDetuning(HyperfinePumping6LiRepumpDetuning);
      Wait(59*ID+-Repump6LiOffTime);
    end else if (Repump6LiOffTime>0) then begin
      DisableMOT6Repump;
      Set6PrincipalDetuning(HyperfinePumping6LiPrincipalDetuning);
      Wait(60*ID+Repump6LiOffTime);
    end;
  end;

begin
  if DetectIsotop=7 then ExpansionTime:=ExpansionTime7
  else ExpansionTime:=ExpansionTime6;
  ende:=false;
  endecharging:=false;
  NormPictureCounter:=NormPictureCounter+1;
  InitPortList;
  StartWaitSum;
  {SwitchOszilloskopTrigger3(on);}
  IOUpdate;
  if FluoStoppedCharging then StartSequenceDuringWait(61*ID+334)
  else begin
    StartSequenceDuringWait(62*ID+SecurityTime);
    if (ChargingTime<SecurityTime) then begin
      if DeChargeMOT then begin
        SwitchAOM20(off);
        SwitchIGBTsOff;
        if SecurityTime-Chargingtime<300
        then Wait(63*ID+300-(SecurityTime-Chargingtime));
      end;
      if (SecurityTime-Chargingtime>0)
      then Wait(64*ID+SecurityTime-Chargingtime);
      SwitchIGBTsToMOT;
      SwitchAOM20(on);
      SwitchOszilloskopTrigger(on);
      Wait(65*ID+ChargingTime);
      Added6LiCharging;
    end;
  end;
  SynchronizeToLine;
  MeasureFluorescence(0);
  ShutterSlower7principal(off);
  ShutterSlower7repump(off);
  ShutterSlower6principal(off);
  ShutterSlower6repump(off);
  Wait(66*ID+20);
  RampSlowerField(off,100,10);
  SwitchSlower7(off);
  SwitchSlower6(off);
  Wait(67*ID+10);
  SwitchSlowerCompensation(off);
  FinalPosition:=LowerTrap;
  if DoShiftMOThorizontal then begin
    SwitchSlowerCompensation(on);
    SwitchFinalSlowerVoltage(on);
    SetFinalSlowerCurrent(ShiftMOThorizontalCurrent);
    FinalTrap:='MOT shifted horizontally';
    AtomPath:=AtomPath+'Sh';
    Wait(69*ID+100);
  end;
  if DoMagTrap then ShutterSlowerJet(close);
  Wait(70*ID+SlowerOffTime);   {Waiting that Slower field is really off}
  if DoOpticalPumping then PrepareOpticalPumping;
  ShutterProbeRepumpAndOpticalPumpingBeam(open);
  SequenceStartPortListDuringWait(71*ID+49);
  {SwitchOszilloskopTrigger(off);
  Wait(72*ID+1);}
  if DoShiftMOT then begin
    ShiftMOT;
    FinalTrap:='shifted MOT';
    AtomPath:=AtomPath+'S';
  end;
  if (DeChargingTime<>0) then Wait(73*ID+DeChargingTime);
  if DoCMOT then begin
    FinalTrap:='CMOT';
    AtomPath:=AtomPath+'C';
    SetAOM20Intensity(CMOTIntensity);
    SetMOTVoltage(CMOTMOTVoltage);
    if (CMOTMOTVoltage=0) then SwitchIGBTsOff;
    SetCMOTDetuning;
    Wait(74*ID+CMOTTime);
  end;
  if DoMolasses then begin
    Molasses;
    FinalTrap:='Molasses';
    AtomPath:=AtomPath+'Mo';
  end;
  if DoHyperfinePumping then begin
    HyperfinePumping;
    FinalTrap:='Hyperfine Pumping';
    AtomPath:=AtomPath+'H';
  end;
  if DoMagTrap then MagneticTrap
  else if DoOpticalPumping then begin
    OpticalPumping;
    SwitchOpticalPumpingField(off);
    IOUpdate;
    FinalTrap:='optical Pumping';
    AtomPath:=AtomPath+'O';
  end;
  {SwitchOszilloskopTrigger(off);}
  {SwitchMOTGuidingField(MOTGuidingFieldOn);
  SwitchIoffeGuidingField(IoffeGuidingFieldOn);}
  SetProbeDetuning(DetectIsotop);
  if not (InjectProbeRepumpLast and PictureFirstWithoutThenWithRepump) then begin
    SwitchMaster7Repump(ProbeWithRepump);
    SwitchMaster6Repump(ProbeWithRepump);
  end;
  SwitchRepumpAndOpticalPumpingEnable(ProbeWithRepump);
  EnableMOT7Principal;
  EnableMOT6Principal;
  IOUpdate;
  LastMillisecondExperiments;
   UsedTimeOfFlight:=0;
  {Between here and TriggerCameraAndExposure no IOUpdate command allowed!}
  if not PictureWithTrapOn then begin
    AtomPath:=AtomPath+'To';
    SwitchAOM20(Off);
    if SwitchOpticalTrapOffFirst then begin
      if SwitchOffOnlyVerticalBeam then begin
        SwitchVerticalNdYAG(off);
      end else begin
        SwitchHorizontalNdYAG(off);
        SwitchVerticalNdYAG(off);
        SetNdYagIntensity(0,ScaleHorizontalNdYagIntensity(0));
        SetNdYagIntensity(0,ScaleHorizontalNdYagIntensity(1));
      end;
      if (DelayOpticalTrapOffTime>0) then begin
        wait(400*ID+DelayOpticalTrapOffTime);
        UsedTimeOfFlight:=UsedTimeOfFlight+DelayOpticalTrapOffTime;
      end;
    end;
    SwitchTrapOff;
    {SwitchOszilloskopTrigger(off);}
    if (AxialConfinementOffTime>0) and PictureWithRadialConfinementOn then begin
      Wait(75*ID+AxialConfinementOffTime);
      UsedTimeOfFlight:=UsedTimeOfFlight+AxialConfinementOffTime;
      SwitchIGBT(8,off);
    end;
  end;
  if (not PictureWithOpticalTrapOn) and (not SwitchOpticalTrapOffFirst) then begin
    if (DelayOpticalTrapOffTime>0) then begin
      wait(401*ID+DelayOpticalTrapOffTime);
      UsedTimeOfFlight:=UsedTimeOfFlight+DelayOpticalTrapOffTime;
    end;
    SwitchHorizontalNdYAG(off);
    SwitchVerticalNdYAG(off);
    SetNdYagIntensity(0,ScaleHorizontalNdYagIntensity(0));
    SetNdYagIntensity(0,ScaleHorizontalNdYagIntensity(1));
    IOUpdate;
  end;
  if SternGerlachTime>0 then begin
    SwitchSternGerlachField(on);
    wait(75*ID+SternGerlachTime);
    UsedTimeOfFlight:=UsedTimeOfFlight+SternGerlachTime;
    SwitchSternGerlachField(off);
    IOUpdate;
  end;
  if FreeFlightTime>0 then begin
    wait(381*ID+FreeFlightTime);
    UsedTimeOfFlight:=UsedTimeOfFlight+FreeFlightTime;
  end;
  if DoRepumpBeforeProbe then begin
     SwitchMaster7Repump(on);     {injection}
     SwitchMaster6Repump(on);
     SwitchOpticalPumpingBeam(on);
  end;
  if (not (TwoPictures and (CameraTyp[CameraPosition]=AndorCamera))) and
    (ExpansionTime+AktExposureTime>=0.9) then begin
      Wait(76*ID+ExpansionTime-SRSWaitTimeBeforePulse);
      UsedTimeOfFlight:=UsedTimeOfFlight+ExpansionTime-SRSWaitTimeBeforePulse;
  end;
  PictureWait:=GetTotalWait;
  {writeln('TotalWait:',GetTotalWait:2:3,' SequenceTotalWait:',GetSequenceTime:2:3);}
  SwitchMOTGuidingField(MOTGuidingFieldOn);
  SwitchIoffeGuidingField(IoffeGuidingFieldOn);
  if (PictureSplitting and (CameraTyp[CameraPosition]=AndorCamera)) then begin
    if TwoPictures then begin
      TriggerCameraTwoPictures(true,true);
      UsedTimeOfFlight:=UsedTimeOfFlight+ExpansionTime;
    end else TriggerCameraAndExposureFKS(true,true);
  end else TriggerCameraAndExposure;
  if CameraTyp[CameraPosition]=AndorCamera then begin
    ShutterProbe(close);
    ShutterProbeRepumpAndOpticalPumpingBeam(close);
  end;
  if ProbeAOMsDefaultOn then begin
    SwitchProbeAndRepumpAOMHeating(off);
    ShutterProbe(open);
    ShutterProbeRepumpAndOpticalPumpingBeam(open);
    IOUpdate;
  end;
  SwitchHorizontalNdYAG(off);
  SwitchVerticalNdYAG(off);
  Wait(77*ID+20);
  SwitchMOTGuidingField(off);
  SwitchIoffeGuidingField(off);
  SwitchSternGerlachField(off);
  {IOUpdate;
  SwitchOszilloskopTrigger(off);}
  SequenceStopPortList;
  if ProbeAOMsDefaultOn then begin
    ShutterProbe(close);
    ShutterProbeRepumpAndOpticalPumpingBeam(close);
    SwitchProbeAndRepumpAOMHeating(on);
    IOUpdate;
  end;
  FluoSecurityTime:=0;
  if FluoStoppedCharging then begin
    PauseSequence;
    PauseDebug;
    StartCounter;
    CheckFluorescence;
    timeout:=0;
    timeout2:=0;
    cx:=wherex;
    cy:=wherey;
    CursorOff;
    MeasureFluorescence(0);
    hfluo:=GetFluorescence(0)*100;
    repeat
      MeasureFluorescence(0);
      Wait(78*ID+1);
      CheckError;
      timeout:=timeout+1;
      gotoxy(55,1);
      hfluo:=hfluo*0.99+0.01*GetFluorescence(0)*100;
      write('Fluo=',hfluo:3:1,'% in ',timeout:5,' ms');
      if AkkusticFluorescence then sound(round(10*System.Exp(GetFluorescence(0)*ln(2000))))
      else nosound;
      if keypressed then begin
        gotoxy(cx,cy);
        CursorOn;
        SaveSystemMenu;
        if ende then begin
          DoTakeAbsPicture:=true;
          exit;
        end;
        cx:=wherex;
        cy:=wherey;
        CursorOff;
        StartCounter;
      end else begin
        ende:=false;
        endecharging:=false;
      end;
      if (timeout>1000000) then begin
        InitExp(false);
        VisionMessage('System reinitialised');
        inc(timeout2);
        timeout:=0;
        StartCounter;
      end;
    until (hfluo>=FluoEndLevel) or (timeout2>5) or ende or endecharging;
    if FluoStopped6LiCharging then DoFluoStopped6LiCharging;
    if DeltaFluoStopped6LiCharging then DoDeltaFluoStopped6LiCharging;
    if (timeout2>5) then begin
      writeln;
      Writeln('MOT is not charging (press any key)');
      writeln;
      errorbeep;
      VisionMessage('MOT is not charging');
      action:=true;
      repeat IdleProc(action);action:=false; until keypressed;
      while keypressed do readkey;
    end;
    gotoxy(cx,cy);
    CursorOn;
    if keypressed then begin
      DoTakeAbsPicture:=true;
      exit;
    end;
    LoadingTime:=timeout;
    Added6LiCharging;
    if keypressed then begin
      while keypressed do readkey;
      DoTakeAbsPicture:=true;
      exit;
    end;
    UnPauseSequence;
    UnPauseDebug;
    StartCounter;
    if PictureWait<SecurityTime then begin
      FluoSecurityTime:=SecurityTime-PictureWait;
      PictureWait:=SecurityTime;
    end;
  end else begin
    LoadingTime:=0;
    if PictureWait<SecurityTime then begin
      writeln('Error : DoTakeAbsPicture : PictureWait too small');
      delay(1000);
      exit;
    end;
  end;
  {writeln('TimeSinceLastWait:',GetTimeSinceLastWait:2:3, ' Total=',
     PictureWait-AndorTriggerDelay-GetTimeSinceLastWait-AndorTriggerCalibrationAbsorption:2:3);}
  if ExpansionTime7<ExpansionTime6 then begin
    UsedTimeOfFlight7:=UsedTimeOfFlight;
    UsedTimeOfFlight6:=UsedTimeOfFlight+ExpansionTime6-ExpansionTime7;
  end else begin
    UsedTimeOfFlight7:=UsedTimeOfFlight+ExpansionTime7-ExpansionTime6;
    UsedTimeOfFlight6:=UsedTimeOfFlight;
  end;
  if not VisionTakeAbsorptionPicture(PictureTyp,TwoPictures and (CameraTyp[CameraPosition]=AndorCamera),UsedTimeOfFlight6,
    UsedTimeOfFlight7,NoisePictureMode,Calibration,
    PictureWait-AndorTriggerDelay-GetTimeSinceLastWait-AndorTriggerCalibrationAbsorption,0,
    AtomPath+' '+FinalTrap)
  then begin
    Writeln('Exp : DoTakeAbsPicture : Vision not ready');
    delay(1000);
    exit;
  end;
  case CameraTyp[CameraPosition] of
    AndorCamera:if not VisionReady then writeln('DoTakeAbsPicture : Vision not Ready for Absorption picture');
  end;
  if FluoSecurityTime>0 then begin
    PauseSequence;
    Wait(79*ID+FluoSecurityTime);
    UnPauseSequence;
  end;
  PauseSequence;
    PrepareProbeBeam;
  UnPauseSequence;
  StopSequence;
  SwitchAOM20(Off);
  Wait(80*ID+10);
  LastTimingError:=GetTimingError;
  MatroxCameraWait:=MatroxWait-60;
  if CameraTyp[CameraPosition]=AndorCamera then begin
    SwitchMagTrapSlowlyCompletelyOff;
    MultiTriggerAndorCamera;
  end else SwitchMagTrapCompletelyOff; {Takes 50ms}
  if (NoisePictureMode=1) or (NoisePictureMode=2) or (CameraTyp[CameraPosition]=MatroxCamera) then begin
    case CameraTyp[CameraPosition] of
      MatroxCamera:begin
        if ProbeAOMsDefaultOn then begin
          StartPortListDuringWait(81*ID+MatroxCameraWait-ShutterProbeAndRepumpDelay-ProbeAOMOffTimeBeforeShutterOpen);
          SwitchProbeAndRepumpAOMHeating(off);
          Wait(82*ID+ProbeAOMOffTimeBeforeShutterOpen);
          ShutterProbe(open);
          ShutterProbeRepumpAndOpticalPumpingBeam(open);
          Wait(83*ID+ShutterProbeAndRepumpDelay);
        end else Wait(84*ID+MatroxCameraWait);
        TriggerCameraAndExposure;
        StartPortListDuringWait(85*ID+MatroxWait);
      end;
      AndorCamera:begin
        if KeepMOTAOMOn then begin
          ShutterAOM20(Off);
          Wait(86*ID+MOTShutterDelay);
          SwitchAOM20(On);
        end;
        if not VisionReady then writeln('DoTakeAbsPicture : Vision not Ready for Probe picture');
        StartCounter;
        ShutterProbe(open);
        ShutterProbeRepumpAndOpticalPumpingBeam(ProbeWithRepump);
        if ProbeAOMsDefaultOn then begin
          StartPortListDuringWait(87*ID+AndorTriggerCalibrationProbeNoise+AndorTriggerDelay-
            ShutterProbeAndRepumpDelay-ProbeAOMOffTimeBeforeShutterOpen);
          SwitchProbeAndRepumpAOMHeating(off);
          Wait(88*ID+ProbeAOMOffTimeBeforeShutterOpen);
          ShutterProbe(open);
          ShutterProbeRepumpAndOpticalPumpingBeam(open);
          Wait(89*ID+ShutterProbeAndRepumpDelay);
        end else begin
          StartPortListDuringWait(90*ID+AndorTriggerCalibrationProbeNoise+AndorTriggerDelay);
        end;
        if PictureSplitting then begin
          if TwoPictures then TriggerCameraTwoPictures(true,false)
          else TriggerCameraAndExposureFKS(true,false);
        end else TriggerCameraAndExposure;
        ShutterProbe(close);
        ShutterProbeRepumpAndOpticalPumpingBeam(close);
        if ProbeAOMsDefaultOn then begin
          Wait(91*ID+20);
          ShutterProbe(close);
          ShutterProbeRepumpAndOpticalPumpingBeam(close);
          Wait(92*ID+ShutterProbeAndRepumpDelay);
          SwitchProbeAndRepumpAOMHeating(on);
          IOUpdate;
        end;
        MultiTriggerAndorCamera;
        StopPortList;
        if not VisionReady then writeln('DoTakeAbsPicture : Vision not Ready for Noise picture');
        StartCounter;
        ShutterProbe(open);
        ShutterProbeRepumpAndOpticalPumpingBeam(ProbeWithRepump);
        if ProbeAOMsDefaultOn then begin
          StartPortListDuringWait(93*ID+AndorTriggerCalibrationProbeNoise+AndorTriggerDelay-
            ShutterProbeAndRepumpDelay-ProbeAOMOffTimeBeforeShutterOpen);
          SwitchProbeAndRepumpAOMHeating(off);
          Wait(94*ID+ProbeAOMOffTimeBeforeShutterOpen);
          ShutterProbe(open);
          ShutterProbeRepumpAndOpticalPumpingBeam(open);
          Wait(95*ID+ShutterProbeAndRepumpDelay);
        end else begin
          StartPortListDuringWait(96*ID+AndorTriggerCalibrationProbeNoise+AndorTriggerDelay);
        end;
      end;
    end;
    if (PictureSplitting and (CameraTyp[CameraPosition]=AndorCamera)) then begin
      if TwoPictures then TriggerCameraTwoPictures(false,true)
      else TriggerCameraAndExposureFKS(false,true);
    end else TriggerCamera;
    if CameraTyp[CameraPosition]=AndorCamera then begin
      ShutterProbe(close);
      ShutterProbeRepumpAndOpticalPumpingBeam(close);
    end;
    if ProbeAOMsDefaultOn then begin
      Wait(97*ID+20);
      ShutterProbe(close);
      ShutterProbeRepumpAndOpticalPumpingBeam(close);
      Wait(98*ID+ShutterProbeAndRepumpDelay);
      SwitchProbeAndRepumpAOMHeating(on);
      IOUpdate;
    end;
    if CameraTyp[CameraPosition]=AndorCamera then MultiTriggerAndorCamera;
    StopPortList;
  end;
  if CameraTyp[CameraPosition]<>AndorCamera then ReInitialize;
  if DoDebug then StopDebug;
  if keypressed then SaveSystemMenu;
  if not VisionPictureOk and not ende then TakeAbsPicture;
  if CameraTyp[CameraPosition]=AndorCamera then ReInitialize;
  if DoHideMagField then FinalTrapMOTcurrent:=0;
  if MeasuredOffset<>0 then begin
    helpoffset:=MeasuredOffset;
  end else helpoffset:=(FinalTrapTransferCurrent+FinalTrapMOTCurrent-
    BiasFieldCurrentOffset+FinalTrapPinchOffsetCurrent)*PinchOffset-
     FinalTrapTransferCurrent*CompensationOffset;
  if FluoStopped6LiCharging then begin
    SendFluo1:=End7LiFluorescence;
    SendFluo2:=End6LiFluorescence;
  end else begin
    SendFluo1:=GetFluorescence(0);
    SendFluo2:=GetFluorescence(1);
  end;
  if FinalTrapIsOpticalTrap then begin
    helpoffset:=1;
    FinalTrapBarCurrent:=100;
    FinalTrapTransferCurrent:=2;
    FinalTrapMOTcurrent:=1;
    FinalTrapPinchOffsetCurrent:=1;
  end;
  FinalTrapIsOpticalTrap:=FALSE;
  VisionSendPictureData(SendFluo1,SendFluo2,FinalTrapType,FinalTrapBarCurrent,
    FinalTrapTransfercurrent,FinalTrapMOTcurrent,helpoffset,
    FinalTrapBarCurrent*BarGradiant,
    (FinalTrapTransferCurrent+FinalTrapMOTcurrent+FinalTrapPinchOffsetCurrent)*PinchCurbature,
    LoadingTime,LastTimingError,Measured7LiOffsetFrequency,LastFrequency[0]);
  AktPicture:=Finished;
  DoTakeAbsPicture:=ende;
  if DoHeatupCycle then begin
    HeatupCycle;
    ReInitialize;
  end;
end;

function TakeAbsPicture:boolean;
begin
  PrepareTakeAbsPicture;
  TakeAbsPicture:=DoTakeAbsPicture(1,'-');
end;

procedure TakeProbePicture;
begin
  Title('Probe Picture');
  PrepareSRSforTakeAbsPicture;
  SwitchIGBTsOff;
  SwitchAOM20(off);
  RampSlowerField(off,100,10);
  SetProbeDetuning(DetectIsotop);
  IOUpdate;
  while not keypressed do if VisionTakeProbePicture then begin
    StartCounter;
    TriggerCameraAndExposure;
    write('*');
  end;
  SwitchAOM20(on);
  SetFinalSlowerCurrent(SlowerCurrent);
  SwitchFinalSlowerVoltage(on);
  RampSlowerField(on,100,10);
  SwitchIGBTsToMOT;
  SetMOTDetuning;
  SetAOM20Intensity(MOTIntensity);
  SetMOTCurrent(MOTMOTCurrent);
  SetTransfer1Voltage(0);
  IOUpdate;
  SetTransfer1Current(MaxTransfer1Current);
  SetTransfer2Voltage(MOTTransfer2Voltage);
  SetSlowerDetuning;
  IOUpdate;
end;

function CameraReady:boolean;
begin
  if CameraPosition=2 then begin

  end else begin
    case CameraTyp[CameraPosition] of
      MatroxCamera:SetMatroxCameraParameters;
      AndorCamera:SetAndorCameraParameters;
    end;
  end;
  if not VisionCameraReady(CameraTyp[CameraPosition])
  then begin
    writeln('Camera not ready!');
    delay(1000);
    CameraReady:=false;
  end else CameraReady:=true;
end;

procedure TakeAbsorptionPicture;
var n:integer;
begin
  BlackTitle('Absorption Picture');
  if not CameraReady then exit;
  SetAOMFrequencies;
  VisionInSerie:=true;
  VisionSerieFirstPicture:=true;
  if NumberOfPictures>1 then write('*');
  TakeAbsPicture;
  For n:=2 to NumberOfPictures do begin
    write('*');
    PrepareTakeAbsPicture;
    if DoTakeAbsPicture(1,'-') then begin
      exit;
      VisionInSerie:=false;
    end;
  end;
  VisionInSerie:=false;
end;

procedure FilmAbsorptionPicture;
begin
  repeat TakeAbsorptionPicture; until keypressed;
end;

procedure TestNumber;
var oldExpTime7,oldExpTime6,oldCMOTTime:double;
    oldP7P,oldP7R,oldP6P,oldP6R:double;
begin
  if not CameraReady then exit;
  oldCMOTTime:=CMOTTime;
  oldP7P:=Probe7PrincipalDetuning;
  oldP7R:=Probe7RepumpDetuning;
  oldP6P:=Probe6PrincipalDetuning;
  oldP6R:=Probe6RepumpDetuning;
  oldExpTime7:=ExpansionTime7;
  oldExpTime6:=ExpansionTime6;
  Probe7PrincipalDetuning:=0;
  Probe7RepumpDetuning:=0;
  Probe6PrincipalDetuning:=0;
  Probe6RepumpDetuning:=0;
  CMOTTime:=0;
  ExpansionTime7:=5.5;
  ExpansionTime6:=2;
  VisionCheckNumber;
  TakeAbsPicture;
  CMOTTime:=oldCMOTTime;
  ExpansionTime7:=oldExpTime7;
  ExpansionTime6:=oldExpTime6;
  Probe7PrincipalDetuning:=oldP7P;
  Probe7RepumpDetuning:=oldP7R;
  Probe6PrincipalDetuning:=oldP6P;
  Probe6RepumpDetuning:=oldP6R;
end;

procedure TakeNormPicture;
var odetp,odetr,oint,lastDetP,lastDetR,lastInt,IntNr,
	 oldExp,oldProbe,oldCMOTtime:double;
	 n,m,o,p,q,h,totalNr,aktNr:integer;
	 ein:char;
	 mix:PMix;

  procedure Reset;
  begin
    if DetectIsotop=7 then begin
      CMOT7PrincipalDetuning:=oDetP;
      CMOT7RepumpDetuning:=oDetR;
      Probe7PrincipalDetuning:=oldProbe;
      ExpansionTime7:=oldExp;
    end else begin
      CMOT6PrincipalDetuning:=oDetP;
      CMOT6RepumpDetuning:=oDetR;
      Probe6PrincipalDetuning:=oldProbe;
      ExpansionTime6:=oldExp;
    end;
    CMOTtime:=oldCMOTtime;
  end;

begin
  if NormPictureCounter>NormPictureCounterMax then begin
    oldCMOTtime:=CMOTtime;
    CMOTtime:=0;
    if DetectIsotop=7 then begin
      oldProbe:=Probe7PrincipalDetuning;
      oldExp:=ExpansionTime7;
      oDetP:=CMOT7PrincipalDetuning;
      oDetR:=CMOT7RepumpDetuning;
    end else begin
      oldProbe:=Probe6PrincipalDetuning;
      oldExp:=ExpansionTime6;
      oDetP:=CMOT6PrincipalDetuning;
      oDetR:=CMOT6RepumpDetuning;
    end;
    NormPictureCounter:=0;
    if DetectIsotop=7 then begin
      Probe7PrincipalDetuning:=0;
      ExpansionTime7:=1;
    end else begin
      Probe6PrincipalDetuning:=0;
      ExpansionTime6:=1;
    end;
    writeln('Taking Norm Picture');
    if not CameraReady then begin Reset;exit; end;
    PrepareTakeAbsPicture;
    if DoTakeAbsPicture(1,'C') then begin
      Reset;
      exit;
    end;
{    VisionRegisterInformation1(NormPicNumber);
    NormPicNumber:=NormPicNumber+1;}
    if DetectIsotop=7 then begin
      Probe7PrincipalDetuning:=oldProbe;
      ExpansionTime7:=oldExp;
    end else begin
      Probe6PrincipalDetuning:=oldProbe;
      ExpansionTime6:=oldExp;
    end;
    Reset;
  end;
end;

procedure Queue(Mess:string);
var DirInfo: SearchRec;
    QueuePath:string;
    D: DirStr;
    N: NameStr;
    E: ExtStr;
    err:integer;
    akt,nr:byte;
    name:string;
    out:text;
begin
  Title('Add Measurement Queue');
  writeln('Adding ',mess);
  if remote then QueuePath:=MeasureQueueRemoteDirectory
  else QueuePath:=MeasureQueueDirectory;
  FindFirst(QueuePath+'\*.*', Directory, DirInfo);
  akt:=0;
  while DosError = 0 do begin
    FSplit(DirInfo.Name, D, N, E);
    val(N,nr,err);
    if err=0 then begin
      akt:=nr;
      name:=n;
    end;
    FindNext(DirInfo);
  end;
  inc(akt);
  str(akt,name);
  mkdir(QueuePath+'\'+name);
  SaveParameters(QueuePath+'\'+name+'\');
  assign(out,QueuePath+'\'+name+'\MESS.dat');
  rewrite(out);
  writeln(out,Mess);
  system.close(out);
end;

procedure QueueUniMess(mess:PUniMess);
begin
  Queue('UniMess:'+mess^.name^);
end;

procedure QueueTakeAbsorptionPicture;
begin
  Queue('TakeAbsorptionPicture');
end;

procedure ExecuteQueue(EraseQueue:boolean);
var DirInfo,MessDirInfo: SearchRec;
    QueuePath:string;
    inp:text;
    mess:string;
    D: DirStr;
    N: NameStr;
    E: ExtStr;
    oldremote:boolean;
    old:file;
begin
  Title('Executing Measurement Queue');
  oldremote:=remote;
  {writeln('MemAvail:',MemAvail,' MaxAvail:',MaxAvail);}
  writeln('Saving Parameters...');
  SaveParameters(MainDir+'Save\');
  if remote then QueuePath:=MeasureQueueRemoteDirectory
  else QueuePath:=MeasureQueueDirectory;
  FindFirst(QueuePath+'\*.*', Directory, DirInfo);
  while (DosError = 0) and (not keypressed) do begin
    if (DirInfo.Name<>'.') and (DirInfo.Name<>'..') and (DirInfo.Attr=Directory) then begin
      assign(inp,QueuePath+'\'+DirInfo.Name+'\MESS.dat');
      reset(inp);
      readln(inp,mess);
      system.close(inp);
      writeln('Loading parameters for ',mess);
      LoadParameters(QueuePath+'\'+DirInfo.Name+'\');
      remote:=oldremote;
      if not keypressed then begin
        if (copy(mess,1,7)='UniMess') and (UniMeasure<>nil)
        then DoUniMess(UniMeasure^.GetUniMessFromName(copy(mess,9,255)))
        else if copy(mess,1,7)='TakeAbs' then TakeAbsorptionPicture;
        Title('Executing Measurement Queue');
        {writeln('MemAvail:',MemAvail,' MaxAvail:',MaxAvail);}
        if (EraseQueue) then begin
          FindFirst(QueuePath+'\'+DirInfo.Name+'\*.*', AnyFile, MessDirInfo);
          while DosError = 0 do begin
            if (MessDirInfo.Name<>'.') and (MessDirInfo.Name<>'..') then begin
              assign(old,QueuePath+'\'+DirInfo.Name+'\'+MessDirInfo.Name);
              erase(old);
            end;
            FindNext(MessDirInfo);
          end;
          chdir(QueuePath);
          FSplit(DirInfo.Name, D, N,E);
          rmdir(N);
        end;
      end;
    end;
    FindNext(DirInfo);
  end;
  writeln('Loading Parameters...');
  {writeln('MemAvail:',MemAvail,' MaxAvail:',MaxAvail);}
  LoadParameters(MainDir+'Save\');
end;

procedure IterateQueue;
begin
  while not keypressed do ExecuteQueue(false);
end;

procedure ClearQueue;
var DirInfo,MessDirInfo: SearchRec;
    QueuePath:string;
    inp:text;
    mess:string;
    D: DirStr;
    N: NameStr;
    E: ExtStr;
    old:file;
begin
  Title('Clearing Measurement Queue');
  if remote then QueuePath:=MeasureQueueRemoteDirectory
  else QueuePath:=MeasureQueueDirectory;
  FindFirst(QueuePath+'\*.*', Directory, DirInfo);
  while DosError = 0 do begin
    if (DirInfo.Name<>'.') and (DirInfo.Name<>'..') and (DirInfo.Attr=Directory) then begin
      FindFirst(QueuePath+'\'+DirInfo.Name+'\*.*', AnyFile, MessDirInfo);
      while DosError = 0 do begin
        if (MessDirInfo.Name<>'.') and (MessDirInfo.Name<>'..') then begin
          assign(old,QueuePath+'\'+DirInfo.Name+'\'+MessDirInfo.Name);
          erase(old);
        end;
        FindNext(MessDirInfo);
      end;
      chdir(QueuePath);
      FSplit(DirInfo.Name, D, N, E);
      rmdir(N);
    end;
    FindNext(DirInfo);
  end;
end;

procedure ShowMemory;
var action:boolean;
begin
  Title('Memory');
  writeln('MemAvail:',MemAvail,' MaxAvail:',MaxAvail);
  action:=true;
  repeat IdleProc(action);action:=false; until keypressed;
end;

procedure DoUniMess(mess:PUniMess);
var oldParam,lastParam:array[1..MaxVars] of double;
    l,m,n,o,p:integer;
    c:char;
    mix:PMix;
    buf:string;
    aktpoint,nrpoints:integer;
    oldProbe,oldExp:double;
    HelpVisionCommand:string;

  procedure Reset;
  var n:integer;
  begin
    with mess^ do begin
      VisionStopSerie;
      for n:=1 to mess^.nrvars do if vars[n]<>nil then vars[n]^.GetPointer^:=oldParam[n];
      dispose(mix,done);
    end;
  end;

  procedure PrepareParameters;
  var n:integer;
      buf,buf2,hbuf:string;
  begin
    with mess^ do begin
      for n:=1 to nrvars do if vars[n]<>nil then LastParam[n]:=vars[n]^.GetPointer^;
      mix^.GetPoint;
      inc(aktpoint);
      str(aktpoint,buf);
      str(nrpoints,buf2);
      buf:=buf+'/'+buf2;
      writeln(aktpoint,'/',nrpoints);
      for n:=1 to nrvars do begin
        writeln(vars[n]^.Getname+' : ',vars[n]^.GetPointer^:8:4);
        str(vars[n]^.GetPointer^:8:4,buf);
        hbuf:=vars[n]^.Getname+'='+buf2;
        if length(buf)+length(hbuf)<255 then buf:=buf+' '+hbuf;
      end;
      VisionMessage(buf);
    end;
  end;

begin
  if mess=nil then exit;
  with mess^ do begin
    BlackTitle(Name^+' Measurement');
    writeln('Saving parameters...');
    SaveParameters(MainDir);
    VisionMessage(Name^+' Measurement');
    if not CameraReady then exit;
    NormPictureCounter:=NormPictureCounterMax div 2;
    for n:=1 to nrvars do if vars[n]<>nil then oldParam[n]:=vars[n]^.GetPointer^;
    mix:=new(PMix,init(nrvars,PDouble(vars[1]^.GetPointer),
      PDouble(vars[2]^.GetPointer),PDouble(vars[3]^.GetPointer),PDouble(vars[4]^.GetPointer),true));
    for l:=1 to Repetitions do begin
      for m:=1 to nr[1] do begin
        if nrvars=1 then begin
          mix^.AddPoint1(start[1]+(Ende[1]-Start[1])*(m-1)/(nr[1]-1));
        end else begin
          for n:=1 to nr[2] do begin
            if nrvars=2 then begin
              mix^.AddPoint2(start[1]+(Ende[1]-Start[1])*(m-1)/(nr[1]-1),
                             start[2]+(Ende[2]-Start[2])*(n-1)/(nr[2]-1));
            end else begin
              for o:=1 to nr[3] do begin
                if nrvars=3 then begin
                  mix^.AddPoint3(start[1]+(Ende[1]-Start[1])*(m-1)/(nr[1]-1),
                                 start[2]+(Ende[2]-Start[2])*(n-1)/(nr[2]-1),
                                 start[3]+(Ende[3]-Start[3])*(o-1)/(nr[3]-1));
                end else begin
                  for p:=1 to nr[4] do
                  mix^.AddPoint4(start[1]+(Ende[1]-Start[1])*(m-1)/(nr[1]-1),
                                 start[2]+(Ende[2]-Start[2])*(n-1)/(nr[2]-1),
                                 start[3]+(Ende[3]-Start[3])*(o-1)/(nr[3]-1),
                                 start[4]+(Ende[4]-Start[4])*(p-1)/(nr[4]-1));
                end;
              end;
            end;
          end;
        end;
      end;
    end;
    nrpoints:=Repetitions*nr[1];
    if nrvars>1 then nrpoints:=nrpoints*nr[2];
    if nrvars>2 then nrpoints:=nrpoints*nr[3];
    if nrvars>3 then nrpoints:=nrpoints*nr[4];
    aktpoint:=0;
    buf:=vars[1]^.GetName;
    if nrvars>1 then buf:=buf+' '+vars[2]^.GetName;
    if nrvars>2 then buf:=buf+' '+vars[3]^.GetName;
    if nrvars>3 then buf:=buf+' '+vars[4]^.GetName;
    for n:=1 to (MeasurementStartPoint-1) do PrepareParameters;
    VisionStartSerie(Name^+' measurement',nrvars,buf,true,mess);
    PrepareParameters;
    if TakeAbsPicture then begin
      Reset;
      exit;
    end;
    while (not mix^.eof) do begin
      PrepareParameters;
      PrepareTakeAbsPicture;
      VisionRegisterInformation(nrvars,lastParam);
      if DoTakeAbsPicture(1,'-') then begin
        Reset;
        exit;
      end;
    end;
    for n:=1 to nrvars do if vars[n]<>nil then LastParam[n]:=vars[n]^.GetPointer^;
    VisionRegisterInformation(nrvars,lastParam);
    Reset;
    HelpVisionCommand:=VisionCommand^;
    while ((HelpVisionCommand<>'') and (HelpVisionCommand[1]=' ')) do HelpVisionCommand:=copy(HelpVisionCommand,2,255);
    if HelpVisionCommand<>'' then Vision.VisionCommand('Vision'+HelpVisionCommand);
    VisionMessage(Name^+' Measurement finished');
  end;
end;

procedure OpenShutters;
begin
  ShutterAndorProbe(on);
  ShutterProbeRepumpAndOpticalPumpingBeam(on);
  ShutterMaster7AndMOT7LiPrincipal(on);
  ShutterMOT7principal(on);
  ShutterMOT7repump(on);
  ShutterProbe(open);
  ShutterMOT6principal(on);
  ShutterMOT6repump(on);
  ShutterMOT(on);
  ShutterAOM20(on);
  {ShutterSlowerJet(open);}
  ShutterSlower7principal(open);
  ShutterSlower7repump(open);
  ShutterSlower6principal(open);
  ShutterSlower6repump(open);
  IOUpdate;
end;

procedure CloseShutters;
begin
  ShutterAndorProbe(off);
  ShutterProbeRepumpAndOpticalPumpingBeam(off);
  ShutterMaster7AndMOT7LiPrincipal(off);
  ShutterMOT7principal(off);
  ShutterMOT7repump(off);
  ShutterProbe(close);
  ShutterMOT6principal(off);
  ShutterMOT6repump(off);
  ShutterMOT(off);
  ShutterAOM20(off);
  {ShutterSlowerJet(close);}
  ShutterSlower7principal(close);
  ShutterSlower7repump(close);
  ShutterSlower6principal(close);
  ShutterSlower6repump(close);
  IOUpdate;
end;

procedure TestShutters;
begin
  Title('Testing Shutters');
  StartCounter;
  repeat
    OpenShutters;
    Wait(99*ID+1000);
    CloseShutters;
    Wait(100*ID+1000);
  until keypressed;
  while keypressed do readkey;
end;

procedure TestAnalogOutputs;
var channel:byte;
    value:word;
begin
  Title('Test Analog Outputs');
  write('Channel : ');
  readln(channel);
  write('Value   : ');
  readln(Value);
  IOAnalogOutDirect(channel,value);
end;

procedure TestAnalogInputs;
var n,x,y:byte;
begin
  Title('Test Analog Inputs');
  x:=wherex;
  y:=wherey;
  repeat
    gotoxy(x,y);
    for n:=0 to MaxAnaIn-1 do writeln(n:4,IOAnalogIn(n):10,'     ');
    delay(100);
  until keypressed;
end;

procedure TestDigitalInputs;
var n,x,y,b:byte;
begin
  Title('Test Digital Inputs');
  x:=wherex;
  y:=wherey;
  repeat
    gotoxy(x,y);
    for n:=0 to MaxDigIn-1 do begin
      for b:=0 to IOgetMaxInBitNr(n)-1 do begin
        writeln(n:4,b:4,IODigitalIn(n,b):10,'     ');
      end;
    end;
    delay(100);
  until keypressed;
end;

procedure CalibrateFluorescence;
var n:word;
    h:double;
    action:boolean;
begin
  Title('Calibrating Fluorescence');
  h:=0;
  for n:=1 to 1000 do begin
    MeasureFluorescence(0);
    h:=h+GetLastFluorescence(0);
    delay(1);
  end;
  FluorescenceMax:=h/1000.0;
  writeln('Fluorescence Max = ',FluorescenceMax:6:2);
  SwitchIGBTsOff;
  IOUpdate;
  h:=0;
  for n:=1 to 1000 do begin
    MeasureFluorescence(0);
    h:=h+GetLastFluorescence(0);
    delay(1);
  end;
  FluorescenceMin:=h/1000.0;
  SwitchIGBTsToMOT;
  IOUpdate;
  writeln('Fluorescence Min = ',FluorescenceMin:6:2);
  writeln;
  writeln('Press any key');
  action:=true;
  repeat IdleProc(action);action:=false; until keypressed;
end;

procedure SweepAnalogOutputs;
var i:word;
    minchannel,maxchannel,n:byte;
begin
  Title('Sweep Analog Outputs');
  write('MinChannel : ');
  readln(minchannel);
  write('MaxChannel : ');
  readln(maxchannel);
  if minchannel>maxchannel then minchannel:=maxchannel;
  if minchannel>MaxAnaOut-1 then minchannel:=MaxAnaOut-1;
  if maxchannel>MaxAnaOut-1 then maxchannel:=MaxAnaOut-1;
  i:=0;
  repeat
    for n:=minchannel to Maxchannel do IOAnalogOutDirect(n,i+n*10000);
    i:=i+1;
  until keypressed;
end;

procedure TestDigitalOutputs;
var channel,bit,value:word;
begin
  Title('Test Digital Outputs');
  write('Channel : ');
  readln(channel);
  write('Bit [0..15]    : ');
  readln(bit);
  write('Value [1/0 2=blink]    : ');
  readln(Value);
  if value=2 then begin
    repeat
      IODigitalOutDirect(channel,bit,false);
      Wait(101*ID+500);
      IODigitalOutDirect(channel,bit,true);
      Wait(102*ID+500);
    until keypressed;
  end else IODigitalOutDirect(channel,bit,value<>0);
end;

procedure SweepDigitalOutputs;
var channel,n,altn:word;
begin
  Title('Sweep Digital Outputs');
  write('Channel : ');
  readln(channel);
  writeln('Chain');
  altn:=0;
  repeat
    for n:=0 to 15 do begin
      IODigitalOutDirect(channel,altn,off);
      IODigitalOutDirect(channel,n,on);
      altn:=n;
      delay(100);
    end;
  until keypressed;
  while keypressed do readkey;
  writeln('Flashing');
  repeat
    for n:=0 to 15 do IODigitalOutDirect(channel,n,off);
    delay(2000);
    for n:=0 to 15 do IODigitalOutDirect(channel,n,on);
    delay(2000);
  until keypressed;
end;

procedure SendIEEE;
var status:integer;
    adress:byte;
    command:string;
begin
  title('Send IEEE command');
  write('Adress : ');
  readln(adress);
  repeat;
    write('Command : ');
    readln(command);
    if command<>'' then begin
      IEEE.Send(adress,command,status);
      writeln('Status = ',status);
    end;
  until command='';
end;

procedure WatchSRQ;
var status:integer;
    adress:byte;
    command:string;
begin
  title('Watch SRQ');
  repeat;
    if srq then write('SRQ');
  until keypressed;
end;

procedure DancingMOT;
const Period=500;
      MaxFinalSlowerCurrent=20;{50;}
      MaxTransfer2Voltage=0.6;{1.2;}
      Transfer2VoltageOffset=3;
var t:double;
begin
  StartCounter;
  SwitchFinalSlowerVoltage(off);
  SetFinalSlowerCurrent(0);
  Wait(103*ID+10);
  RampSlowerField(off,100,10);
  SwitchSlowerCompensation(off);
  SwitchSlower7(off);
  SwitchSlower6(off);
  ShutterSlowerJet(close);
  Wait(104*ID+200);
  SwitchSlowerCompensation(on);
  SwitchFinalSlowerVoltage(on);
  t:=0;
  repeat
    SetFinalSlowerCurrent(MaxFinalSlowerCurrent*(1+sin(t*2*Pi/Period))/2);
    SetTransfer2Voltage(MaxTransfer2Voltage*(1+cos(t*2*Pi/Period))/2+Transfer2VoltageOffset);
    t:=t+1;
    Wait(105*ID+1);
  until keypressed;
  SwitchSlowerCompensation(off);
  SetFinalSlowerCurrent(SlowerCurrent);
  SwitchFinalSlowerVoltage(on);
  RampSlowerField(on,100,10);
  SetTransfer2Voltage(MOTTransfer2Voltage);
  IOUpdate;
end;

procedure TestSyncOnLineTrigger;
begin
  repeat
    StartCounter;
    SynchronizeToLine;
    IOdigitalOut(1,15,on);
    IOUpdate;
    Delay(1);
    IOdigitalOut(1,15,off);
    IOUpdate;
    Delay(1);
  until  keypressed;
end;

procedure TestDualFlou;
var Li7Fluorescence,Li6Fluorescence:double;
begin
  Title('TestDualFlou');
  StartCounter;
  repeat
    MeasureTwoIsotopeFlourescence(Li7Fluorescence,Li6Fluorescence,50,3);
    gotoxy(10,10);
    writeln('7LiFluo:',Li7Fluorescence*100:2:2,'6LiFluo:',Li6Fluorescence*100:2:4);
    Wait(100);
  until keypressed;
end;

var LastTime:longint;
    LastFluo6Time:longint;
    gepiept:boolean;
    SaveOp:boolean;

const ErrorOccured:boolean=true;
      ErrorLogName='c:\Lithium\Error.Log';
function IdleProc(action:boolean):boolean;
var Hour, Minute, Second, Sec100: Word;
    AktTime:longint;
    altx,alty:byte;
    log:text;
    f:file of byte;
    size:longint;
    Year, Month, Day, DayOfWeek: Word;
    ScreenSaverX,ScreenSaverY:integer;
    ScreenSaverTimout:word;
    ScreenSaverDirectionX:integer;
    ScreenSaverDirectionY:integer;

  procedure ScreenSaverInit;
  begin
    TextBackground(black);
    textcolor(yellow);
    randomize;
    ScreenSaverX:=66;
    ScreenSaverY:=1;
    ScreenSaverDirectionX:=random(3);
    dec(ScreenSaverDirectionX);
    ScreenSaverDirectionY:=random(3);
    dec(ScreenSaverDirectionY);
  end;

  procedure Screensaver;
  begin
    inc(ScreenSaverTimout);
    if ScreenSaverTimout>3000 then begin
      ScreenSaverTimout:=0;
      gotoxy(ScreenSaverX,ScreenSaverY);
      TextBackground(black);
      textcolor(yellow);
      write('        ');
      if random<0.01 then begin
        ScreenSaverDirectionX:=random(3);
        dec(ScreenSaverDirectionX);
        ScreenSaverDirectionY:=random(3);
        dec(ScreenSaverDirectionY);
      end;
      if random<0.01 then begin
        gotoxy(random(80)+1,random(ScreenMaxY)+1);
        TextColor(random(16));
        write('*');
        TextColor(yellow);
      end;
      ScreenSaverX:=ScreenSaverX+ScreenSaverDirectionX;
      ScreenSaverY:=ScreenSaverY+ScreenSaverDirectionY;
      if (ScreenSaverX>73) then begin
        ScreenSaverDirectionX:=-ScreenSaverDirectionX;
        ScreenSaverX:=72;
      end;
      if ScreenSaverY>ScreenMaxY then begin
        ScreenSaverDirectionY:=-ScreenSaverDirectionY;
        ScreenSaverY:=ScreenMaxY-1;
      end;
      if ScreenSaverX<1 then begin
        ScreenSaverDirectionX:=-ScreenSaverDirectionX;
        ScreenSaverX:=2;
      end;
      if ScreenSaverY<1 then begin
        ScreenSaverDirectionY:=-ScreenSaverDirectionY;
        ScreenSaverY:=2;
      end;
      gotoxy(ScreenSaverX,ScreenSaverY);
      write(hour:2,':',minute:2,':',second:2);
    end;
  end;

var help:word;
    Li7Fluorescence,Li6Fluorescence:double;
begin
  IdleProc:=false;
  GetTime(Hour, Minute, Second, Sec100);

  GetDate(Year, Month, Day, DayOfWeek);
  if not NoTemperatureOrWaterError then begin
    if not SaveOp then begin
      gotoxy(5,1);
      textcolor(yellow+128);
      textbackground(LightRed);
      write(' Error ');
    end;
    if not ErrorOccured then begin
      errorOccured:=true;
      ErrorBeep;
      Assign(f,ErrorLogName);
      Reset(f);
      size := FileSize(f);
      system.close(f);
      assign(log,ErrorLogName);
      if (size>100000) then rewrite(log) else append(log);
      writeln(log,'Error occured the ',day,'.',month,'.',year,' at ',hour,':',minute,':',second);
      system.close(log);
    end;
  end else begin
    if errorOccured and not SaveOp then begin
      gotoxy(5,1);
      textcolor(yellow);
      textbackground(Blue);
      write('');
      errorOccured:=false;
    end;
  end;
  if SaveOp and not action then ScreenSaver;
  altx:=wherex;
  alty:=wherey;
  MeasureFluorescence(0);
  gotoxy(13,1);
  textcolor(yellow);
  textbackground(Blue);
  if not SaveOp then begin
    if (Fluo6Delay>0) then begin
      if (((AktTime-LastFluo6Time)<0) or ((AktTime-LastFluo6Time)>Fluo6Delay)) then begin
        LastFluo6Time:=AktTime;
        StartCounter;
        MeasureTwoIsotopeFlourescence(Li7Fluorescence,Li6Fluorescence,5,10);
        gotoxy(4,1);
        write(' 6Li=',Li6Fluorescence*10000:3:0,' ');
        gotoxy(13,1);
        write(' 7Li=',Li7Fluorescence*100:3:0,'% ');
      end;
    end else begin
      write(' 7Li=',GetFluorescence(0)*100:3:0,'% ');
    end;
    if AkkusticFluorescence then begin
      Help:=round(10*System.Exp(GetFluorescence(0)*ln(2000)));
      sound(help)
    end else nosound;
  end;
  if DoDebug then begin
    gotoxy(25,1);
    textcolor(yellow+128);
    textbackground(LightGreen);
    if not SaveOp then write(' Debug ');
  end;
  gotoxy(65,1);
  textcolor(yellow);
  textbackground(blue);
  if not SaveOp then write(' ',hour:2,':',minute:2,':',second:2,' ');
  gotoxy(altx,alty);
  AktTime:=longint(Hour)*3600+longint(Minute)*60+longint(Second);
  if ((Hour=12) or (Hour=19)) and (Minute>=50) and (Minute<=59) then begin
    if (second=0) then begin
      if not gepiept then Potbeep;
      gepiept:=true;
    end else gepiept:=false;
  end;
  if action then begin
    LastTime:=AktTime;
    if saveOp then IdleProc:=true;
    saveOp:=false;
  end else begin
    if (AktTime-LastTime)>longint(IdleTime) then begin
      CloseShutters;
      ShutterSlowerJet(close);
      IOUpdate;
      if not SaveOp then begin
        beep;beep;beep;
        SaveOp:=true;
        ScreenSaverInit;
        ScreenSaver;
      end;
    end;
  end;
end;

procedure InitExp(initSRS:boolean);
var n:byte;
begin
  {everything that has to be initialised has to stand here}
  VisionSetComPort(2);
  NdYAGSetComPort(1);
  StartCounter;
  OpenShutters;
  ShutterSlowerJet(open);
  Wait(106*ID+500);
  PulsGenInit(0);
  CloseShutters;
  ShutterSlowerJet(close);
  Wait(107*ID+500);
  ShutterSlowerJet(open);
  Beep;
  writeln('Branche les deux ordinateurs avec un cable serielle et test le SRS');

  SetMOTCurrent(0);
  SetMOTVoltage(0);
  SetTransfer1Voltage(0);
  SetTransfer1Current(0);
  SetTransfer2Voltage(0);
  SetNdYAGIntensity(0,ScaleHorizontalNdYagIntensity(0));
  SetNdYAGIntensity(1,ScaleHorizontalNdYagIntensity(0));
  SwitchHorizontalNdYAG(off);
  SwitchVerticalNdYAG(off);
  SwitchPushbeam(off);
  MatroxCameraTrigger(off);
  AndorCameraTrigger(off);
  ExposureTrigger(off);
  SwitchProbeBeamGPIB(off);
  SwitchSlowerCompensation(off);
  SetFinalSlowerCurrent(SlowerCurrent);
  SwitchFinalSlowerVoltage(on);
  RampSlowerField(on,100,10);
  SwitchIGBTsToMOT;
  SwitchOpticalPumpingBeam(off);
  SetAOM20Intensity(MOTIntensity);
  SetMOTCurrent(MOTMOTCurrent);
  SetTransfer1Voltage(0);
  IOUpdate;
  SetTransfer1Current(MaxTransfer1Current);
  SetTransfer2Voltage(MOTTransfer2Voltage);
  SetMOTVoltage(MOTMOTVoltage);
  SwitchAOM20(on);
  PrepareShutters;
  PrepareProbeBeam;
  SwitchProbeAndRepumpAOMHeating(ProbeAOMsDefaultOn);
  SwitchSlower7(SlowLithium7);
  SwitchSlower6(SlowLithium6);
  SwitchMaster7Repump(on);
  SwitchMaster7Principal(on);
  SwitchMaster6Repump(on);
  SwitchMaster6Principal(on);
  SwitchRepumpAndOpticalPumpingEnable(on);
  InitMagTrap;
  SwitchIoffeGuidingField(off);
  SwitchAOM20Frequency(off);
  SetPinchOffsetCurrent(0);
  SetAOMFrequencies;
  Set6LiProbeIntensity(Probe6MOTIntensity);
  Set7LiProbeIntensity(Probe7MOTIntensity);
  SetProbeConfiguration(ProbeMixed);
  SelectProbeBeam(Li6Probe);
  for n:=0 to 3 do SynthTrigger(n,off);
  SwitchSternGerlachField(off);
  IOCompleteUpdate;
  SetIdleProc(IdleProc);
  TriggerExposure;
  {if (InitSRS) then SRSSinus(1,0,0,0,0);}
  UsedFeshbachField:=NotInitialised;
end;

procedure CloseExp;
begin
  {everything that has to be done to set the experiment in a stable state
   after the termination of the program has to be done here}
  StartCounter;
{  ShutterSlowerJet(close);
  {SwitchMagTrapCompletelyOff;
  SwitchFinalSlowerVoltage(off);}
  SwitchRFGPIB(0,-144,off);
  Wait(108*ID+GPIBCommandTime);
  SwitchRFGPIB(1,-144,off);SwitchRF(1,off); 
  Wait(109*ID+GPIBCommandTime);
  {CloseShutters;}
  IOUpdate;
end;

procedure InitCards;
begin
  DAS1601Init;
  DDA06initialize(AOBOCO);
  MultiOutInit;
  IOUpdate;
end;

procedure Myexitproc;far;
begin
  exitproc:=oldexitproc;
  CloseExp;
end;

begin
  oldexitproc:=exitproc;
  exitproc:=@myexitproc;
end.
