unit ExpIO;
{
  This unit is used to give names which are easy to memorize to the
  different output and input ports.
  It contains also simple IEEE Commands.
}
interface
uses Params;

{Timing and IO Update}

procedure startCounter;
procedure Wait(time:double);
procedure IOUpdate;
procedure IOupdateDigitalPort(port:byte);
procedure IOupdateAnalogPort(port:byte);
procedure StartWaitSum;
function GetTotalWait:double;

{PortList}

procedure InitPortList;
procedure StartPortList;
procedure StartPortListDuringWait(time:double);
procedure StopPortList;
procedure StopAndRepeatPortList(times:longint);
procedure SequenceStartPortListDuringWait(time:double);
procedure SequenceStopPortList;
function GetSequenceTime:double;

{Sequence}

procedure StartSequenceDuringWait(time:double);
procedure StartSequence;
procedure StopSequence;
procedure DoSequence;
procedure PauseSequence;
procedure UnPauseSequence;

{digital Outputs}

procedure SwitchAOM20(onoff:boolean);
procedure SwitchAOM20Frequency(onoff:boolean);
procedure SwitchAOM20Delayed(onoff:boolean);
procedure SwitchMaster7Repump(onoff:boolean);
procedure SwitchMaster7Principal(onoff:boolean);
procedure SwitchMaster6Repump(onoff:boolean);
procedure SwitchMaster6Principal(onoff:boolean);
procedure SwitchSternGerlachField(onoff:boolean);
procedure SwitchAxialGradient(onoff:boolean);
procedure MatroxCameraTrigger(onoff:boolean);
procedure AndorCameraTrigger(onoff:boolean);
procedure ExposureTrigger(onoff:boolean);
procedure SynthTrigger(Nr:byte;onoff:boolean);
procedure SwitchSlowerField(onoff:boolean);
procedure SwitchOpticalPumpingField(onoff:boolean);
procedure SwitchMOTGuidingField(onoff:boolean); {Identical with Optical Pumping Field, but Pretriggered}
procedure SwitchIoffeGuidingField(onoff:boolean);
procedure SwitchSlower7(onoff:boolean);
procedure SwitchSlower6(onoff:boolean);
procedure SwitchOpticalPumpingBeam(onoff:boolean);
procedure SwitchProbeAndRepumpAOMHeating(onoff:boolean);
procedure SwitchProbeAndRepumpAOMHeatingDirect(onoff:boolean);
procedure SwitchRepumpAndOpticalPumpingEnable(onoff:boolean);
procedure Select6LiProbeBeam(probe:boolean);
procedure Select7LiProbeBeam(probe:boolean);
procedure SelectProbeBeam(Li7:boolean);
procedure SwitchPushbeam(onoff:boolean);
procedure ShutterAndorProbe(onoff:boolean);
procedure ShutterMOT7principal(onoff:boolean);
procedure ShutterMOT7repump(onoff:boolean);
procedure ShutterSlowerJet(onoff:boolean);
procedure ShutterMOT6principal(onoff:boolean);
procedure ShutterMOT6repump(onoff:boolean);
procedure ShutterProbe(onoff:boolean);
procedure ShutterMOT(onoff:boolean);
procedure ShutterAOM20(onoff:boolean);
procedure ShutterSlower7principal(onoff:boolean);
procedure ShutterSlower7repump(onoff:boolean);
procedure ShutterSlower6principal(onoff:boolean);
procedure ShutterSlower6repump(onoff:boolean);
procedure ShutterProbeRepumpAndOpticalPumpingBeam(onoff:boolean);
procedure ShutterMaster7AndMOT7LiPrincipal(onoff:boolean);
procedure SwitchOszilloskopTrigger(onoff:boolean);
procedure LockIGBTs;
procedure UnLockIGBTs;
procedure SwitchIGBT(nr:byte;onoff:boolean);
procedure SwitchTransferDecharging(onoff:boolean);
procedure SwitchMOTDecharging(onoff:boolean);
function GetMOTDecharging:boolean;
procedure SwitchSlowerCompensation(onoff:boolean);
procedure SwitchHorizontalNdYAG(onoff:boolean);
procedure SwitchVerticalNdYAG(onoff:boolean);
procedure SwitchRF(Synth:byte;onoff:boolean);

{analog Outputs}

function GetVCOValue(n:integer;frequ:double):word;
procedure SetAOMfrequency(n:integer;frequ:double);
procedure EnableAOMs;
procedure DisableAOMs;
procedure DisableMOT6Repump;
procedure DisableMOT7Repump;
procedure EnableMOT6Repump;
procedure EnableMOT7Repump;
procedure DisableMOT6Principal;
procedure DisableMOT7Principal;
procedure EnableMOT6Principal;
procedure EnableMOT7Principal;
procedure SetAOM20Intensity(Intensity:double);
procedure SetMOTCurrent(Current:double);
procedure SetMOTVoltage(Voltage:double);
procedure SetTransfer1Current(Current:double);
procedure SetTransfer1Voltage(Voltage:double);
procedure SetTransfer2Voltage(Voltage:double);
procedure SetTransferCurrent(current:double);
procedure SetTransferCurrentControlVoltage(f:double);
procedure SetMOTCurrentControlVoltage(f:double);
procedure SetPinchCurrentControlVoltage(f:double);
procedure SetBarsCurrentControlVoltage(f:double);
procedure SetBarsCurrent(Current:double);
procedure SetBarsVoltage(Voltage:double);
procedure SetFinalSlowerCurrent(Current:double);
procedure SetPinchOffsetCurrent(Current:double);
procedure SwitchFinalSlowerVoltage(onoff:boolean);
procedure SetRFSweepCopyFrequency(Synth:byte;Frequency:double);
procedure Set6LiProbeIntensity(Intensity:double);
procedure Set7LiProbeIntensity(Intensity:double);
procedure SetProbeConfiguration(Angle:double);
procedure SetNdYAGIntensity(BeamDir:byte;Intensity:double);
procedure SetVCORFSweepValue(Synth:byte;value:double);

{analog Inputs}

procedure SaveAktCurrents;
function GetFluorescence(n:integer):double;
function GetLastFluorescence(n:integer):double;
procedure MeasureFluorescence(n:integer);
procedure MeasureIntegratedFluorescence(n:integer);
procedure NormalizeIntegratedFluorescence(n:integer;number:double);

{digital Inputs}

function NoTemperatureOrWaterError:boolean;
function LineTrigger:boolean;
procedure SynchronizeToLine;

{GPIB Commands}

procedure SwitchProbeBeamGPIB(onoff:boolean);
function GetFrequency:double;
procedure PrepareSinglePuls(SRSNr:byte;delaytime,pulstime,Resolution,ForceReprogram:double);
procedure PrepareRFPuls(SRSNr:byte;delaytime,pulstime,Frequency,Level:double);
procedure PrepareRFBurstPuls(SRSNr:byte;BurstCount,Frequency,Level,ForceReprogram:double);
procedure SRSSinus(SRSNr:byte;Frequency:double;Amplitude,dummy1,dummy2:double);
procedure PulsGenInit(Adr:byte);
procedure PulsGenPrepareChannelDirect(Adr:byte;Channel:byte;Delay:double;Width:double);
procedure PulsGenStoreDirect(Adr:byte;slot:byte);
procedure PulsGenRecall(Adr:byte;slot:double);

const open=true;
      close=false;
      Li7Probe=true;
      Li6Probe=false;

implementation
uses crt,IO,IEEE,Timing,Sequence,PortList,debug,Pulsgen;
var TotalWait:double;

{PortList}

const PortListStartedInSequenceOutMode:boolean=false;
      InPortList:boolean=false;
      PortListStartedDuringWait:boolean=false;
      PortListWaitTime:double=0;
      InSequencePortList:boolean=false;
      OldInSequencePortList:boolean=false;

procedure InitPortList;
begin
  PortListInit;
end;

procedure StartPortList;
begin
  IOUpdate;
  if SequenceOut then begin
    PortListBegin;
    PortListStartedInSequenceOutMode:=true;
  end else begin
    SequenceStartBlockByteArg(PortListExecute,PortListBegin);
    SequenceOut:=true;
    PortListStartedInSequenceOutMode:=false;
  end;
  InPortList:=true;
  PortListStartedDuringWait:=false;
end;

procedure StartPortListDuringWait(time:double);
begin
  if not SequenceOut then Wait(time);
  StartPortList;
  PortListStartedDuringWait:=true;
  PortListWaitTime:=time;
end;

procedure StopAndRepeatPortList(times:longint);
begin
  If not InPortList then begin
    writeln('ExpIO : StopPortList : not in Port List mode');
    delay(1000);
  end;
  IOUpdate;
  InPortList:=false;
  if SequenceOut then begin
    if PortListStartedInSequenceOutMode then begin
      if PortListStartedDuringWait then Wait(PortListWaitTime);
      PortListExecuteBuffer(times);
      PortListInit;
    end else begin
      PortListEnd;
      SequenceOut:=false;
      SequenceEndBlock;
    end;
  end else begin
    writeln('ExpIO : StopPortList: strange error : SequenceOutMode');
    delay(1000);
  end;
end;

procedure StopPortList;
begin
  StopAndRepeatPortList(1);
end;

procedure SequenceStartPortListDuringWait(time:double);
begin
  if SequenceOut then begin
    StartPortListDuringWait(time);
  end else begin
    TotalWait:=TotalWait+time;
    SequenceDoubleArg(SequenceStartPortListDuringWait,time,0);
    InSequencePortList:=true;
  end;
end;

procedure SequenceStopPortList;
begin
  if SequenceOut then StopPortList
  else begin
    SequenceNoArg(SequenceStopPortList,0);
    InSequencePortList:=false;
  end;
end;

procedure DoSequenceWait(time:double);far;
begin
  IO.IOUpdate;
  if InPortList then PortListWait(time) else TimingWait(time);
end;

function GetSequenceTime:double;
begin
  GetSequenceTime:=Sequence.GetSequenceTime;
end;

{Timing}

function GetTotalWait:double;
begin
  GetTotalWait:=TotalWait;
end;

procedure StartWaitSum;
begin
  TotalWait:=0;
end;

procedure startCounter;
begin
  Timing.startCounter;
end;

procedure IOUpdate;
begin
  if SequenceOut then IO.IOUpdate
  else SequenceNoArg(IO.IOUpdate,0);
end;

procedure IOupdateDigitalPort(port:byte);
begin
  if SequenceOut then IO.IOUpdateDigitalPort(port)
  else SequenceByteArg(IO.IOUpdateDigitalPort,port,0);
end;

procedure IOupdateAnalogPort(port:byte);
begin
  if SequenceOut then IO.IOUpdate
  else SequenceByteArg(IO.IOUpdateAnalogPort,port,0);
end;

procedure Wait(time:double);
begin
  TotalWait:=TotalWait+time;
  if InSequencePortList then begin
    {SequenceDoubleArg(PortListWait,time,0);}
    SequenceWait(time);
    exit;
  end;
  if InPortList then begin
    IO.IOUpdate;
    PortListWait(time);
  end else begin
    if SequenceOut then begin
      IO.IOUpdate;
      TimingWait(time);
    end else begin
      SequenceWait(time);
    end;
  end;
end;

{Sequence}

procedure StartSequenceDuringWait(time:double);
begin
  Sequence.StartSequence;
  Wait(time);
end;

procedure StartSequence;
begin
  Sequence.StartSequence;
end;

procedure StopSequence;
begin
  Sequence.StopSequence;
end;

procedure DoSequence;
begin
  Sequence.DoSequence;
end;

procedure PauseSequence;
begin
  Sequence.PauseSequence;
  OldInSequencePortList:=InSequencePortList;
  InSequencePortList:=false;
end;

procedure UnPauseSequence;
begin
  Sequence.UnPauseSequence;
  InSequencePortList:=OldInSequencePortList;
end;

{digital Outputs}

procedure SwitchAOM20(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(0,0,onoff)
  else SequenceBooleanArg(SwitchAOM20,onoff,0);
end;

procedure SwitchAOM20Delayed(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(0,0,onoff)
  else SequenceBooleanArg(SwitchAOM20Delayed,onoff,MOTShutterDelay);
end;

procedure SynthTrigger(Nr:byte;onoff:boolean);
begin
  if SequenceOut then begin
    case nr of
      0:IOdigitalOut(4,0,onoff);
      1,3:IOdigitalOut(4,1,onoff);
      2:IOdigitalOut(4,2,onoff);
    end;
  end else SequenceByteBooleanArg(SynthTrigger,nr,onoff,0);
end;

procedure SetVCORFSweepValue(Synth:byte;value:double);
var h:double;
    scale:double;
begin
  if SequenceOut then begin
    case Synth of
      0:scale:=0.12;
      1:scale:=0.18;
    end;
    h:=value*65535.0*scale;
    if (h>65535) then h:=65535;
    if (h<0) then h:=0;
    case Synth of
      0:IOanalogOut(30,round(h));
      1:IOanalogOut(31,round(h));
      {2:IOanalogOut(28,round(h));}
    end;
  end else SequenceByteDoubleArg(SetVCORFSweepValue,Synth,Value,0);
end;

procedure ExposureTrigger(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(0,10,onoff)
  else SequenceBooleanArg(ExposureTrigger,onoff,0);
end;

procedure MatroxCameraTrigger(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(0,11,onoff)
  else SequenceBooleanArg(MatroxCameraTrigger,onoff,0);
end;

procedure AndorCameraTrigger(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(0,4,onoff)
  else SequenceBooleanArg(AndorCameraTrigger,onoff,-AndorPreTriggerDelay);
end;

const SlowerCompensationOn:boolean=true;

procedure SwitchSlowerField(onoff:boolean);
begin
  if SequenceOut then begin
    if SlowerCompensationOn then begin
      SwitchSlowerCompensation(off);
      IOUpdate;
      writeln('ExpIO : SwitchSlowerField : SlowerCompensation On');
    end;
    IOdigitalOut(1,7,onoff);
    SlowerState:=onoff;
  end else SequenceBooleanArg(SwitchSlowerField,onoff,0);
end;

procedure SwitchOpticalPumpingField(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(1,5,onoff)
  else SequenceBooleanArg(SwitchOpticalPumpingField,onoff,0);
end;

procedure SwitchOpticalPumpingBeam(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(0,5,onoff)
  else SequenceBooleanArg(SwitchOpticalPumpingBeam,onoff,0);
end;

procedure SwitchProbeAndRepumpAOMHeating(onoff:boolean);
begin
  if SequenceOut then {IOdigitalOut(1,6,onoff)}
  else begin
    if onoff then SequenceBooleanArg(SwitchProbeAndRepumpAOMHeating,onoff,0)
    {-(ShutterProbeOffDelay-ProbeAOMOffTimeBeforeShutterOpen))}
    else SequenceBooleanArg(SwitchProbeAndRepumpAOMHeating,onoff,0);{-(ShutterProbeOnDelay+ProbeAOMOffTimeBeforeShutterOpen));}
  end;
end;

procedure SwitchProbeAndRepumpAOMHeatingDirect(onoff:boolean);
begin
  if SequenceOut then {IOdigitalOut(1,6,onoff)}
  else SequenceBooleanArg(SwitchProbeAndRepumpAOMHeating,onoff,0);
end;

procedure SwitchRepumpAndOpticalPumpingEnable(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(1,6,not onoff)
  else SequenceBooleanArg(SwitchRepumpAndOpticalPumpingEnable,onoff,0);
end;

procedure Select7LiProbeBeam(probe:boolean);
begin
  if SequenceOut then IOdigitalOut(1,10,probe)
  else SequenceBooleanArg(Select7LiProbeBeam,probe,0);
end;

procedure Select6LiProbeBeam(probe:boolean);
begin
  if SequenceOut then IOdigitalOut(1,1,probe)
  else SequenceBooleanArg(Select6LiProbeBeam,probe,0);
end;

procedure SelectProbeBeam(Li7:boolean);
begin
  if Li7 then begin
    Select7LiProbeBeam(true);
    Select6LiProbeBeam(false);
  end else begin
    Select7LiProbeBeam(false);
    Select6LiProbeBeam(true);
  end;
end;

procedure SwitchPushbeam(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(4,4,onoff)
  else SequenceBooleanArg(SwitchPushbeam,onoff,0)
end;

procedure ShutterAndorProbe(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(2,15,not onoff)
  else SequenceBooleanArg(ShutterAndorProbe,onoff,0)
end;

procedure SwitchMOTGuidingField(onoff:boolean); {Identical with Optical Pumping Field}
begin
  if SequenceOut then IOdigitalOut(0,6,onoff)
  else begin
    if onoff then SequenceBooleanArg(SwitchMOTGuidingField,on,-MOTGuidingFieldRaiseTime)
    else SequenceBooleanArg(SwitchMOTGuidingField,off,0);
  end;
end;

procedure SwitchIoffeGuidingField(onoff:boolean);
begin
  if SequenceOut then {IOdigitalOut(1,1,onoff) }else begin
    if onoff then SequenceBooleanArg(SwitchIoffeGuidingField,on,-IoffeGuidingFieldRaiseTime)
    else SequenceBooleanArg(SwitchIoffeGuidingField,off,0);
  end;
end;

procedure ShutterMOT6principal(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(1,8,not onoff)
  else SequenceBooleanArg(ShutterMOT6principal,onoff,0);
end;

procedure ShutterProbe(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(1,9,not onoff)
  else SequenceBooleanArg(ShutterProbe,onoff,0);
end;

procedure ShutterMOT7principal(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(1,11,not onoff)
  else SequenceBooleanArg(ShutterMOT7principal,onoff,0);
end;

procedure ShutterMOT7repump(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(1,12,not onoff)
  else SequenceBooleanArg(ShutterMOT7repump,onoff,0);
end;

procedure ShutterMOT6repump(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(2,0,not onoff)
  else SequenceBooleanArg(ShutterMOT6repump,onoff,0);
end;

procedure ShutterAOM20(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(2,1,not onoff)
  else SequenceBooleanArg(ShutterAOM20,onoff,0);
end;

procedure ShutterMOT(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(2,14,onoff)
  else SequenceBooleanArg(ShutterMOT,onoff,0);
end;

procedure ShutterSlower7principal(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(2,2,not onoff)
  else SequenceBooleanArg(ShutterSlower7principal,onoff,0);
end;

procedure ShutterSlower7repump(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(2,3,not onoff)
  else SequenceBooleanArg(ShutterSlower7repump,onoff,0);
end;

procedure ShutterSlower6principal(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(2,4,not onoff)
  else SequenceBooleanArg(ShutterSlower6principal,onoff,0);
end;

procedure ShutterSlower6repump(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(2,5,not onoff)
  else SequenceBooleanArg(ShutterSlower6repump,onoff,0);
end;

procedure ShutterSlowerJet(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(1,14,onoff)
  else SequenceBooleanArg(ShutterSlowerJet,onoff,0);
end;

{procedure ShutterProbeRepumpAndOpticalPumpingBeamDelayed(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(2,11,not onoff)
  else begin
    if onoff then SequenceBooleanArg(ShutterProbeRepumpAndOpticalPumpingBeam,true,-ShutterRepumpDelay)
    else SequenceBooleanArg(ShutterProbeRepumpAndOpticalPumpingBeamDelayed,false,-ShutterRepumpDelay)
  end;
end;}

procedure ShutterProbeRepumpAndOpticalPumpingBeam(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(2,11,not onoff)
  else SequenceBooleanArg(ShutterProbeRepumpAndOpticalPumpingBeam,onoff,0)
end;

procedure ShutterMaster7AndMOT7LiPrincipal(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(2,12,onoff)
  else SequenceBooleanArg(ShutterMaster7AndMOT7LiPrincipal,onoff,0)
end;

procedure SwitchAOM20Frequency(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(2,13,onoff)
  else SequenceBooleanArg(SwitchAOM20Frequency,onoff,0)
end;

procedure SwitchOszilloskopTrigger(onoff:boolean);
begin
  {if SequenceOut then begin
    if onoff then IOAnalogOut(27,32000) else IOAnalogOut(27,0);}
    {IODigitalOut(1,15,onoff);}
  {end else SequenceBooleanArg(SwitchOszilloskopTrigger,onoff,0);}
end;

const IGBTslocked:boolean=false;

procedure LockIGBTs;
begin
  if SequenceOut then IGBTsLocked:=true
  else SequenceNoArg(LockIGBTs,0);
end;

procedure UnLockIGBTs;
begin
  if SequenceOut then IGBTsLocked:=false
  else SequenceNoArg(UnLockIGBTs,0);
end;

procedure SwitchIGBT(nr:byte;onoff:boolean);
begin
  if ((nr>=1) and (nr<12)) then begin
    if SequenceOut then begin
      if (IGBTsLocked) then begin
        SetTransfer1Voltage(0);
        SetTransfer2Voltage(0);
        SetTransfer1Current(0);
        SetMOTCurrent(0);
        SetMOTVoltage(0);
        SetBarsCurrent(0);
        SetBarsVoltage(0);
        IOUpdate;
        writeln('Error : ExpIO : SwitchIGBT : tried to switch off locked IGBTs');
      end;
      case nr of
        10:IOdigitalOut(0,14,onoff); {offset field}
        9:IOdigitalOut(0,7,onoff);   {IGBT for Feshbach field}
        7:IOdigitalOut(0,6,onoff);   {not in use}
        4:IOdigitalOut(0,3,onoff);  {we do not want to have IGBT4 on the
          security system, since simultaneous Switch off of IGBT4 and IGBT2 produces overtension}
        3:IOdigitalOut(0,2,onoff);  {we do not want to have IGBT3 on the
          security system, since simultaneous Switch off of IGBT3 and IGBT1 produces overtension}
        else IOdigitalOut(0,nr-1+8,onoff);
      end;
      aktIGBTStatus[nr]:=onoff;
    end else SequenceByteBooleanArg(SwitchIGBT,nr,onoff,0);
  end;
end;

procedure SwitchSternGerlachField(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(0,1,onoff)
  else SequenceBooleanArg(SwitchSternGerlachField,onoff,0);
end;

procedure SwitchAxialGradient(onoff:boolean);
begin
{  if SequenceOut then IOdigitalOut(0,1,onoff)
  else SequenceBooleanArg(SwitchAxialGradient,onoff,0);}
end;

const TransferDecharging:boolean=false;

procedure SwitchTransferDecharging(onoff:boolean);
begin
  if SequenceOut then begin
    if ((aktTransfer1Voltage<>0) or (aktTransfer2Voltage<>0)) and onoff then begin
      writeln('ExpIO : SwitchTransferCapacityDecharging(on) : Transfer1 voltage or Transfer2 voltage not 0',^G);
      delay(200);
      onoff:=false;
    end;
    IOdigitalOut(1,2,onoff);
    TransferDecharging:=onoff;
  end else SequenceBooleanArg(SwitchTransferDecharging,onoff,0);
end;

const MOTDecharging:boolean=false;

function GetMOTDecharging:boolean;
begin
  GetMOTDecharging:=MOTDecharging;
end;

procedure SwitchMOTDecharging(onoff:boolean);
begin
  if SequenceOut then begin
    if (aktMOTCurrent<>0) and (aktMOTVoltage<>0) and onoff then begin
      writeln('ExpIO : SwitchMOTCapacityDecharging(on) : MOT current or voltage not 0',^G);
      delay(200);
      onoff:=false;
    end;
    IOdigitalOut(1,0,onoff);
    MOTDecharging:=onoff;
  end else SequenceBooleanArg(SwitchMOTDecharging,onoff,0);
end;

procedure SwitchHorizontalNdYAG(onoff:boolean);
begin
  if SequenceOut then begin
    IOdigitalOut(2,10,onoff);
  end else SequenceBooleanArg(SwitchHorizontalNdYAG,onoff,0);
end;

procedure SwitchVerticalNdYAG(onoff:boolean);
begin
  if SequenceOut then begin
    IOdigitalOut(4,5,onoff);
  end else SequenceBooleanArg(SwitchVerticalNdYAG,onoff,0);
end;

procedure SwitchSlowerCompensation(onoff:boolean);
begin
  if SequenceOut then begin
    IOdigitalOut(1,3,onoff);
    SlowerCompensationOn:=onoff;
  end else SequenceBooleanArg(SwitchSlowerCompensation,onoff,0);
end;

{analog Outputs}

function GetVCOValue(n:integer;frequ:double):word;
var f,out:double;
const MaxVCO=8;
const MiddleFrequency:array[1..MaxVCO] of double=
      (193,204,204,208,205,205,82,84);

const VCOCalib:array[1..MaxVCO,0..9] of double =((
	31692.261,
	484.81453,
	-1.0666572,
	0.0036458273,
	-1.6731974E-4,
	-5.8133409E-6,
	2.0422899E-7,
	2.2506957E-10,
	-3.2829413E-11,
	1.6338049E-13
),(
	32022.241,
	443.34185,
	-0.18135145,
	-0.013823168,
	2.6129106E-4,
	5.6195774E-6,
	-6.0711583E-8,
	-3.3569E-10,
	4.1328744E-12,
	-4.2493098E-14
),(
	31946.885,
	444.47536,
	-0.74978454,
	-0.0055240760,
	6.3398056E-4,
	1.1595004E-6,
	-1.3791582E-7,
	2.573354E-10,
	9.6854844E-12,
	-5.4950378E-14
),(
	31958.190,
	438.75328,
	-0.020470062,
	-0.0013212124,
	2.6137394E-4,
	-2.6431505E-6,
	-7.2156492E-8,
	8.6126125E-10,
	6.0624052E-12,
	-6.9188702E-14
),(
	31991.410,
	441.30358,
	-0.54164075,
	-0.016562866,
	6.6264738E-4,
	1.0028623E-5,
        -1.7925973E-7,
	-2.2603522E-9,
	1.5546589E-11,
	1.7763996E-13
),(
	31927.617,
	443.62158,
	-0.61174709,
	-0.015798876,
	6.5766196E-4,
	8.8159226E-6,
	-1.7825275E-7,
	-2.0024963E-9,
	1.5913688E-11,
	1.6563881E-13
),(
	31377.258,
	739.80316,
	-1.2176214,
	-0.072122363,
	-0.018151091,
	-1.3372249E-4,
	4.8540847E-5,
	1.6774398E-6,
	1.4313684E-8,
	-3.9721719E-11
),(
	32190.152,
	702.06161,
	-0.94541543,
	-0.16915331,
	-0.019237769,
	2.1114484E-4,
	5.9881941E-5,
	1.7896821E-6,
	1.6538785E-8,
	5.590324E-12
));


begin
  if (n<=MaxVCO) then begin
    f:=frequ-MiddleFrequency[n];
    out:= VCOCalib[n,0]+VCOCalib[n,1]*f+VCOCalib[n,2]*f*f+VCOCalib[n,3]*f*f*f
         +VCOCalib[n,4]*f*f*f*f+VCOCalib[n,5]*f*f*f*f*f+VCOCalib[n,6]*f*f*f*f*f*f
         +VCOCalib[n,7]*f*f*f*f*f*f*f+VCOCalib[n,8]*f*f*f*f*f*f*f*f+VCOCalib[n,9]*f*f*f*f*f*f*f*f*f;
    if (out>65535) then begin
      out:=65535;
      writeln('ExpIO : GetVCOValue : Frequency of VCO ',n,' is too high');
      Delay(100);
    end;
    if (out<0) then begin
      out:=0;
      writeln('ExpIO : GetVCOValue : Frequency of VCO ',n,' is too low');
      Delay(100);
    end;
    GetVCOValue:=round(out);
  end else begin
    GetVCOValue:=0;
    writeln('ExpIO : GetVCOValue : wrong VCO number ',n);
  end;
end;

const MaxAOM=15;                          { 0, 1, 2, 3  4  5  6  7  8  9 10 11 12 13 14 15}
const VCOnumber:array[0..MaxAOM] of byte =(99, 6, 5, 1, 2,99,99,99,99,99,99, 7,99, 8, 3, 4);
      Portnumber:array[0..MaxAOM] of byte=(99, 7, 6, 0, 1,99,99,99,99,99,99, 4,99, 5, 2, 3);
var AOMValue:Array[0..MaxAOM] of word;

procedure SetAOMfrequency(n:integer;frequ:double);
var m:integer;
begin
  if SequenceOut then begin
    if Portnumber[n]<>99 then begin
      AOMValue[n]:=GetVCOValue(VCOnumber[n],abs(frequ));
      IOanalogOut(Portnumber[n],AOMValue[n]);
    end else writeln('ExpIO : SetAOMfrequency : wrong AOM number ',n);
  end else SequenceIntDoubleArg(SetAOMfrequency,n,frequ,0);
end;

procedure DisableAOM(n:integer);far;
var Value:word;
begin
  if SequenceOut then begin
    if Portnumber[n]<>99 then begin
      if AOMValue[n]<32767 then Value:=65535 else Value:=0;
      IOanalogOut(Portnumber[n],Value);
    end;
  end else SequenceIntArg(DisableAOM,n,0);
end;

procedure DisableAOMs;
var n:integer;
begin
  for n:=0 to MaxAOM do DisableAOM(n);
end;

procedure EnableAOM(n:integer);far;
begin
  if SequenceOut then begin
    if Portnumber[n]<>99 then IOanalogOut(Portnumber[n],AOMValue[n])
  end else SequenceIntArg(EnableAOM,n,0);
end;

procedure DisableMOT7Repump;
begin
{  DisableAOM(2);}
  SwitchMaster7repump(off);
end;

procedure DisableMOT6Repump;
begin
  {DisableAOM(13);}
  SwitchMaster6repump(off);
end;

procedure EnableMOT7Repump;
begin
  {EnableAOM(2);}
  SwitchMaster7repump(on);
end;

procedure EnableMOT6Repump;
begin
  {EnableAOM(13);}
  SwitchMaster6repump(on);
end;

procedure DisableMOT7Principal;
begin
  {DisableAOM(1);}
  SwitchMaster7principal(off);
end;

procedure DisableMOT6Principal;
begin
  DisableAOM(11);
  SwitchMaster6principal(off);
end;

procedure EnableMOT7Principal;
begin
  {EnableAOM(1);}
  SwitchMaster7principal(on);
end;

procedure EnableMOT6Principal;
begin
  EnableAOM(11);
  SwitchMaster6principal(on);
end;


procedure EnableAOMs;
var n:integer;
begin
  for n:=0 to MaxAOM do EnableAOM(n);
end;

procedure SwitchMaster7Principal(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(2,6,onoff)
  else SequenceBooleanArg(SwitchMaster7Principal,onoff,0);
end;

procedure SwitchMaster7Repump(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(2,7,onoff)
  else SequenceBooleanArg(SwitchMaster7Repump,onoff,0);
end;

procedure SwitchMaster6Principal(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(2,8,onoff)
  else SequenceBooleanArg(SwitchMaster6Principal,onoff,0);
end;

procedure SwitchMaster6Repump(onoff:boolean);
begin
  if SequenceOut then IOdigitalOut(2,9,onoff)
  else SequenceBooleanArg(SwitchMaster6Repump,onoff,0);
end;

procedure SwitchRF(Synth:byte;onoff:boolean);
begin
  if SequenceOut then begin
    case Synth of
      0:IOdigitalOut(1,15,onoff);
      1:IOdigitalOut(1,15,onoff);
      3:IOdigitalOut(1,13,onoff);
    end;
  end else SequenceByteBooleanArg(SwitchRF,Synth,onoff,0);
end;

procedure SwitchSlower7(onoff:boolean);
begin
  ShutterSlower7principal(onoff);
  ShutterSlower7repump(onoff);
  if onoff then begin
    EnableAOM(3);
    EnableAOM(4);
  end else begin
    DisableAOM(3);
    DisableAOM(4);
  end;
end;

procedure SwitchSlower6(onoff:boolean);
begin
  ShutterSlower6principal(onoff);
  ShutterSlower6repump(onoff);
  if onoff then begin
    EnableAOM(14);
    EnableAOM(15);
  end else begin
    DisableAOM(14);
    DisableAOM(15);
  end;
end;

procedure SetAOM20Intensity(Intensity:double);
var h:double;
begin
  if SequenceOut then begin
    h:=Intensity*65535.0;
    if (h>65535) then h:=65535;
    if (h<0) then h:=0;
    IOanalogOut(16,round(h));
  end else SequenceDoubleArg(SetAOM20Intensity,Intensity,0);
end;

procedure SetMOTCurrent(Current:double);
var h:double;
begin
  if SequenceOut then begin
    h:=Current*65535.0/500.0;
    if (h>65535) then h:=65535;
    if (h<0) then h:=0;
    if MOTDecharging and (h<>0) then begin
      h:=0;
      Current:=0;
      writeln('ExpIO : SetMOTCurrent : MOTDecharging',^G);
      delay(200);
    end;
    IOanalogOut(8,round(h));
{    Sound(round(current));}
    aktMOTCurrent:=Current;
  end else SequenceDoubleArg(SetMOTCurrent,Current,0);
end;

procedure SetMOTVoltage(Voltage:double);
var h:double;
begin
  if SequenceOut then begin
    h:=Voltage*65535.0/30.0;
    if (h>65535) then h:=65535;
    if (h<0) then h:=0;
    if MOTDecharging and (h<>0) then begin
      h:=0;
      Voltage:=0;
      writeln('ExpIO : SetMOTVoltage : MOTDecharging',^G);
      delay(200);
    end;
    IOanalogOut(9,round(h));
    aktMOTVoltage:=Voltage;
  end else SequenceDoubleArg(SetMOTVoltage,Voltage,0);
end;

procedure SetTransfer1Current(Current:double);
var h:double;
begin
  if SequenceOut then begin
    h:=Current*65535.0/500.0;
    if (h>65535) then h:=65535;
    if (h<0) then h:=0;
    if TransferDecharging and (h<>0) and (aktTransfer1Voltage<>0) then begin
      h:=0;
      Current:=0;
      writeln('ExpIO : SetTransfer1Current : TransferDecharging',^G);
      delay(200);
    end;
    IOanalogOut(10,round(h));
    aktTransfer1Current:=Current;
  end else SequenceDoubleArg(SetTransfer1Current,Current,0);
end;

procedure SetTransfer2Voltage(Voltage:double);
var h:double;
begin
  if SequenceOut then begin
    h:=Voltage*65535.0/20.0;
    if (h>65535) then h:=65535;
    if (h<0) then h:=0;
    if TransferDecharging and (h<>0) then begin
      h:=0;
      voltage:=0;
      writeln('ExpIO : SetTransfer2Voltage : TransferDecharging',^G);
      delay(200);
    end;
    IOanalogOut(12,round(h));
    aktTransfer2Voltage:=Voltage;
  end else SequenceDoubleArg(SetTransfer2Voltage,Voltage,0);
end;

procedure SetTransfer1Voltage(Voltage:double);
var h:double;
begin
  if SequenceOut then begin
    h:=Voltage*65535.0/20.0;
    if (h>65535) then h:=65535;
    if (h<0) then h:=0;
    if TransferDecharging and (h<>0) and (aktTransfer1Current<>0) then begin
      h:=0;
      voltage:=0;
      writeln('ExpIO : SetTransfer1Voltage : TransferDecharging',^G);
      delay(200);
    end;
    IOanalogOut(14,round(h));
    aktTransfer1Voltage:=Voltage;
  end else SequenceDoubleArg(SetTransfer1Voltage,Voltage,0);
end;

procedure SetTransferCurrent(current:double);
begin
  SetTransfer1Current(Current);
  if current>Transfer2AddVoltageCurrent
  then SetTransfer2Voltage((current-Transfer2AddVoltageCurrent)*CompPinchResistance/1000);
end;

procedure SetBarsCurrent(Current:double);
var h:double;
begin
  if SequenceOut then begin
    h:=Current*65535.0/1000.0;
    if (h>65535) then h:=65535;
    if (h<0) then h:=0;
    IOanalogOut(11,round(h));
    aktBarCurrent:=Current;
  end else SequenceDoubleArg(SetBarsCurrent,Current,0);
end;

procedure SetBarsVoltage(Voltage:double);
var h:double;
begin
  if SequenceOut then begin
    h:=Voltage*65535.0/10.0;
    if (h>65535) then h:=65535;
    if (h<0) then h:=0;
    IOanalogOut(13,round(h));
    aktBarVoltage:=Voltage;
  end else SequenceDoubleArg(SetBarsVoltage,Voltage,0);
end;

procedure SetTransferCurrentControlVoltage(f:double);

  const TransferCalib:array[0..9] of double =(
    2.38579,
    0.12278,
    -0.00178,
    2.47218E-5,
    -2.01748E-7,
    1.00201E-9,
    -3.05669E-12,
    5.57735E-15,
    -5.57214E-18,
    2.34028E-21

    {1.22383,
    0.16591,
    -0.00336,
    5.1035E-5,
    -4.4066E-7,
    2.27508E-9,
    -7.14514E-12,
    1.33537E-14,
    -1.36303E-17,
    5.84161E-21}
  );

var TransferVoltage:double;
begin
  TransferVoltage:=TransferCalib[0]+TransferCalib[1]*f+TransferCalib[2]*f*f+TransferCalib[3]*f*f*f
         +TransferCalib[4]*f*f*f*f+TransferCalib[5]*f*f*f*f*f+TransferCalib[6]*f*f*f*f*f*f
         +TransferCalib[7]*f*f*f*f*f*f*f+TransferCalib[8]*f*f*f*f*f*f*f*f+TransferCalib[9]*f*f*f*f*f*f*f*f*f;
  if (TransferVoltage<15) then begin
    SetTransfer2Voltage(TransferVoltage);
    SetTransfer1Voltage(0);
  end else begin
    SetTransfer2Voltage(15);
    SetTransfer1Voltage(TransferVoltage-15);
  end;
end;

procedure SetMOTCurrentControlVoltage(f:double);

  const MOTCalib:array[0..9] of double =(
    0.684,
    0.03658,
    -4.75585E-4,
    6.51948E-6,
    -5.35784E-8,
    2.69241E-10,
    -8.32912E-13,
    1.54588E-15,
    -1.57723E-18,
    6.79529E-22
  );

begin
  SetMOTVoltage(MOTCalib[0]+MOTCalib[1]*f+MOTCalib[2]*f*f+MOTCalib[3]*f*f*f
         +MOTCalib[4]*f*f*f*f+MOTCalib[5]*f*f*f*f*f+MOTCalib[6]*f*f*f*f*f*f
         +MOTCalib[7]*f*f*f*f*f*f*f+MOTCalib[8]*f*f*f*f*f*f*f*f+MOTCalib[9]*f*f*f*f*f*f*f*f*f);
end;

procedure SetPinchCurrentControlVoltage(f:double);

  const PinchCalib:array[0..9] of double =(
    2.07399,
    0.07087,
    -0.0013,
    1.72837E-5,
    -1.36417E-7,
    6.55667E-10,
    -1.9398E-12,
    3.449E-15,
    -3.3799E-18,
    1.40285E-21
  );

begin
  SetMOTVoltage(PinchCalib[0]+PinchCalib[1]*f+PinchCalib[2]*f*f+PinchCalib[3]*f*f*f
         +PinchCalib[4]*f*f*f*f+PinchCalib[5]*f*f*f*f*f+PinchCalib[6]*f*f*f*f*f*f
         +PinchCalib[7]*f*f*f*f*f*f*f+PinchCalib[8]*f*f*f*f*f*f*f*f+PinchCalib[9]*f*f*f*f*f*f*f*f*f);
end;

procedure SetBarsCurrentControlVoltage(f:double);

  const BarsCalib:array[0..9] of double =(
	  0.5205,
	  0.0178,
	  -1.0387E-4,
	  6.48795E-7,
	  -2.39245E-9,
	  5.33012E-12,
	  -7.19853E-15,
	  5.73103E-18  ,
	  -2.45783E-21,
	  4.33901E-25
  );

begin
  SetBarsVoltage(BarsCalib[0]+BarsCalib[1]*f+BarsCalib[2]*f*f+BarsCalib[3]*f*f*f
         +BarsCalib[4]*f*f*f*f+BarsCalib[5]*f*f*f*f*f+BarsCalib[6]*f*f*f*f*f*f
         +BarsCalib[7]*f*f*f*f*f*f*f+BarsCalib[8]*f*f*f*f*f*f*f*f+BarsCalib[9]*f*f*f*f*f*f*f*f*f);
end;

procedure SetPinchOffsetCurrent(Current:double);
var h:double;
begin
  if SequenceOut then begin
    h:=Current*65535.0/90.0;
    if (h>65535) then h:=65535;
    if (h<0) then h:=0;
    IOanalogOut(25,round(h));
    aktPinchOffsetCurrent:=Current;
  end else SequenceDoubleArg(SetPinchOffsetCurrent,Current,0);
end;

procedure SetFinalSlowerCurrent(Current:double);
var h:double;
begin
  if SequenceOut then begin
    h:=Current*65535.0/50.0;
    if (h>65535) then h:=65535;
    if (h<0) then h:=0;
    IOanalogOut(15,round(h));
    aktFinalSlowerCurrent:=Current;
  end else SequenceDoubleArg(SetFinalSlowerCurrent,Current,0);
end;

procedure SwitchFinalSlowerVoltage(onoff:boolean);
var h:double;
begin
  if SequenceOut then begin
    IOdigitalOut(1,4,onoff);
    if onoff then aktFinalSlowerVoltage:=MaxFinalSlowerVoltage
    else aktFinalSlowerVoltage:=0;
  end else SequenceBooleanArg(SwitchFinalSlowerVoltage,onoff,0);
end;

procedure SetRFSweepCopyFrequency(Synth:byte;Frequency:double);
var h:double;
begin
  if SequenceOut then begin
    h:=frequency*65535.0/2000.0;
    if (h>65535) then h:=65535;
    if (h<0) then h:=0;
    case Synth of
      0:IOanalogOut(28,round(h));
      1:IOanalogOut(28,round(h));
      2:IOanalogOut(28,round(h));
    end;
  end else SequenceByteDoubleArg(SetRFSweepCopyFrequency,Synth,Frequency,0);
end;

procedure Set6LiProbeIntensity(Intensity:double);
var h:double;
begin
  if SequenceOut then begin
    h:=Intensity*65535.0;
    if (h>65535) then h:=65535;
    if (h<0) then h:=0;
    IOanalogOut(17,round(h));
  end else SequenceDoubleArg(Set6LiProbeIntensity,Intensity,0);
end;

procedure Set7LiProbeIntensity(Intensity:double);
var h:double;
begin
  if SequenceOut then begin
    h:=Intensity*65535.0;
    if (h>65535) then h:=65535;
    if (h<0) then h:=0;
    IOanalogOut(18,round(h));
  end else SequenceDoubleArg(Set7LiProbeIntensity,Intensity,0);
end;

procedure SetProbeConfiguration(Angle:double);
var h:double;
begin
  if SequenceOut then begin
    h:=Angle*65535.0;
    if (h>65535) then h:=65535;
    if (h<0) then h:=0;
    IOanalogOut(19,round(h));
  end else SequenceDoubleArg(SetProbeConfiguration,Angle,0);
end;

procedure SetNdYAGIntensity(BeamDir:byte;Intensity:double);
var h:double;
begin
  if SequenceOut then begin
    h:=Intensity*65535.0;
    if (h>65535) then h:=65535;
    if (h<0) then h:=0;
    case BeamDir of
      0:IOanalogOut(29,round(h));
      1:IOanalogOut(32,round(h));
    end;
  end else SequenceByteDoubleArg(SetNdYAGIntensity,BeamDir,Intensity,0);
end;

{analog Inputs}

procedure SaveAktCurrents;
begin
  if SequenceOut then begin
    FinalTrapBarCurrent:=aktbarcurrent;
    FinalTrapMOTcurrent:=aktMOTcurrent;
    FinalTrapTransfercurrent:=aktTransfer1current;
    FinalTrapPinchOffsetCurrent:=aktPinchOffsetCurrent;
  end else SequenceNoArg(SaveAktCurrents,0);
end;

const MaxFluo=10;
const LastFluorescence:array [0..MaxFluo] of double=(0,0,0,0,0,0,0,0,0,0,0);

function GetFluorescence(n:integer):double;
var h:double;
begin
  if n>MaxFluo then begin
    n:=MaxFluo;
    writeln('ExpIO : GetFluorescence : n too big');
  end;
  h:=FluorescenceMax-FluorescenceMin;
  if h=0 then h:=0.0001;
  GetFluorescence:=((LastFluorescence[n]-FluorescenceMin)/h);
end;

function GetLastFluorescence(n:integer):double;
begin
  if n>MaxFluo then begin
    n:=MaxFluo;
    writeln('ExpIO : GetLastFluorescence : n too big');
  end;
  GetLastFluorescence:=LastFluorescence[n];
end;

procedure MeasureFluorescence(n:integer);
begin
  if n>MaxFluo then begin
    n:=MaxFluo;
    writeln('ExpIO : MeasureFluorescence : n too big');
  end;
  if SequenceOut then begin
    LastFluorescence[n]:=IOanalogIn(0);
  end else SequenceIntArg(MeasureFluorescence,n,0);
end;

procedure MeasureIntegratedFluorescence(n:integer);
begin
  if n>MaxFluo then begin
    n:=MaxFluo;
    writeln('ExpIO : MeasureIntegratedFluorescence : n too big');
  end;
  if SequenceOut then begin
    LastFluorescence[n]:=LastFluorescence[n]+IOanalogIn(0);
  end else SequenceIntArg(MeasureIntegratedFluorescence,n,0);
end;

procedure NormalizeIntegratedFluorescence(n:integer;number:double);
begin
  if n>MaxFluo then begin
    n:=MaxFluo;
    writeln('ExpIO : NormalizeIntegratedFluorescence : n too big');
  end;
  if SequenceOut then begin
    LastFluorescence[n]:=LastFluorescence[n]/number;
  end else SequenceIntDoubleArg(NormalizeIntegratedFluorescence,n,number,0);
end;

{digital Inputs}

function NoTemperatureOrWaterError:boolean;
begin
  NoTemperatureOrWaterError:=IODigitalIn(0,0);
end;

function LineTrigger:boolean;
begin
  LineTrigger:=IODigitalIn(0,1);
end;

procedure SynchronizeToLine;
var timeout:word;
begin
  if SequenceOut then begin
    timeout:=0;
    while LineTrigger and (timeout<2000) do begin
      inc(timeout);
      Wait(400*ID+0.15);
    end;
    if timeout>=2000 then begin
      writeln('ExpIO:SynchronizeToLine: Line Trigger always high');
      exit;
    end;
    timeout:=0;
    while not LineTrigger and (timeout<2000) do begin
      inc(timeout);
      Wait(401*ID+0.15);
    end;
    if timeout>=2000 then begin
      writeln('ExpIO:SynchronizeToLine: Line Trigger always low');
      exit;
    end;
  end else SequenceNoArg(SynchronizeToLine,0);
end;

{GPIB Commands}

{adresses GPIB}
const SystemIEEE=21;
      MaxSRS=2;
      SRS:array[0..MaxSRS] of byte=(19,18,17);
      HAMEG8122=7;
      WAVETEK2510=3;
      MaxPulsGen=0;
const PulsGenAdr:array[0..MaxPulsGen] of byte=(4);

procedure SetFrequency(Frequ:double);
var status:integer;
begin
  IEEE.Send(WAVETEK2510,'FRQ 200 MHZ',status);
end;

var olddelaytime,oldpulstime,oldfrequency,oldlevel:array[0..MaxSRS] of double;
    oldwaveform:array[0..MaxSRS] of byte;
    SRSinitialized:array [0..MaxSRS] of boolean;

procedure PrepareSinglePuls(SRSNr:byte;delaytime,pulstime,Resolution,ForceReprogram:double);
type TPoint=record
              x:word;
              y:integer;
            end;
var deltatime:double;
var status:integer;
    buf:string;
    len,timeout,checksum,n:word;
const ndata=5;
var data:array[1..ndata+1] of TPoint;
begin
  if SequenceOut and
   ((olddelaytime[SRSNr]<>delaytime) or (oldpulstime[SRSNr]<>pulstime)
   or (oldwaveform[SRSNr]<>0)
   or (not SRSinitialized[SRSNr]) or (ForceReprogram>0)) then begin
    oldwaveform[SRSNr]:=0;
    SRSinitialized[SRSNr]:=true;
    oldpulstime[SRSNr]:=pulstime;
    olddelaytime[SRSNr]:=delaytime;
    deltatime:=(0.001*resolution)*1000;
    IEEE.Send(SRS[SRSNr],'AMPL 3VP',status);
    IEEE.Send(SRS[SRSNr],'OFFS 1.5',status);
    IEEE.Send(SRS[SRSNr],'FUNC 5',status);
    IEEE.Send(SRS[SRSNr],'INVT 0',status);
{    IEEE.Send(SRS[SRSNr],'PCLR',status);}
    IEEE.Send(SRS[SRSNr],'BCNT 1',status);
{    IEEE.Send(SRS[SRSNr],'MDWF 0',status);}
    IEEE.Send(SRS[SRSNr],'MENA 1',status);
    IEEE.Send(SRS[SRSNr],'MTYP 5',status);
    IEEE.Send(SRS[SRSNr],'TSRC 2',status);
    IEEE.Send(SRS[SRSNr],'AMRT 1',status);
    str(1/(0.001*Resolution),buf);
    IEEE.Send(SRS[SRSNr],'FSMP '+buf,status);
    IEEE.Send(SRS[SRSNr],'LDWF? 1,5',status);
{    Timeout:=0;
    buf:='';
    while ((buf<>'1') and (timeout<300)) do begin
      delay(1);
      timeout:=timeout+1;
      IEEE.enter(buf,1,len,SRS[SRSNr],status);
    end;}
    delay(200);buf:='1';
    if (buf='1') then begin
      data[1].x:=0;
      data[1].y:=-2047;
      data[2].x:=round(delaytime/deltatime);
      data[2].y:=-2047;
      data[3].x:=round(delaytime/deltatime)+1;
      data[3].y:=2047;
      data[4].x:=round((delaytime+pulstime)/deltatime);
      data[4].y:=2047;
      data[5].x:=round((delaytime+pulstime)/deltatime)+1;
      data[5].y:=-2047;
      {data[6].x:=round((delaytime+pulstime+0.05)/deltatime);
      data[6].y:=-2047;}
      checksum:=0;
      {$Q-}
      for n:=1 to ndata do checksum:=checksum+word(data[n].x)+word(data[n].y);
      data[ndata+1].x:=checksum;
      str(SRS[SRSNr],buf);
      IEEE.Transmit('MTA LISTEN '+buf,status);
      IEEE.tarray(data,26,false,status);
    end else begin
      writeln('Could not program arbitrary waveform!',^G);
      delay(1000);
    end;
  end else SequenceByteDoubleDoubleDoubleDoubleArg(PrepareSinglePuls,SRSNr,delaytime,pulstime,Resolution,ForceReprogram,0);
end;

procedure PrepareRFPuls(SRSNr:byte;delaytime,pulstime,Frequency,Level:double);
var deltatime,akttime:double;
var status:integer;
    buf:string;
    len,timeout,n:word;
    checksum:integer;
    Resolution:double;
    ForceReprogram:double;
    aktndata:word;
const ndata=10000;
var data:array[1..ndata+1] of integer;
begin
  Resolution:=0.001;
  ForceReprogram:=0;
  if SequenceOut and
   ((olddelaytime[SRSNr]<>delaytime) or (oldpulstime[SRSNr]<>pulstime)
   or (oldfrequency[SRSNr]<>frequency) or (oldlevel[SRSNr]<>level)
   or (oldwaveform[SRSNr]<>1)
   or (not SRSinitialized[SRSNr]) or (ForceReprogram>0)) then begin
    oldwaveform[SRSNr]:=1;
    SRSinitialized[SRSNr]:=true;
    oldpulstime[SRSNr]:=pulstime;
    olddelaytime[SRSNr]:=delaytime;
    oldlevel[SRSNr]:=level;
    oldfrequency[SRSNr]:=frequency;
    deltatime:=(0.001*resolution)*1000;
    str(Level:5:5,buf);
    IEEE.Send(SRS[SRSNr],'AMPL '+buf+'DB',status);
    IEEE.Send(SRS[SRSNr],'OFFS 0',status);
    IEEE.Send(SRS[SRSNr],'FUNC 0',status);
    str(Frequency*1000000:5:5,buf);
    IEEE.Send(SRS[SRSNr],'FREQ '+buf,status);
    IEEE.Send(SRS[SRSNr],'INVT 0',status);
    IEEE.Send(SRS[SRSNr],'BCNT 1',status);
    IEEE.Send(SRS[SRSNr],'MENA 0',status);
    IEEE.Send(SRS[SRSNr],'MTYP 2',status);
    IEEE.Send(SRS[SRSNr],'TSRC 2',status);
    IEEE.Send(SRS[SRSNr],'MDWF 5',status);
    IEEE.Send(SRS[SRSNr],'AMRT 1',status);
    if ndata*0.0003<(pulstime+delaytime+0.001) then begin
      writeln('Arbitrary AM Modulation too long!',^G);
      delay(1000);
    end else begin
      aktndata:=round((pulstime+delaytime+0.001)/0.0003);
      str(aktndata,buf);
      IEEE.Send(SRS[SRSNr],'AMOD? '+buf,status);
      for n:=1 to aktndata do begin
        akttime:=n*0.003;
        if (akttime<delaytime) or (akttime>(delaytime+pulstime))
        then data[n]:=0
        else data[n]:=32767;
      end;
      checksum:=0;
      {$R-}
      for n:=1 to aktndata do checksum:=checksum+word(data[n]);
      data[aktndata+1]:=integer(checksum);
      delay(200);
      str(SRS[SRSNr],buf);
      IEEE.Transmit('MTA LISTEN '+buf,status);
      IEEE.tarray(data,(aktndata+1)*2,false,status);
    end;
    IEEE.Send(SRS[SRSNr],'MENA 1',status);
  end else SequenceByteDoubleDoubleDoubleDoubleArg(PrepareRFPuls,SRSNr,delaytime,pulstime,Frequency,Level,0);
end;

procedure PrepareRFBurstPuls(SRSNr:byte;BurstCount,Frequency,Level,ForceReprogram:double);
var deltatime:double;
var status:integer;
    buf:string;
    len,timeout,checksum,n:word;
begin
  if SequenceOut then begin
    if ((abs(oldpulstime[SRSNr]-burstcount)>0.1)
      or (abs(oldfrequency[SRSNr]-frequency)>0.000001)
      or (abs(oldlevel[SRSNr]-level)>0.01)
      or (oldwaveform[SRSNr]<>2)
      or (not SRSinitialized[SRSNr]) or (ForceReprogram>0)) then begin
       if oldwaveform[SRSNr]<>2 then begin
         IEEE.Send(SRS[SRSNr],'OFFS 0',status);
         IEEE.Send(SRS[SRSNr],'FUNC 0',status);
         IEEE.Send(SRS[SRSNr],'INVT 0',status);
         IEEE.Send(SRS[SRSNr],'MTYP 5',status);
         IEEE.Send(SRS[SRSNr],'MENA 1',status);
         IEEE.Send(SRS[SRSNr],'TSRC 2',status);
       end;
       oldwaveform[SRSNr]:=2;
       SRSinitialized[SRSNr]:=true;
       oldpulstime[SRSNr]:=burstcount;
       oldlevel[SRSNr]:=level;
       oldfrequency[SRSNr]:=frequency;
       str(Level:5:5,buf);
       IEEE.Send(SRS[SRSNr],'AMPL '+buf+'DB',status);
       str(Frequency*1000000:5:5,buf);
       IEEE.Send(SRS[SRSNr],'FREQ '+buf,status);
       str(BurstCount:5:0,buf);
       IEEE.Send(SRS[SRSNr],'BCNT '+buf,status);
     end
  end else SequenceByteDoubleDoubleDoubleDoubleArg(PrepareRFBurstPuls,SRSNr,burstcount,Frequency,Level,ForceReprogram,0);
end;

procedure StanfordTTL(SRSNr:byte;onoff:boolean);far;
var status:integer;
    buf:string;
begin
  if SequenceOut then begin
    IEEE.Send(SRS[SRSNr],'AMPL 0VP',status);
    SRSinitialized[SRSNr]:=false;
    if onoff then IEEE.Send(SRS[SRSNr],'OFFS 2.5',status)
    else IEEE.Send(SRS[SRSNr],'OFFS 0',status);
  end else SequenceByteBooleanArg(StanfordTTL,SRSNr,onoff,0);
end;

procedure SwitchProbeBeamGPIB(onoff:boolean);
begin
  StanfordTTL(0,onoff);
end;

procedure SRSSinus(SRSNr:byte;Frequency:double;Amplitude,dummy1,dummy2:double);
var status:integer;
    buf:string;
begin
  if Amplitude>1 then begin
    Amplitude:=0;
    writeln('EXPIO : SRSSinus : Amplitude too high',^G);
    delay(1000);
  end;
  if SequenceOut then begin
    IEEE.Send(SRS[SRSNr],'FUNC 0',status); {Sinus}
    str(Frequency:2:6,buf);
    IEEE.Send(SRS[SRSNr],'FREQ '+buf,status);
    str(Amplitude*5:2:2,buf);
    IEEE.Send(SRS[SRSNr],'AMPL '+buf+'VP',status);
    IEEE.Send(SRS[SRSNr],'OFFS 0',status)
  end else SequenceByteDoubleDoubleDoubleDoubleArg(SRSSinus,SRSNr,Frequency,Amplitude,0,0,0);
end;

function GetFrequency:double;
var s:string;
    len:word;
    error,status:integer;
    r:double;
begin
  IEEE.Send(Hameg8122,'FREQ?',status);
  IEEE.Enter(s,50,len,Hameg8122,status);
  s:=copy(s,5,50);
  val(s,r,error);
  GetFrequency:=r;
end;

procedure PulsGenInit(Adr:byte);
var n:byte;
    s:string;
    len:word;
    st:integer;
begin
  PulsgenEcho(PulsGenAdr[Adr],0);
  PulsgenSetMode(PulsGenAdr[Adr],4);
  PulsgenSetTriggerPolarity(PulsGenAdr[Adr],1);
  PulsgenSetNrPulses(PulsGenAdr[Adr],50000);
  PulsgenRun(PulsGenAdr[Adr],1);
  for n:=1 to 4 do PulsgenSetPulsPolarity(PulsGenAdr[Adr],n,1);
end;

procedure PulsGenPrepareChannelDirect(Adr:byte;Channel:byte;Delay:double;Width:double);
begin
  PulsgenSetPulsWidth(PulsGenAdr[Adr],Channel,width*10000);
  PulsgenSetPulsDelay(PulsGenAdr[Adr],Channel,delay*10000);
end;

procedure PulsGenStoreDirect(Adr:byte;slot:byte);
begin
  PulsGen.PulsgenStore(PulsGenAdr[Adr],slot);
end;

procedure PulsGenRecall(Adr:byte;slot:double);
begin
  if SequenceOut then begin
    PulsGen.PulsgenRecall(PulsGenAdr[Adr],round(slot));
  end else SequenceByteDoubleArg(PulsgenRecall,adr,slot,0);
end;

begin
  IEEE.Initialize(SystemIEEE,0);
  olddelaytime[0]:=0;
  oldpulstime[0]:=0;
  SRSinitialized[0]:=false;
  olddelaytime[1]:=0;
  oldpulstime[1]:=0;
  SRSinitialized[1]:=false;
  olddelaytime[2]:=0;
  oldpulstime[2]:=0;
  SRSinitialized[2]:=false;
  SetSequenceWaitProc(DoSequenceWait);
end.