unit IO;

{
 This unit unites all Input and output operations
 The digital ports are projected to virtual 8 Bit ports
 All channels are sorted and get a unique channel number
 The number of IO operations is minimized using buffers
 Data is written out when IOUpdate is invoked or if
 IOdigital/analogIn/OutDirect is used.
 The analog Values go from 0 to $FFFF even for 12bit DACs.
}

interface

const ON=true;
      OFF=false;
      MaxDigOut=9;
      MaxAnaOut=36;
      MaxDigIn=1;
      MaxAnaIn=8;

procedure IOanalogOut(Channel: byte; Value: word);
procedure IOanalogOutDirect(aChannel: byte; aValue: word);
function  IOanalogIn(channel:byte):word;
Procedure IOdigitalOut(channel:byte;bitNR:byte;aValue:boolean);
Procedure IOdigitalOutDirect(achannel:byte;bitNR:byte;aValue:boolean);
Function  IOdigitalIn(channel:byte;bitNR:byte):boolean;
procedure IOupdate; far;
procedure IOupdateDigitalPort(port:byte); far;
procedure IOupdateAnalogPort(port:byte); far;
procedure IOCompleteUpdate;
function IOgetMaxOutBitNr(channel:byte):byte;
function IOgetMaxInBitNr(channel:byte):byte;
procedure SaveStatusBuffer(var buffer:pointer);
procedure RetrieveStatusBuffer(var buffer:pointer);
procedure DeleteStatusBuffer(var buffer:pointer);

implementation
uses crt,DAS1601,DDA06,MultiOut,LPT;

Type TCard=(CDAS1601,CDDA06,CMultiOut,CLPT);

     TDigBuf=record  {projection of the buffers to hardware ports}
       card:TCard;
       channel:byte;
       NrBits:byte;
       value:word;
       update:boolean;
     end;

     TAnaBuf=record  {projection of the buffers to hardware ports}
       card:TCard;
       channel:byte;
       value:word;
       update:boolean;
     end;

const DigBuf:array[0..MaxDigOut-1] of TDigBuf=(
  (card:CMultiOut;channel:0;NrBits:16;value:0;update:true),  {channel 0}
  (card:CMultiOut;channel:1;NrBits:16;value:0;update:true),  {channel 1}
  (card:CMultiOut;channel:2;NrBits:16;value:0;update:true),  {channel 2}
  (card:CMultiOut;channel:3;NrBits:16;value:0;update:true),  {channel 3}
  (card:CDDA06   ;channel:0;NrBits:8;value:0;update:true),   {channel 4}
  (card:CDDA06   ;channel:1;NrBits:8;value:0;update:true),   {channel 5}
  (card:CDDA06   ;channel:2;NrBits:8;value:0;update:true),   {channel 6}
  (card:CDAS1601 ;channel:3;NrBits:4;value:0;update:true),   {channel 7}
  (card:CLPT     ;channel:1;NrBits:8;value:0;update:true)    {channel 8}
);

const AnaBuf:array[0..MaxAnaOut-1] of TAnaBuf=(
  (card:CMultiOut;channel: 0;value:0;update:true),  {channel  0}
  (card:CMultiOut;channel: 1;value:0;update:true),  {channel  1}
  (card:CMultiOut;channel: 2;value:0;update:true),  {channel  2}
  (card:CMultiOut;channel: 3;value:0;update:true),  {channel  3}

  (card:CMultiOut;channel: 4;value:0;update:true),  {channel  4}
  (card:CMultiOut;channel: 5;value:0;update:true),  {channel  5}
  (card:CMultiOut;channel: 6;value:0;update:true),  {channel  6}
  (card:CMultiOut;channel: 7;value:0;update:true),  {channel  7}

  (card:CMultiOut;channel: 8;value:0;update:true),  {channel  8}
  (card:CMultiOut;channel: 9;value:0;update:true),  {channel  9}
  (card:CMultiOut;channel:10;value:0;update:true),  {channel 10}
  (card:CMultiOut;channel:11;value:0;update:true),  {channel 11}

  (card:CMultiOut;channel:12;value:0;update:true),  {channel 12}
  (card:CMultiOut;channel:13;value:0;update:true),  {channel 13}
  (card:CMultiOut;channel:14;value:0;update:true),  {channel 14}
  (card:CMultiOut;channel:15;value:0;update:true),  {channel 15}

  (card:CMultiOut;channel:16;value:0;update:true),  {channel 16}
  (card:CMultiOut;channel:17;value:0;update:true),  {channel 17}
  (card:CMultiOut;channel:18;value:0;update:true),  {channel 18}
  (card:CMultiOut;channel:19;value:0;update:true),  {channel 19}

  (card:CMultiOut;channel:20;value:0;update:true),  {channel 20}
  (card:CMultiOut;channel:21;value:0;update:true),  {channel 21}
  (card:CMultiOut;channel:22;value:0;update:true),  {channel 22}
  (card:CMultiOut;channel:23;value:0;update:true),  {channel 23}

  (card:CMultiOut;channel:24;value:0;update:true),  {channel 24}
  (card:CMultiOut;channel:25;value:0;update:true),  {channel 25}
  (card:CMultiOut;channel:26;value:0;update:true),  {channel 26}
  (card:CMultiOut;channel:27;value:0;update:true),  {channel 27}

  (card:CDDA06   ;channel: 0;value:0;update:true),  {channel 28}
  (card:CDDA06   ;channel: 1;value:0;update:true),  {channel 29}
  (card:CDDA06   ;channel: 2;value:0;update:true),  {channel 30}
  (card:CDDA06   ;channel: 3;value:0;update:true),  {channel 31}
  (card:CDDA06   ;channel: 4;value:0;update:true),  {channel 32}
  (card:CDDA06   ;channel: 5;value:0;update:true),  {channel 33}

  (card:CDAS1601 ;channel: 0;value:0;update:true),  {channel 34}
  (card:CDAS1601 ;channel: 1;value:0;update:true)   {channel 35}
);

procedure IOanalogOut(Channel: byte; Value: word );
begin
  if (channel<MaxAnaOut) then begin
    if AnaBuf[channel].value<>Value then begin
      AnaBuf[channel].update:=true;
      AnaBuf[channel].value:=Value;
    end;
  end else writeln('IO analogout : wrong channel');
end;

procedure IOanalogOutDirect(aChannel: byte; aValue: word );
begin
  if (achannel<MaxAnaOut) then begin
    if AnaBuf[achannel].value<>aValue then with AnaBuf[achannel] do begin
      update:=false;
      value:=aValue;
      case card of
        CDAS1601:DAS1601analogOut(channel,value shr 4);
        CDDA06:  DDA06analogOut(channel,value shr 4);
        CMultiOut:MultiOutanalogOut(channel,value);
      end;
    end;
  end else writeln('IO analogoutdirect : wrong channel');
end;

function IOanalogIn(channel:byte):word;
begin
  IOanalogIn:=DAS1601analogIn(channel) shl 4;
end;

Procedure IOdigitalOut(channel:byte;bitNR:byte;aValue:boolean);
var mask:word;
begin
  if (channel<MaxDigOut) then begin
    if (bitNr<DigBuf[channel].NrBits) then begin
      mask:=1 shl bitNr;
      with DigBuf[channel] do begin
        if aValue then begin
          if (value and mask)=0 then update:=true;
          value:=value or mask;
        end else begin
          if (value and mask)>0 then update:=true;
          value:=value and not mask;
        end;
      end;
    end else writeln('IOdigitalOut : wrong BitNr');
  end else writeln('IOdigitalOut : wrong channel');
end;

Procedure IOdigitalOutDirect(achannel:byte;bitNR:byte;aValue:boolean);
var mask:word;
begin
  if (achannel<MaxDigOut) then begin
    if (bitNr<DigBuf[achannel].NrBits) then begin
      mask:=1 shl bitNr;
      with DigBuf[achannel] do begin
        if aValue then begin
          if (value and mask)=0 then update:=true;
          value:=value or mask;
        end else begin
          if (value and mask)>0 then update:=true;
          value:=value and not mask;
        end;
        if update then begin
          update:=false;
          case card of
            CDAS1601: DAS1601digitalOut(channel,Lo(value));
            CDDA06:   DDA06digitalOut(channel,Lo(value));
            CMultiOut:MultiOutdigitalOut(channel,value);
            CLPT:     LPTdigitalOut(channel,value);
          end;
        end;
      end;
    end else writeln('IOdigitalOut : wrong BitNr');
  end else writeln('IOdigitalOut : wrong channel');
end;

Function  IOdigitalIn(channel:byte;bitNR:byte):boolean;
begin
  IOdigitalIn:=DAS1601digitalIn(bitNR);
end;

procedure IOUpdate;
var n:byte;
begin
  for n:=0 to MaxDigOut-1 do with DigBuf[n] do if update then begin
    update:=false;
    case card of
      CDAS1601: DAS1601digitalOut(channel,Lo(value));
      CDDA06:   DDA06digitalOut(channel,Lo(value));
      CMultiOut:MultiOutdigitalOut(channel,value);
      CLPT:     LPTdigitalOut(channel,value);
    end;
  end;
  for n:=0 to MaxAnaOut-1 do with AnaBuf[n] do if update then begin
    update:=false;
    case card of
      CDAS1601:DAS1601analogOut(channel,value shr 4);
      CDDA06:  DDA06analogOut(channel,value shr 4);
      CMultiOut:MultiOutanalogOut(channel,value);
    end;
  end;
end;

procedure IOupdateDigitalPort(port:byte);
begin
  with DigBuf[port] do if update then begin
    update:=false;
    case card of
      CDAS1601: DAS1601digitalOut(channel,Lo(value));
      CDDA06:   DDA06digitalOut(channel,Lo(value));
      CMultiOut:MultiOutdigitalOut(channel,value);
      CLPT:     LPTdigitalOut(channel,value);
    end;
  end;
end;

procedure IOupdateAnalogPort(port:byte);
begin
  with AnaBuf[port] do if update then begin
    update:=false;
    case card of
      CDAS1601:DAS1601analogOut(channel,value shr 4);
      CDDA06:  DDA06analogOut(channel,value shr 4);
      CMultiOut:MultiOutanalogOut(channel,value);
    end;
  end;
end;

procedure IOCompleteUpdate;
var n:byte;
begin
  for n:=0 to MaxDigOut-1 do with DigBuf[n] do begin
    update:=false;
    case card of
      CDAS1601:DAS1601digitalOut(channel,Lo(value));
      CDDA06:  DDA06digitalOut(channel,Lo(value));
      CMultiOut:MultiOutdigitalOut(channel,value);
    end;
  end;
  for n:=0 to MaxAnaOut-1 do with AnaBuf[n] do begin
    update:=false;
    case card of
      CDAS1601:DAS1601analogOut(channel,value shr 4);
      CDDA06:  DDA06analogOut(channel,value shr 4);
      CMultiOut:MultiOutanalogOut(channel,value);
    end;
  end;
end;

function IOgetMaxOutBitNr(channel:byte):byte;
var h:byte;
begin
  if (channel<MaxDigOut) then h:=DigBuf[channel].NrBits
  else begin
    writeln('IOdigitalOut : wrong channel');
    h:=0;
  end;
  IOgetMaxOutBitNr:=h;
end;

function IOgetMaxInBitNr(channel:byte):byte;
begin
  IOgetMaxInBitNr:=4;
end;

Type TWordArray=array[0..MaxAnaOut+MaxDigOut-1] of word;

procedure SaveStatusBuffer(var buffer:pointer);
var n:byte;
begin
  if MaxAvail<(MaxAnaOut+MaxDigOut)*sizeof(word) then begin
    writeln('IO : SaveStatusBuffer : Out of memory');
    delay(1000);
    buffer:=nil;
    exit;
  end;
  Getmem(buffer,(MaxAnaOut+MaxDigOut)*sizeof(word));
  for n:=0 to MaxAnaOut+MaxDigOut-1 do begin
    if n<MaxDigOut then TWordArray(buffer^)[n]:=DigBuf[n].value
    else TWordArray(buffer^)[n]:=AnaBuf[n-MaxDigOut].value;
  end;
end;

procedure RetrieveStatusBuffer(var buffer:pointer);
var n:byte;
begin
  if buffer<>nil then begin
    for n:=0 to MaxAnaOut+MaxDigOut-1 do begin
      if n<MaxDigOut then DigBuf[n].value:=TWordArray(buffer^)[n]
      else AnaBuf[n-MaxDigOut].value:=TWordArray(buffer^)[n];
    end;
  end else begin
    Writeln('IO : RetrieveStatusBuffer : Empty buffer');
    delay(1000);
  end;
end;

procedure DeleteStatusBuffer(var buffer:pointer);
begin
  if buffer<>nil then begin
    Freemem(buffer,(MaxAnaOut+MaxDigOut)*sizeof(word));
    buffer:=nil;
  end else begin
    Writeln('IO : DeleteStatusBuffer : Empty buffer');
    delay(1000);
  end;
end;

begin
  IOUpdate;
end.