UNIT Menu;
{General-purpose user interface unit}

INTERFACE

CONST
  MaxVar=50;   {Maximum number of variables in a 'Dialog Box'}
  MaxItem=47;  {Maximum number of items in a 'Menu'}

TYPE
  tItemType=(ITdouble,ITInteger,ITString,ITBoolean,ITText,ITMenuChoice);
  PDialogInfo=^TDialogInfo;
  tDialogInfo=RECORD
		nVar:INTEGER;
                iSel:INTEGER;
		pVar:     ARRAY[1..MaxVar] OF POINTER;
		VarType:  ARRAY[1..MaxVar] OF tItemType;
		Desc,Help:ARRAY[1..MaxVar] OF STRING[78];
		LoBd,HiBd:ARRAY[1..MaxVar] OF DOUBLE;
                ScaleFact:ARRAY[1..MaxVar] OF DOUBLE;
		Backup:   ARRAY[1..MaxVar] OF DOUBLE;
		SBackup:  ARRAY[1..MaxVar] OF STRING[15];
                Color:    ARRAY[1..MaxVar] OF BYTE;
	      END;
  PMenuInfo=^TMenuInfo;
  tMenuInfo=RECORD
              nItem:INTEGER;
              iSel:INTEGER;
              ItemType: ARRAY[1..MaxItem] OF tItemType;
              Desc,Help:ARRAY[1..MaxItem] OF STRING[78];
              Key:      ARRAY[1..MaxItem] OF CHAR;
              Color:    ARRAY[1..MaxItem] OF BYTE;
            END;


{ ---------------------------------------------------------------------------- }
{ |                       Easy String I/O                                    | }
{ ---------------------------------------------------------------------------- }

PROCEDURE WriteMessage(Title,Message:STRING);
{Write a message on the screen}

FUNCTION GetString(Title:STRING):STRING;
{Prompt user for a string}

type TIdleProc=function(action:boolean):boolean;

PROCEDURE SetIdleProc(aIdleProc:TIdleProc);

{ ---------------------------------------------------------------------------- }
{ |                    User interface: Dialog                                | }
{ ---------------------------------------------------------------------------- }

{A "Dialog Box", in the restricted sense in which it is used in this program,
 is just a collection of variables that can be changed by the user. Each
 variable has a description text and a help text associated to it. Numeric
 variables have upper and lower bounds.}
{To use a dialog box, you must have a structure of type tDialogInfo defined.
 First call NewDialog to initialize this structure. Then add
 variables with the Add... procedures. Once you've done that, you can call
 DoDialog as often as you want to bring the dialog box to the screen.}

PROCEDURE NewDialog(VAR DialogInfo:PDialogInfo);
{Initialize a DialogInfo data structure}

PROCEDURE AddR(aColor:byte;VAR DialogInfo:PDialogInfo;VAR Vari:DOUBLE;Description,
              HelpText:STRING;LoBound,HiBound:DOUBLE;Scale:DOUBLE);
{Add a DOUBLE variable to a dialog structure. The value of this variable is
 multiplied by Scale before being displayed, the user's new value is divided
 by Scale before being stored in the variable}

PROCEDURE AddI(aColor:byte;VAR DialogInfo:PDialogInfo;VAR Vari:INTEGER;Description,
               HelpText:STRING;LoBound,HiBound:DOUBLE);
{Add an INTEGER variable to a dialog structure}

PROCEDURE AddS(aColor:byte;VAR DialogInfo:PDialogInfo;VAR Vari:STRING;Description,HelpText:STRING);
{Dto. for a String}
PROCEDURE AddB(aColor:byte;VAR DialogInfo:PDialogInfo;VAR Vari:BOOLEAN;Description,HelpText:STRING);
{Dto. for a Boolean}
PROCEDURE AddT(aColor:byte;VAR DialogInfo:PDialogInfo;Description:STRING);
{Add a text line (without variable)}

PROCEDURE BackupVars(VAR DialogInfo:PDialogInfo);
{Back up the variables}

PROCEDURE DoDialog(VAR DialogInfo:PDialogInfo;Title:STRING;
		   VAR UpdateFlag:BOOLEAN);
{Let the user change values until he choses either to quit or to update.
 After a call to DoDialog, the variables will contain the new values in
 case the user has chosen "Update", or the old variables otherwise.}

PROCEDURE RestoreOld(DialogInfo:PDialogInfo);
{With this procedure, the old values can be restored even after an update,
 provided they had been backed up with BackupVars.}

{ ---------------------------------------------------------------------------- }
{ |                    User interface: Menu                                  | }
{ ---------------------------------------------------------------------------- }

{Menus work much like Dialog Boxes, except that there are no variables to be
 changed. Rather, the user selects one command line and DoMenu returns this
 line's ID code. Lines will appear on the screen in the order in which they
 have been defined}
PROCEDURE NewMenu(VAR MenuInfo:PMenuInfo);

PROCEDURE AddChoice(aColor:byte;VAR MenuInfo:PMenuInfo;Description,HelpText:STRING;
                    Short:CHAR;VAR ID:INTEGER);
{Add a "command" to the menu structure and return the ID that has been
 assigned to it. Short is a one-character abbreviation the user can type to
 select a command}

PROCEDURE AddText(aColor:byte;VAR MenuInfo:PMenuInfo;Description:STRING;VAR ID:INTEGER);

PROCEDURE DoMenu(VAR MenuInfo:PMenuInfo;Title:STRING;VAR Choice:INTEGER);
{The user can either select an entry in the menu, in which case his choice
 is returned in the variable Choice, or he can use ESC to leave the menu,
 in which case DoMenu returns Choice=0.}


PROCEDURE CursorOff;
PROCEDURE CursorOn;
procedure HighRes;
procedure LowRes;
function ScreenMaxY:byte;

{==============================================================================}


{ ---------------------------------------------------------------------------- }
{ |                         Implementation part                              | }
{ ---------------------------------------------------------------------------- }

IMPLEMENTATION

USES CRT,dos;

const MainBack=blue;
      MainText=yellow+;
      MarkText=lightred;
      BarBack=black;
      BarText=white;
      helpback=green;
      helptext=blue;
      textback=blue;
      texttext=lightgreen;
      endback=blue;
      endtext=yellow;
      boxback=black;
      boxtext=lightred;
      stringback=lightred;
      stringtext=blue;

CONST
  CPFrac=1;
  ppxborder=10;
  ppyborder=10;
  DisplaySigInfo=TRUE; {Display the SigInfo parameters in Continuous mode}
  ESC=#$1B;
  CR=#$0D;
  LF=#$0A;

const IdleProc:TIdleProc=NIL;
      IdleProcSet:boolean=false;

PROCEDURE CursorOff;
var regs:registers;
begin
  with regs do begin
    ah:=$01;cx:=$AAAA;intr($10,regs);
  end;
end;

PROCEDURE CursorOn;
var regs:registers;
begin
  with regs do begin
    ah:=$01;cx:=$0A0C;intr($10,regs);
  end;
end;


PROCEDURE SetIdleProc(aIdleProc:TIdleProc);
begin
  IdleProc:=aIdleProc;
  IdleProcSet:=true;
end;


{ ---------------------------------------------------------------------------- }
{ |                       Easy String I/O                                    | }
{ ---------------------------------------------------------------------------- }

PROCEDURE WriteMessage{(Title,Message:STRING)};
TYPE
  String78=STRING[78];
VAR
  i,j,l:integer;
  WLine:ARRAY[1..8] OF String78;
BEGIN
  {Separate Message into lines}
  l:=1;
  j:=0;
  FOR i:=1 TO Length(Message) DO BEGIN
    IF Message[i]<>'|' THEN BEGIN
      j:=j+1;
      WLine[l,j]:=Message[i];
    END ELSE BEGIN
      WLine[l,0]:=Chr(j);
      l:=l+1;
      j:=0;
    END;
  END;
  WLine[l,0]:=Chr(j);
  Window(2,10,79,10+l+3);
  textcolor(boxtext);
  textbackground(boxback);
  clrscr;
  write('Ŀ');
  for i:=2 to 3+l do begin
    gotoxy(1,i);write('');
    gotoxy(77,i);write('');
  end;
  gotoxy(1,4+l);
  write('');
  GotoXY(40-Length(Title) DIV 2,1);
  Write(Title);
  FOR i:=1 TO l DO BEGIN
    gotoxy(40-(Length(WLine[i]) DIV 2),2+i);
    write(WLine[i]);
  END;
  window(1,1,80,25);
END;

FUNCTION GetString(Title:STRING):STRING;
{Prompt user for a string}
CONST
  l=2;
VAR
  i,j:integer;
  s:STRING;
BEGIN
  Window(2,10,79,10+l+3);
  textcolor(stringtext);
  textbackground(stringback);
  clrscr;
  write('Ŀ');
  for i:=2 to 3+l do begin
    gotoxy(1,i);write('');
    gotoxy(78,i);write('');
  end;
  gotoxy(1,4+l);
  write('');
  GotoXY(40-Length(Title) DIV 2,1);
  Write(Title);
  GotoXY(3,3);
  ReadLn(s);
  GetString:=s;
  window(1,1,80,25);
END;


{ --------------------------------------------------------------------------- }
{ |                       User interface: Setup & Menu                      | }
{ --------------------------------------------------------------------------- }

PROCEDURE NewDialog(VAR DialogInfo:PDialogInfo);
BEGIN
  new(DialogInfo);
  DialogInfo^.nVar:=0;
  DialogInfo^.iSel:=0;
END;

PROCEDURE AddR;
BEGIN
  WITH DialogInfo^ DO BEGIN
    nVar:=nVar+1;
    pVar[nVar]:=@Vari;
    VarType[nVar]:=ITDOUBLE;
    Desc[nVar]:=Description;
    Help[nVar]:=HelpText;
    LoBd[nVar]:=LoBound;
    HiBd[nVar]:=HiBound;
    Color[nVar]:=aColor;
    ScaleFact[nVar]:=Scale;
  END;
END;

PROCEDURE AddI;
BEGIN
  WITH DialogInfo^ DO BEGIN
    nVar:=nVar+1;
    pVar[nVar]:=@Vari;
    VarType[nVar]:=ITInteger;
    Desc[nVar]:=Description;
    Help[nVar]:=HelpText;
    LoBd[nVar]:=LoBound;
    HiBd[nVar]:=HiBound;
    Color[nVar]:=aColor;
  END;
END;

PROCEDURE AddS;
BEGIN
  WITH DialogInfo^ DO BEGIN
    nVar:=nVar+1;
    pVar[nVar]:=@Vari;
    VarType[nVar]:=ITString;
    Desc[nVar]:=Description;
    Help[nVar]:=HelpText;
    Color[nVar]:=aColor;
  END;
END;

PROCEDURE AddB;
BEGIN
  WITH DialogInfo^ DO BEGIN
    nVar:=nVar+1;
    pVar[nVar]:=@Vari;
    VarType[nVar]:=ITBoolean;
    Desc[nVar]:=Description;
    Help[nVar]:=HelpText;
    Color[nVar]:=aColor;
  END;
END;

PROCEDURE AddT;
BEGIN
  WITH DialogInfo^ DO BEGIN
    nVar:=nVar+1;
    VarType[nVar]:=ITText;
    Desc[nVar]:=Description;
    Color[nVar]:=aColor;
  END;
END;

PROCEDURE BackupVars{(VAR DialogInfo:PDialogInfo)};
VAR
  i:INTEGER;
BEGIN
  WITH DialogInfo^ DO BEGIN
    FOR i:=1 TO nVar DO BEGIN
      CASE VarType[i] OF
	ITDOUBLE:    Backup[i]:=DOUBLE(pVar[i]^);
	ITInteger: Backup[i]:=INTEGER(pVar[i]^);
	ITString:  SBackup[i]:=STRING(pVar[i]^);
        ITBoolean: IF BOOLEAN(pVar[i]^) THEN
                     Backup[i]:=1
                   ELSE
                     Backup[i]:=0;
      END;
    END;
  END;
END;

PROCEDURE RestoreOld{(DialogInfo:PDialogInfo)};
VAR
  i:INTEGER;
BEGIN
  WITH DialogInfo^ DO BEGIN
    FOR i:=1 TO nVar DO BEGIN
      CASE VarType[i] OF
	ITdouble:  DOUBLE(pVar[i]^):=Backup[i];
	ITInteger: INTEGER(pVar[i]^):=Round(Backup[i]);
	ITString:  STRING(pVar[i]^):=SBackup[i];
        ITBoolean: BOOLEAN(pVar[i]^):=(Backup[i]=1);
      END;
    END;
  END;
END;

PROCEDURE DoDialog{(VAR DialogInfo:PDialogInfo;Title:STRING;
		   VAR UpdateFlag:BOOLEAN)};
CONST
  help0=' '+chr(24)+' , '+chr(25)+' , <Enter> to change value, U(pdate, Q(uit without updating';
  TitleLine=1;
  FirstLine=2;
  FirstCol=3;
  LastCol=78;
  VarCol=60;
VAR
  helpline:INTEGER;
  i,iCurrentVar,iOldVar:INTEGER;
  Ant:CHAR;
  Action:BOOLEAN;

  PROCEDURE WriteVar(Line,VarNo:INTEGER);
  BEGIN
    GotoXY(VarCol,Line);
    Write('                   ');
    GotoXY(VarCol,Line);
    WITH DialogInfo^ DO BEGIN
      CASE VarType[VarNo] OF
        ITDOUBLE:   Write(ScaleFact[VarNo]*DOUBLE(pVar[VarNo]^):18:3);
        ITInteger:Write(Integer(pVar[VarNo]^):18);
        ITString: BEGIN
                    IF Length(String(pVar[VarNo]^))<(LastCol-VarCol+1) THEN
                      GotoXY(LastCol-Length(String(pVar[VarNo]^)),Line);
                    Write(String(pVar[VarNo]^));
                  END;
        ITBoolean:IF Boolean(pVar[VarNo]^) THEN BEGIN
                    GotoXY(LastCol-3,Line);
                    Write('Yes')
                  END ELSE BEGIN
                    GotoXY(LastCol-3,Line);
                    Write('No ');
                  END;
      END;
    END;
  END;

  PROCEDURE VarSelect(VAR DI:PDialogInfo;VAR iOld:INTEGER;iNew:INTEGER);
  BEGIN
    IF DI^.VarType[iNew]<>ITText THEN BEGIN
      GotoXY(FirstCol,HelpLine);
      TextBackground(helpback);
      TextColor(helptext);
      write('                                                                            ');
      GotoXY(FirstCol,HelpLine);
      Write(DialogInfo^.help[iNew]);
    END;
    TextBackground(mainback);{?}
    IF DialogInfo^.VarType[iOld]=ITText THEN
      TextColor(texttext)
    ELSE
      TextColor(DialogInfo^.Color[iOld]);
    GotoXY(FirstCol,FirstLine+iOld-1);
    Write('                                                                            ');
    GotoXY(5,iOld+FirstLine-1);
    Write(DialogInfo^.Desc[iOld]);
    WriteVar(FirstLine+iOld-1,iOld);
    iOld:=iNew;
    TextBackground(barback);TextColor(bartext);{?}
    GotoXY(FirstCol,FirstLine+iNew-1);
    write('                                                                            ');
    GotoXY(5,FirstLine+iNew-1);
    write(DialogInfo^.Desc[iNew]);
    WriteVar(FirstLine+iNew-1,iNew);
    GotoXY(80,25);
  END;

  PROCEDURE ReadVar(VarNo:integer);
  VAR
    Buf:STRING;
    rBuf:DOUBLE;
    iBuf:INTEGER;
    Err:INTEGER;
    Error:BOOLEAN;
  BEGIN
    GotoXY(FirstCol,HelpLine);
    TextBackground(helpback);
    TextColor(helptext);
    write('                                                                            ');
    GotoXY(FirstCol,HelpLine);
    Write(DialogInfo^.help[VarNo]);
    REPEAT
      TextBackground(barback);{??}
      TextColor(bartext);
      Error:=FALSE;
      GotoXY(VarCol,FirstLine+VarNo-1);Write('                   ');
      GotoXY(VarCol,FirstLine+VarNo-1);
      CursorOn;
      ReadLn(Buf);
      CursorOff;
      IF Buf<>'' THEN BEGIN
	CASE DialogInfo^.VarType[VarNo] OF
	  ITDOUBLE:
	    BEGIN
	      Val(Buf,rBuf,Err);
              IF Err=0 THEN rBuf:=rBuf/DialogInfo^.ScaleFact[VarNo];
	      IF (Err<>0) OR (rBuf<DialogInfo^.LoBd[VarNo]) OR
			     (rBuf>DialogInfo^.HiBd[VarNo]) THEN BEGIN
		Write(Chr(7));
		Error:=TRUE;
	      END ELSE BEGIN
		DOUBLE(DialogInfo^.PVar[VarNo]^):=rBuf
	      END;
	    END;
	  ITInteger:
	    BEGIN
	      Val(Buf,iBuf,Err);
	      IF (Err<>0) OR (iBuf<DialogInfo^.LoBd[VarNo]) OR
			     (iBuf>DialogInfo^.HiBd[VarNo]) THEN BEGIN
		Write(Chr(7));
		Error:=TRUE;
	      END ELSE BEGIN
		INTEGER(DialogInfo^.PVar[VarNo]^):=iBuf
	      END;
	    END;
	  ITString:
	    BEGIN
	      STRING(DialogInfo^.PVar[VarNo]^):=Buf
	    END;
          ITBoolean:
            BEGIN
              IF Buf[1] IN ['O','o','1','y','Y','s','S','j','J'] THEN
                BOOLEAN(DialogInfo^.PVar[VarNo]^):=TRUE
              ELSE IF Buf[1] IN ['N','n','0'] THEN
                BOOLEAN(DialogInfo^.PVar[VarNo]^):=FALSE
              ELSE BEGIN
		Write(Chr(7));
                Error:=TRUE;
              END;
            END;
	END;
      END;
    UNTIL NOT Error;
    WriteVar(FirstLine+VarNo-1,VarNo);
    GotoXY(FirstCol,HelpLine);
    TextBackground(helpback);
    TextColor(helptext);
    Write('                                                                            ');
    GotoXY(FirstCol,HelpLine);
    Write(Help0);
  END;

  procedure DrawDialog;
  var i:integer;
  begin
    HelpLine:=FirstLine+DialogInfo^.nVar;
    iOldVar:=DialogInfo^.nVar;
    TextBackground(mainback);
    TextColor(maintext);
    clrscr;
    GotoXY(1,TitleLine);
    write(' Ŀ');
    for i:=TitleLine+1 to HelpLine do begin
      GotoXY(2,i);write('');
      GotoXY(79,i);write('');
    end;
    GotoXY(1,HelpLine+1);
    write(' ');
    GotoXY(40-Length(Title) DIV 2,TitleLine);
    Write(Title);
    for i:=DialogInfo^.nVar downto 1 do begin
      VarSelect(DialogInfo,iOldVar,i);
    end;
    iCurrentVar:=1;
    WHILE DialogInfo^.VarType[iCurrentVar]=ITText DO Inc(iCurrentVar);
    VarSelect(DialogInfo,iOldVar,iCurrentVar);
    GotoXY(FirstCol,HelpLine);
    TextBackground(helpback);
    TextColor(helptext);
    write('                                                                            ');
    GotoXY(FirstCol,HelpLine);
    write(help0);
  end;

BEGIN
  if DialogInfo^.nVar>22 then HighRes;
  CursorOff;
  DrawDialog;
  Action:=true;
  repeat
    repeat
      if IdleProcSet then if IdleProc(Action) then DrawDialog;
      Action:=false;
    until keypressed;
    Action:=true;
    ant:=UpCase(ReadKey);
    if (ant=chr(0)) or (ant in ['8','3','9','2']) then begin
      if (ant=chr(0)) then ant:=readkey;
      case ant of
	'8',chr(72):begin {up-arrow}
                  REPEAT
		    iCurrentVar:=iCurrentVar-1;
		    if iCurrentVar<1 then iCurrentVar:=DialogInfo^.nVar;
                  UNTIL DialogInfo^.VarType[iCurrentVar]<>ITText;
		  VarSelect(DialogInfo,iOldVar,iCurrentVar);
		end;
	'2',chr(80):begin {down-arrow}
                  REPEAT
		    iCurrentVar:=iCurrentVar+1;
  		    if iCurrentVar>DialogInfo^.nVar then iCurrentVar:=1;
                  UNTIL DialogInfo^.VarType[iCurrentVar]<>ITText;
	          VarSelect(DialogInfo,iOldVar,iCurrentVar);
		end;
        '9',chr(73):begin {page-up}
                  REPEAT
		    iCurrentVar:=iCurrentVar-5;
		    {if iCurrentVar<1 then iCurrentVar:=DialogInfo^.nVar+iCurrentVar;}
                    if iCurrentVar<1 then iCurrentVar:=DialogInfo^.nVar;
                  UNTIL DialogInfo^.VarType[iCurrentVar]<>ITText;
		  VarSelect(DialogInfo,iOldVar,iCurrentVar);
		end;
	'3',chr(81):begin {page-down}
                  REPEAT
		    iCurrentVar:=iCurrentVar+5;
  		    {if iCurrentVar>DialogInfo^.nVar then iCurrentVar:=0+iCurrentVar-DialogInfo^.nVar;}
                    if iCurrentVar>DialogInfo^.nVar then iCurrentVar:=1;
                  UNTIL DialogInfo^.VarType[iCurrentVar]<>ITText;
	          VarSelect(DialogInfo,iOldVar,iCurrentVar);
		end;
      end;
      ant:='*';
    end else begin
      if ant=CR then begin
	ReadVar(iCurrentVar);
      end;
    end;
  until Ant in [ESC,'Q','U','+','-'];
  UpdateFlag:=(Ant='U') or (Ant='+');
  IF NOT UpdateFlag THEN RestoreOld(DialogInfo);
  if DialogInfo^.nVar>22 then LowRes;
  CursorOn;
  dispose(DialogInfo);
END;


PROCEDURE NewMenu(VAR MenuInfo:PMenuInfo);
BEGIN
  new(MenuInfo);
  MenuInfo^.nItem:=0;
  MenuInfo^.iSel:=0;
END;

PROCEDURE AddChoice;
BEGIN
  WITH MenuInfo^ DO BEGIN
    if nItem=MaxItem then begin
      writeln('Menu: too many items');
      delay(1000);
      exit;
    end;
    Inc(nItem);
    ItemType[nItem]:=ITMenuChoice;
    Desc[nItem]:=Description;
    Help[nItem]:=HelpText;
    Key[nItem]:=Short;
    ID:=nItem;
    Color[nItem]:=aColor;
  END;
END;

PROCEDURE AddText;
BEGIN
  WITH MenuInfo^ DO BEGIN
    Inc(nItem);
    ItemType[nItem]:=ITText;
    Desc[nItem]:=Description;
    ID:=nItem;
    Color[nItem]:=aColor;
  END;
END;


PROCEDURE DoMenu(VAR MenuInfo:PMenuInfo;Title:STRING;VAR Choice:INTEGER);
CONST
  help0=' '+chr(24)+' , '+chr(25)+
     ' , <Enter> to select, <ESC> to quit';
  TitleLine=1;
  FirstLine=2;
  FirstCol=3;
  LastCol=78;
VAR
  HelpLine:INTEGER;
  iOldItem,i:INTEGER;
  Ant:CHAR;
  Select:BOOLEAN;
  Action:BOOLEAN;

  PROCEDURE ItemSelect(MenuInfo:PMenuInfo;VAR iOld:INTEGER;iNew:INTEGER);
  BEGIN
    IF MenuInfo^.ItemType[iNew]<>ITText THEN BEGIN
      GotoXY(FirstCol,HelpLine);
      TextBackground(helpback);
      TextColor(helptext);
      write('                                                                            ');
      GotoXY(FirstCol,HelpLine);
      Write(MenuInfo^.help[iNew]);
    END;
    TextBackground(mainBack);
    IF MenuInfo^.ItemType[iOld]=ITText THEN
      TextColor(texttext)
    ELSE
      TextColor(DialogInfo^.Color[iOld]);
    GotoXY(FirstCol,FirstLine-1+iOld);
    Write('                                                                            ');
    GotoXY(5,FirstLine-1+iOld);
    Write(MenuInfo^.Desc[iOld]);
    iOld:=iNew;
    TextBackground(barback);
    TextColor(bartext);
    GotoXY(FirstCol,FirstLine-1+iNew);
    write('                                                                            ');
    GotoXY(5,FirstLine-1+iNew);
    write(MenuInfo^.Desc[iNew]);
    GotoXY(80,80);
  END;

  procedure DrawMenu;
  var i:integer;
  begin
    HelpLine:=FirstLine+MenuInfo^.nItem;
    TextBackground(mainback);
    TextColor(maintext);
    clrscr;
    GotoXY(1,TitleLine);
    write(' Ŀ');
    for i:=TitleLine+1 to HelpLine do begin
      GotoXY(2,i);write('');
      GotoXY(79,i);write('');
    end;
    GotoXY(1,HelpLine+1);
    write(' ');
    GotoXY(40-Length(Title) DIV 2,TitleLine);
    Write(Title);
    iOldItem:=MenuInfo^.nItem;
    for i:=MenuInfo^.nItem downto 1 do begin
      ItemSelect(MenuInfo,iOldItem,i);
    end;
    IF MenuInfo^.iSel<=0 THEN BEGIN
      MenuInfo^.iSel:=1;
      WHILE MenuInfo^.ItemType[MenuInfo^.iSel]=ITText DO Inc(MenuInfo^.iSel);
    END;
    ItemSelect(MenuInfo,iOldItem,MenuInfo^.iSel);
    GotoXY(FirstCol,HelpLine);
    TextBackground(helpback);
    TextColor(helptext);
    write('                                                                            ');
    GotoXY(FirstCol,HelpLine);
    write(help0);
    Select:=FALSE;
  end;

BEGIN
  if MenuInfo^.nItem>22 then HighRes;
  CursorOff;
  DrawMenu;
  Action:=true;
  repeat
    repeat
      if IdleProcSet then if IdleProc(Action) then DrawMenu;
      Action:=false;
    until keypressed;
    Action:=true;
    ant:=UpCase(ReadKey);
    if (ant=chr(0)) or (ant in ['8','3','9','2']) then begin
      if (ant=chr(0)) then ant:=readkey;
      case ant of
	'8',chr(72):begin {up-arrow}
                  REPEAT
		    MenuInfo^.iSel:=MenuInfo^.iSel-1;
                    if MenuInfo^.iSel<1 then MenuInfo^.iSel:=MenuInfo^.nItem;
                  UNTIL MenuInfo^.ItemType[MenuInfo^.iSel]<>ITText;
		  ItemSelect(MenuInfo,iOldItem,MenuInfo^.iSel);
		end;
	'2',chr(80):begin {down-arrow}
                  REPEAT
		    MenuInfo^.iSel:=MenuInfo^.iSel+1;
                    if MenuInfo^.iSel>MenuInfo^.nItem then MenuInfo^.iSel:=1;
                  UNTIL MenuInfo^.ItemType[MenuInfo^.iSel]<>ITText;
		  ItemSelect(MenuInfo,iOldItem,MenuInfo^.iSel);
		end;
        '9',chr(73):begin {page-up}
                  REPEAT
		    MenuInfo^.iSel:=MenuInfo^.iSel-5;
                    {if MenuInfo^.iSel<1 then MenuInfo^.iSel:=MenuInfo^.nItem+MenuInfo^.iSel;}
		    if MenuInfo^.iSel<1 then MenuInfo^.iSel:=MenuInfo^.nItem;
                  UNTIL MenuInfo^.ItemType[MenuInfo^.iSel]<>ITText;
		  ItemSelect(MenuInfo,iOldItem,MenuInfo^.iSel);
		end;
	'3',chr(81):begin {page-down}
                  REPEAT
		    MenuInfo^.iSel:=MenuInfo^.iSel+5;
                    {if MenuInfo^.iSel>MenuInfo^.nItem then MenuInfo^.iSel:=0+MenuInfo^.iSel-MenuInfo^.nItem;}
  		    if MenuInfo^.iSel>MenuInfo^.nItem then MenuInfo^.iSel:=1;
                  UNTIL MenuInfo^.ItemType[MenuInfo^.iSel]<>ITText;
		  ItemSelect(MenuInfo,iOldItem,MenuInfo^.iSel);
		end;
      end;
      ant:='*';
    end else begin
      if ant=CR then begin
	Choice:=MenuInfo^.iSel;
        Select:=TRUE;
      end ELSE
        FOR i:=1 TO MenuInfo^.nItem DO
          IF Ant=MenuInfo^.Key[i] THEN
            IF MenuInfo^.ItemType[i]=ITMenuChoice THEN BEGIN
              Select:=TRUE;
              Choice:=i;
              MenuInfo^.iSel:=Choice;
            END;
    end;
  until Select OR (Ant=ESC) or (Ant='-') or (Ant='+');
  IF NOT Select THEN Choice:=0;
  if MenuInfo^.nItem>22 then LowRes;
  CursorOn;
END;

var oldexitproc:pointer;
    OrigMode: Integer;
const HighResMode:boolean=false;

procedure myexitproc;far;
begin
  exitproc:=oldexitproc;
  textcolor(endtext);
  textbackground(endback);
  TextMode(OrigMode);
  clrscr;
end;

procedure HighRes;
begin
  TextMode(C80 + Font8x8);
  HighResMode:=true;
end;

procedure LowRes;
begin
  TextMode(CO80);
  HighResMode:=false;
end;

function ScreenMaxY:byte;
begin
  if HighResMode then ScreenMaxY:=50 else ScreenMaxY:=25;
end;

BEGIN
  OrigMode := LastMode;

  oldexitproc:=exitproc;
  exitproc:=@myExitProc;
END.
