unit NDYAG;
interface
uses Serial;

procedure NdYAGSetComPort(aPort:byte);

function NdYAGReadDiodeCurrent(diodeNr:byte):double;
function NdYAGReadStatusHistoryArray:string;
function NdYAGReadDiodeTemperature(diodeNr:byte):double;
function NdYAGReadSoftwareSerialNumber:string;
function NdYAGReadOutputPower:double;
function NdYAGReadDiodeInformation(diodeNr:byte;InformationNr:byte):string;

procedure NdYAGReboot;
procedure NdYAGSetDiodeCurrent(diodeNr:byte;Current:double); {in mA}
procedure NdYAGQueueDiodeOnOffEvent(OnOff:boolean);
procedure NdYAGSetLaserPower(power:double); {in mW}
procedure NdYAGSetDiodeTemperature(DiodeNr:byte;Temperature:double);
Type TDiodePumpRegulationMode=(CurrentFeedback,LaserPowerFeedback,DiodePowerFeedback,CurrentFeedbackStandbySetting);
const DiodePumpRegulationMode:TDiodePumpRegulationMode=CurrentFeedback;
procedure NdYAGSetDiodePumpRegulationMode(Mode:TDiodePumpRegulationMode);

implementation
uses crt;
var Port:byte;
const initialized:boolean=false;

procedure PutLine(Pt:byte;s:string);
begin
  ClearInQueue(Pt);
  Serial.Putline(Pt,s+char(#13));
end;

procedure NdYAGSetComPort(aPort:byte);
begin
  if not initialized then begin
    Port:=aPort;
    {InitCOM(Port);
    SetProtocol (Port,9600,8,1,Pnone,true,true);
    SetTimeOut (Port,1000);}
    OpenPort(Port,9600);
  end;
  initialized:=true;
  Putline(1,char(#13));
end;

function NdYAGReadDiodeCurrent(diodeNr:byte):double;
var buf:string;
    help:double;
    Error:integer;
begin
  if diodeNr>2 then diodeNr:=2;
  if diodeNr<1 then diodeNr:=1;
  buf:='?C'+char(diodeNr+byte('0'))+';';
  PutLine(Port,buf);
  {delay(100);
  writeln('Current');
  PrintBuffer(1);}
  buf:=GetLine(Port,';',255);
  if buf[length(buf)]='A' then buf[0]:=char(length(buf)-1);
  val(buf,help,Error);
  NdYAGReadDiodeCurrent:=help;
end;

function NdYAGReadStatusHistoryArray:string;
var buf2,buf1,buf:string;
    n:byte;

  function ToHex(b:byte):char;
  begin
    if b<10 then ToHex:=char(b+byte('0'))
    else ToHex:=char(b-10+byte('A'));
  end;

begin
  PutLine(Port,'?H;');
  buf1:=GetLine(Port,';',17);
  buf:='';
  for n:=1 to length(buf1) do begin
    str(byte(buf1[n]):4,buf2);
    buf:=buf+buf2;
  end;

  {for n:=1 to length(buf1) do begin
    buf[3*n-2]:=char(ToHex(byte(buf[n]) shr 4));
    buf[3*n-1]:=char(ToHex(byte(buf[n]) and $F));
    buf[3*n]:=' ';
  end;}
  {buf[0]:=char(3*byte(buf1[0])-1);}
  NdYAGReadStatusHistoryArray:=buf;
end;

function NdYAGReadDiodeTemperature(diodeNr:byte):double;
var buf:string;
    help:double;
    Error:integer;
begin
  if diodeNr>2 then diodeNr:=2;
  if diodeNr<1 then diodeNr:=1;
  buf:='?T'+char(diodeNr+byte('0'))+';';
  PutLine(Port,buf);
  buf:=GetLine(Port,';',255);
  if buf[length(buf)]='C' then buf[0]:=char(length(buf)-1);
  val(buf,help,Error);
  NdYAGReadDiodeTemperature:=help;
end;

function NdYAGReadSoftwareSerialNumber:string;
begin
  PutLine(Port,'?V;');
  NdYAGReadSoftwareSerialNumber:=GetLine(Port,';',17);
end;

function NdYAGReadOutputPower:double;
var buf:string;
    help:double;
    Error:integer;
begin
  PutLine(Port,'?P;');
  buf:=GetLine(Port,';',255);
  if buf[length(buf)]='W' then buf[0]:=char(length(buf)-1);
  val(buf,help,Error);
  NdYAGReadOutputPower:=help;
  {writeln('P=',buf);
  PrintBuffer(1);
  readln;
  Putline(1,char(#13));
  delay(500);
  PrintBuffer(1);
  readln;
  ClearInQueue(1);}
end;

function NdYAGReadDiodeInformation(diodeNr:byte;InformationNr:byte):string;
var buf:string;
    help:double;
    Error:integer;
    help2:string;
begin
  if diodeNr>2 then diodeNr:=2;
  if diodeNr<1 then diodeNr:=1;
  if InformationNr>6 then InformationNr:=6;
  if InformationNr<1 then InformationNr:=1;
  buf:='?X'+char(InformationNr+byte('0'))+char(diodeNr+byte('0'))+';';
  PutLine(Port,buf);
  delay(70);
  help2:=GetLine(Port,';',255);
  NdYAGReadDiodeInformation:=help2;
  {writeln(buf,help2);}
end;

procedure NdYAGReboot;
begin
  PutLine(Port,'B1;');
end;

procedure NdYAGSetDiodeCurrent(diodeNr:byte;Current:double); {in A}
var buf:string;
begin
  if diodeNr>2 then diodeNr:=2;
  if diodeNr<1 then diodeNr:=1;
  current:=current*100;
  if Current>9999 then current:=9999;
  str(Current:0:0,buf);
  while length(buf)<4 do buf:='0'+buf;
  buf:='C'+char(diodeNr+byte('0'))+':'+buf+';';
  PutLine(Port,buf);
end;

procedure NdYAGQueueDiodeOnOffEvent(OnOff:boolean);
begin
  if OnOff then PutLine(Port,'D1;')
  else PutLine(Port,'D0;');
end;

procedure NdYAGSetLaserPower(power:double); {in mW}
var buf:string;
begin
  str(Power:0:0,buf);
  while length(buf)<4 do buf:='0'+buf;
  buf:='P'+buf+';';
  PutLine(Port,buf);
end;

procedure NdYAGSetDiodeTemperature(DiodeNr:byte;Temperature:double);
var buf:string;
begin
  if diodeNr>2 then diodeNr:=2;
  if diodeNr<1 then diodeNr:=1;
  if Temperature>99.9 then Temperature:=99.9;
  str(Temperature*10:0:0,buf);
  while length(buf)<3 do buf:='0'+buf;
  buf:='T'+char(diodeNr+byte('0'))+':'+buf+';';
  PutLine(Port,buf);
end;

procedure NdYAGSetDiodePumpRegulationMode(Mode:TDiodePumpRegulationMode);
begin
  case Mode of
    CurrentFeedback:PutLine(Port,'M0;');
    LaserPowerFeedback:PutLine(Port,'M1;');
    DiodePowerFeedback:PutLine(Port,'M2;');
    CurrentFeedbackStandbySetting:PutLine(Port,'M3;');
  end;
end;

begin
  Port:=1; {just for security, but it's necessary to use NdYAGSetComPort before usage}
end.
