unit PortList;
interface

procedure PortListInit;
function PortListBegin:byte;
procedure PortListPort(aportnr:word;avalue:byte);
procedure PortListWait(atime:double);
procedure PortListMultiOut(aaddr:byte;avalue:word);
procedure PortListEnd;
procedure PortListExecute(nr:byte);
procedure PortListExecuteBuffer(cycles:longint);

const PortListRegister:boolean=false;

implementation
uses crt,timing,io,MultiOut;

Type TTyp=(PortCommand,WaitCommand,MultiOutCommand);
     TPortListEntry=record
       case typ:TTyp of
         PortCommand:(portnr:word;value:byte;);
         MultiOutCommand:(addr:byte;MultiOutvalue:word;);
         WaitCommand:(time:single);
     end;

const MaxPortList=500;
      MaxPortLists=100;
      MaxBufferPortLists=10;
type PPortList=^TPortList;
     TPortList=array[1..MaxPortList] of TPortListEntry;

Type PList=^TList;
     TList=array[1..MaxPortLists] of record
       PortList:PPortList;
       size:word;
       buffer:pointer;
     end;

var BufferPortList:array[1..MaxBufferPortLists] of PPortList;
const Lists:PList=nil;
      AktList:byte=0;
      AktBufferEntry:word=0;
      AktBufferList:byte=0;
      oldexitproc:pointer=nil;
      StatusBuffer:pointer=nil;

procedure PortListDelete;
var n:byte;
begin
  n:=1;
  while (n<MaxPortLists) and (Lists^[n].PortList<>nil) do begin
    freemem(Lists^[n].PortList,Lists^[n].size*sizeof(TPortListEntry));
    Lists^[n].PortList:=nil;
    Lists^[n].size:=0;
    DeleteStatusBuffer(Lists^[n].buffer);
    n:=n+1;
  end;
end;

procedure BufferPortListDelete;
var n:word;
begin
  AktBufferEntry:=0;
  AktBufferList:=1;
  for n:=1 to MaxBufferPortLists do begin
    if BufferPortList[n]<>nil then dispose(BufferPortList[n]);
    BufferPortList[n]:=nil;
  end;
end;

procedure PortListInit;
begin
  PortListDelete;
  aktList:=0;
  BufferPortListDelete;
  PortListRegister:=false;
end;

function PortListBegin:byte;
begin
  if PortListRegister then begin
    writeln('PortList : PortListBegin : Error : List already started');
    delay(1000);
    exit;
  end;
  PortListRegister:=true;
  aktList:=aktList+1;
  BufferPortListDelete;
  PortListBegin:=aktList;
end;

procedure GetBufferSpace;
begin
  inc(AktBufferEntry);
  if AktBufferEntry>MaxPortList then begin
    inc(AktBufferList);
    AktBufferEntry:=1;
    if AktBufferList>MaxBufferPortLists then begin
      writeln('PortList : GetBufferSpace : Error : Too many Entries');
      delay(1000);
      dec(AktBufferList);
      exit;
    end;
  end;
  if BufferPortList[AktBufferList]=nil then begin
    if MaxAvail<sizeof(TPortList) then begin
      writeln('PortList : GetBufferSpace : Out of memory');
      delay(1000);
      if AktBufferList>1 then dec(AktBufferList);
      if AktBufferEntry>1 then dec(AktBufferEntry);
    end else new(BufferPortList[AktBufferList]);
  end;
end;

procedure PortListPort(aportnr:word;avalue:byte);
begin
  GetBufferSpace;
  with BufferPortList[AktBufferList]^[AktBufferEntry] do begin
    typ:=PortCommand;
    portnr:=aportnr;
    value:=avalue;
  end;
end;

procedure PortListMultiOut(aaddr:byte;avalue:word);
begin
  GetBufferSpace;
  with BufferPortList[AktBufferList]^[AktBufferEntry] do begin
    typ:=MultiOutCommand;
    addr:=aaddr;
    MultiOutvalue:=avalue;
  end;
end;

procedure PortListWait(atime:double);
begin
  GetBufferSpace;
  with BufferPortList[AktBufferList]^[AktBufferEntry] do begin
    typ:=WaitCommand;
    time:=atime;
  end;
end;

procedure PortListEnd;
var h:word;
begin
  if not PortListRegister then begin
    writeln('PortList : PortListEnd : Error : List not started');
    delay(1000);
    exit;
  end;
  PortListRegister:=false;
  If AktList=MaxPortLists then begin
    writeln('PortList : PortListEnd : Error : Too many Lists');
    delay(1000);
    exit;
  end;
  if AktBufferList<>1 then begin
    writeln('PortList : PortListEnd : Error : Buffer too big ');
    delay(1000);
    exit;
  end;
  with Lists^[AktList] do begin
    if MaxAvail<AktBufferEntry*sizeof(TPortListEntry) then begin
      h:=MaxAvail;
      writeln('PortList : PortListEnd : Error : Out of memory',^G);
      delay(1000);
      exit;
    end;
    getmem(PortList,AktBufferEntry*sizeof(TPortListEntry));
    size:=AktBufferEntry;
    move(BufferPortList[1]^,PortList^,AktBufferEntry*sizeof(TPortListEntry));
    SaveStatusBuffer(buffer);
  end;
end;

procedure PortListExecute(nr:byte);
var n:word;
begin
  if nr>AktList then begin
    writeln('PortList : PortListExecute : Error : List Nr ',nr,' not defined');
    delay(1000);
    exit;
  end;
  with Lists^[nr] do begin
    for n:=1 to size do with PortList^[n] do begin
      case typ of
        PortCommand:port[portnr]:=value;
        WaitCommand:TimingWait(time);
        MultiOutCommand:MultiOut.WritePortDirect(addr,MultiOutvalue);
      end;
    end;
    RetrieveStatusBuffer(buffer);
  end;
end;

procedure PortListExecuteBuffer(cycles:longint);
var n:word;
    m,l:word;
    c:longint;
begin
  for c:=1 to cycles do begin
    for m:=1 to AktBufferList do begin
      if m=AktBufferList then l:=AktBufferEntry else l:=MaxPortList;
      for n:=1 to l do with BufferPortList[m]^[n] do begin
        case typ of
          PortCommand:port[portnr]:=value;
          WaitCommand:TimingWait(time);
          MultiOutCommand:MultiOut.WritePortDirect(addr,MultiOutvalue);
        end;
      end;
    end;
  end;
end;

procedure MyExitProc;far;
begin
  exitproc:=oldexitproc;
  BufferPortListDelete;
  PortListDelete;
  dispose(Lists);
end;

var n:byte;
begin
  if MaxAvail<sizeof(TList) then begin
    writeln('PortList : Out of memory');
    delay(1000);
    Lists:=nil;
  end else new(Lists);
  aktList:=0;
  AktBufferEntry:=0;
  AktBufferList:=1;
  for n:=1 to MaxBufferPortLists do BufferPortList[n]:=nil;
  for n:=1 to MaxPortLists do with Lists^[n] do begin
    PortList:=nil;
    size:=0;
    buffer:=nil;
  end;
  oldexitproc:=exitproc;
  exitproc:=@myexitproc;
  PortListRegister:=false;
end.