unit Register;
{
  This unit contains routines to manage (store, load) the parameters of the experiment.
}
interface
uses Menu,serial,strings,crt;
const MarkColor=lightgreen;

Type IntPtr=^Integer;
     WordPtr=^Word;
     DoublePtr=^Double;
     BoolPtr=^Boolean;
     TProc=procedure;

type PRegister=^TRegister;
     TRegister=object
       next:PRegister;
       name:StringPtr;
       Question:StringPtr;
       Color:byte;
       Fix:boolean;
       constructor init(aname:string;aquestion:string;acolor:byte);
       constructor initFix(aname:stringPtr;aquestion:stringPtr;acolor:byte);
       destructor done;
       function GetRegister(var aname:string):PRegister;
       procedure MakeDialogEntry(var Dialog:PDialogInfo);virtual;
       procedure MakeSelectionEntry(var Menu:PMenuInfo);virtual;
       function GetSelectionEntry(choice:integer):PRegister;virtual;
       procedure Load(var inp:text);virtual;
       procedure Save(var out:text);
       procedure GetString(var s:string);virtual;
       procedure Send(port:byte;onlychanged:boolean);virtual;
       procedure ConvertFromString(s:string);virtual;
       function GetName:string;
       function Changed:boolean;virtual;
       function GetTypeChar:char;virtual;
       function Activated:boolean;virtual;
     end;

     PRegInt=^TRegInt;
     TRegInt=object(TRegister)
       p:IntPtr;
       old:integer;
       min,max:integer;
       constructor init(ap:IntPtr;aname:string;ainit:integer;amin:integer;amax:integer;aquestion:string;acolor:byte);
       destructor done;
       procedure MakeDialogEntry(var Dialog:PDialogInfo);virtual;
       procedure GetString(var s:string);virtual;
       procedure ConvertFromString(s:string);virtual;
       function Changed:boolean;virtual;
       function GetTypeChar:char;virtual;
     end;

     PRegDouble=^TRegDouble;
     TRegDouble=object(TRegister)
       p:DoublePtr;
       old:double;
       min,max:double;
       mychoice:integer;
       constructor init(ap:DoublePtr;aname:string;ainit:double;amin:double;amax:double;aquestion:string;acolor:byte);
       constructor initFix(ap:DoublePtr;aname:stringPtr;ainit:double;
         amin:double;amax:double;aquestion:stringPtr;acolor:byte);
       destructor done;
       function GetPointer:DoublePtr;
       procedure MakeDialogEntry(var Dialog:PDialogInfo);virtual;
       procedure MakeSelectionEntry(var Menu:PMenuInfo);virtual;
       function GetSelectionEntry(choice:integer):PRegister;virtual;
       procedure GetData(var aname:string;var amin:double;var amax:double;var aquestion:string);
       procedure GetString(var s:string);virtual;
       procedure ConvertFromString(s:string);virtual;
       function Changed:boolean;virtual;
       function GetTypeChar:char;virtual;
     end;

     PRegBoolean=^TRegBoolean;
     TRegBoolean=object(TRegister)
       p:BoolPtr;
       old:boolean;
       constructor init(ap:BoolPtr;aname:string;ainit:Boolean;aquestion:string;acolor:byte);
       destructor done;
       procedure MakeDialogEntry(var Dialog:PDialogInfo);virtual;
       procedure GetString(var s:string);virtual;
       procedure ConvertFromString(s:string);virtual;
       function Changed:boolean;virtual;
       function GetTypeChar:char;virtual;
       function Activated:boolean;virtual;
     end;

     PRegString=^TRegString;
     TRegString=object(TRegister)
       p:StringPtr;
       old:string;
       constructor init(ap:StringPtr;aname:string;ainit:String;aquestion:string;acolor:byte);
       destructor done;
       procedure MakeDialogEntry(var Dialog:PDialogInfo);virtual;
       procedure GetString(var s:string);virtual;
       procedure ConvertFromString(s:string);virtual;
       function Changed:boolean;virtual;
       function GetTypeChar:char;virtual;
     end;

const MaxMenuEntryId=1;

type PPRegistry=^PRegistry;
     PRegistry=^TRegistry;
     TRegistry=object
       next:PRegistry;
       name:StringPtr;
       Register:PRegister;
       SetupDialog:boolean;
       mychoice:array[0..MaxMenuEntryId] of integer;
       UpdateProcedure:TProc;
       root:PPRegistry;
       constructor init(aroot:PPRegistry;anext:PRegistry;aSetupDialog:boolean;aname:string;aUpdateProcedure:TProc);
       destructor done(deleteall:boolean);
       procedure Insert(ains:PRegister);
       procedure MakeMenuEntrys(var Menu:PMenuInfo;nr:byte;id:byte);
       procedure ExecuteMenuDialog(choice:integer;id:byte);
       function ExecuteSelectMenu(choice:integer;id:byte):PRegister;
       procedure SaveVariables(aname:string);
       procedure SaveVariablesInFile(var out:text);
       procedure LoadVariables(aname:string);
       procedure SendVariables(port:byte;onlychanged:boolean);
       procedure LoadVariablesFromFile(var inp:text);
       function GetRegister(aname:string):PRegister;
       procedure SetValue(aname:string;value:string);
     end;

procedure RegisterRegistry(aRegistry:PPRegistry);
function MakeSelectMenu:PRegister;
procedure SendVariables(port:byte;SendAll:boolean;ShutDownSuccessfull:boolean);

implementation
uses Dos;

const MaxRegistry=10;
var RegistryList:array[1..MaxRegistry] of PPRegistry;
const LastRegistry:byte=0;

procedure eatWhite(var buf:string);
begin
  while (buf[length(buf)]=' ') and (length(buf)>1) do buf:=copy(buf,1,length(buf)-1);
  while (buf[1]=' ') and (length(buf)>1) do buf:=copy(buf,2,length(buf)-1);
end;

{TRegister}

constructor TRegister.init(aname:string;aquestion:string;acolor:byte);
begin
  next:=NIL;
  name:=NewString(aname);
  Question:=NewString(aQuestion);
  Color:=aColor;
  Fix:=false;
end;

constructor TRegister.initFix(aname:stringPtr;aquestion:stringPtr;acolor:byte);
begin
  next:=NIL;
  name:=aname;
  Question:=aQuestion;
  Color:=aColor;
  Fix:=true;
end;

destructor TRegister.done;
begin
  if next<>nil then dispose(next,done);
  if not Fix then begin
    DeleteString(name);
    DeleteString(Question);
  end;
end;

{function TRegister.GetPointer(aname:Namestr):pointer;
begin
  if next<>nil then GetPointer:=next^.GetPointer(name)
  else GetPointer:=NIL;
end;}

function TRegister.GetName:string;
begin
  GetName:=name^;
end;

function TRegister.GetRegister(var aname:string):PRegister;
begin
  if aname=name^ then GetRegister:=@self
  else begin
    if next<>nil then GetRegister:=next^.GetRegister(aname)
    else GetRegister:=Nil;
  end;
end;

procedure TRegister.MakeDialogEntry(var Dialog:PDialogInfo);
begin
  if next<>nil then next^.MakeDialogEntry(Dialog);
end;

procedure TRegister.MakeSelectionEntry(var Menu:PMenuInfo);
begin
end;

function TRegister.GetSelectionEntry(choice:integer):PRegister;
begin
  if next<>nil then GetSelectionEntry:=next^.GetSelectionEntry(choice)
  else GetSelectionEntry:=nil;
end;

procedure TRegister.Load(var inp:text);
var aname,value,buf:string;
begin
  if eof(inp) then reset(inp);
  if not eof(inp) then begin
    readln(inp,buf);
    aname:=copy(buf,1,pos('=',buf)-1);
    value:=copy(buf,pos('=',buf)+1,pos(';',buf)-pos('=',buf)-1);
    eatWhite(value);
    eatWhite(aname);
    if not (name^=aname) then begin
      reset(inp);
      while (not eof(inp)) and (name^<>aname) do begin
        readln(inp,buf);
        aname:=copy(buf,1,pos('=',buf)-1);
        value:=copy(buf,pos('=',buf)+1,pos(';',buf)-pos('=',buf)-1);
        eatWhite(value);
        eatWhite(aname);
      end;
      if (name^=aname) then ConvertFromString(value)
      else begin
        writeln('Register : '+name^+' not found');
      end;
    end else ConvertFromString(value);
  end;
end;

procedure TRegister.ConvertFromString(s:string);
begin
end;

procedure TRegister.Save(var out:text);
var buf:string;
begin
  GetString(buf);
  write(out,name^:15,'='+buf);
  writeln(out,' ; ',question^);
end;

procedure TRegister.GetString(var s:string);
begin
  s:='';
end;

procedure PutLine(Pt:byte;OutStr:string);
var i:integer;
begin
  for i:=1 to length(OutStr) do PutChar(Pt,OutStr[i]);
end;

procedure TRegister.Send(port:byte;onlychanged:boolean);
var buf:string;
    ischanged:boolean;
    c:char;
begin
  ischanged:=changed;
  if (not onlychanged) or ischanged then begin
    GetString(buf);
    PutLine(port,'*'+GetTypeChar+name^+'='+buf+'#');
  end;
end;

function TRegister.Changed:boolean;
begin
  changed:=true;
end;

function TRegister.GetTypeChar:char;
begin
  GetTypeChar:='?';
end;

function TRegister.Activated:boolean;
begin
  if next<>NIL then Activated:=next^.activated
  else Activated:=false;
end;

{TRegInt}

constructor TRegInt.init(ap:IntPtr;aname:string;ainit:integer;amin:integer;amax:integer;aquestion:string;acolor:byte);
begin
  TRegister.Init(aname,aquestion,acolor);
  p:=ap;
  min:=amin;
  max:=amax;
  p^:=ainit;
end;

destructor TRegInt.done;
begin
  TRegister.done;
end;

procedure TRegInt.MakeDialogEntry(var Dialog:PDialogInfo);
begin
  TRegister.MakeDialogEntry(Dialog);
  AddI(color,Dialog,p^,question^,name^,min,max);
end;

procedure TRegInt.GetString(var s:string);
begin
  str(p^{:15},s);
end;

function TRegInt.Changed:boolean;
begin
  changed:=old<>p^;
  old:=p^;
end;

function TRegInt.GetTypeChar:char;
begin
  GetTypeChar:='I';
end;

procedure TRegInt.ConvertFromString(s:string);
var err,hint:integer;
begin
  err:=0;
  val(s,hint,err);
  if err=0 then p^:=hint
  else writeln('Register : LoadVariables : could not convert ',s,' at position ',err,' Variable ',name^);
end;

{TRegDouble}

constructor TRegDouble.init(ap:DoublePtr;aname:string;ainit:double;
  amin:double;amax:double;aquestion:string;acolor:byte);
begin
  TRegister.Init(aname,aquestion,acolor);
  p:=ap;
  min:=amin;
  max:=amax;
  p^:=ainit;
  mychoice:=0;
end;

constructor TRegDouble.initFix(ap:DoublePtr;aname:stringPtr;ainit:double;
  amin:double;amax:double;aquestion:stringPtr;acolor:byte);
begin
  TRegister.InitFix(aname,aquestion,acolor);
  p:=ap;
  min:=amin;
  max:=amax;
  p^:=ainit;
  mychoice:=0;
end;

destructor TRegDouble.done;
begin
  TRegister.done;
end;

function TRegDouble.GetPointer:DoublePtr;
begin
  GetPointer:=p;
end;

procedure TRegDouble.MakeDialogEntry(var Dialog:PDialogInfo);
begin
  TRegister.MakeDialogEntry(Dialog);
  AddR(color,Dialog,p^,question^,name^,min,max,1);
end;

procedure TRegDouble.MakeSelectionEntry(var Menu:PMenuInfo);
begin
  TRegister.MakeSelectionEntry(Menu);
  AddChoice(yellow,' ',yellow,Menu,name^,question^,'*',mychoice);
end;

function TRegDouble.GetSelectionEntry(choice:integer):PRegister;
begin
  if (choice<>mychoice)
  then GetSelectionEntry:=TRegister.GetSelectionEntry(choice)
  else GetSelectionEntry:=@self;
end;

procedure TRegDouble.GetData(var aname:string;var amin:double;var amax:double;var aquestion:string);
begin
  aname:=name^;
  amin:=min;
  amax:=max;
  aquestion:=question^;
end;

function DoubleToString(help1:double):string;
var sign:boolean;
    help:longint;
    s,s2:string;
    n:byte;
begin
  sign:=help1<0;
  if sign then help1:=-help1;
  help:=round(help1*100000);
  str(help,s);
  if length(s)<6 then begin
    s2:='000000';
    for n:=1 to length(s) do s2[6-n+1]:=s[length(s)-n+1];
  end else s2:=s;
  for n:=0 to 4 do s2[length(s2)-n+1]:=s2[length(s2)-n];
  s2[length(s2)-5+1]:='.';
  inc(s2[0]);
  if sign then s:='-'+s2 else s:=s2;
  DoubleToString:=s;
end;

procedure TRegDouble.GetString(var s:string);
begin
  {writeln(p^:7:7);}
  {s:=DoubleToString(p^);}
  str(p^:7:7,s);
end;

function TRegDouble.Changed:boolean;
begin
  changed:=old<>p^;
  old:=p^;
end;

function TRegDouble.GetTypeChar:char;
begin
  GetTypeChar:='D';
end;

procedure TRegDouble.ConvertFromString(s:string);
var err:integer;
    hdoub:double;
begin
  err:=0;
  val(s,hdoub,err);
  if err=0 then p^:=hdoub
  else writeln('Register : LoadVariables : could not convert ',s,' at position ',err,' Variable ',name^);
end;

{TRegBoolean}

constructor TRegBoolean.init(ap:BoolPtr;aname:string;ainit:Boolean;aquestion:string;acolor:byte);
begin
  TRegister.init(aname,aquestion,acolor);
  p:=ap;
  p^:=ainit;
end;

destructor TRegBoolean.done;
begin
  TRegister.done;
end;

procedure TRegBoolean.MakeDialogEntry(var Dialog:PDialogInfo);
begin
  TRegister.MakeDialogEntry(Dialog);
  AddB(color,Dialog,p^,question^,name^);
end;

procedure TRegBoolean.GetString(var s:string);
begin
  if p^ then s:='TRUE' else s:='FALSE';
end;

function TRegBoolean.Changed:boolean;
begin
  changed:=old<>p^;
  old:=p^;
end;

function TRegBoolean.GetTypeChar:char;
begin
  GetTypeChar:='B';
end;

procedure TRegBoolean.ConvertFromString(s:string);
begin
  p^:=s='TRUE';
end;

function TRegBoolean.activated:boolean;
var help:boolean;
begin
  help:=(Color=MarkColor) and p^;
  if not help then begin
    if next<>nil then Activated:=help or next^.activated
    else Activated:=help;
  end else Activated:=help;
end;

{TRegString}

constructor TRegString.init(ap:StringPtr;aname:string;ainit:String;aquestion:string;acolor:byte);
begin
  TRegister.init(aname,aquestion,acolor);
  p:=ap;
  p^:=ainit;
end;

destructor TRegString.done;
begin
  TRegister.done;
end;

procedure TRegString.MakeDialogEntry(var Dialog:PDialogInfo);
begin
  TRegister.MakeDialogEntry(Dialog);
  AddS(color,Dialog,p^,question^,name^);
end;

procedure TRegString.GetString(var s:string);
begin
  s:=p^;
end;

function TRegString.Changed:boolean;
begin
  changed:=old<>p^;
  old:=p^;
end;

function TRegString.GetTypeChar:char;
begin
  GetTypeChar:='S';
end;

procedure TRegString.ConvertFromString(s:string);
begin
  p^:=s;
end;

{TRegistry}

constructor TRegistry.init(aroot:PPRegistry;anext:PRegistry;aSetupDialog:boolean;aname:string;aUpdateProcedure:TProc);
var h:PRegistry;
    n:byte;
begin
  root:=aroot;
  if root^<>nil then begin
    if (root^=anext) then begin
      root^:=@self;
    end else begin
      h:=root^;
      while (h<>nil) and (h^.next<>anext) do h:=h^.next;
      if h^.next=anext then begin
        h^.next:=@self;
      end;
    end;
  end else root^:=@self;
  next:=anext;
  name:=NewString(aname);
  Register:=NIL;
  SetupDialog:=aSetupDialog;
  for n:=0 to MaxMenuEntryid do mychoice[n]:=0;
  UpdateProcedure:=aUpdateProcedure;
end;

destructor TRegistry.done(deleteall:boolean);
var h:PRegistry;
begin
  if root^<>nil then begin
    h:=root^;
    while (h<>nil) and (h^.next<>@self) do h:=h^.next;
    if h^.next=@self then begin
      h^.next:=next;
    end else if root^=@self then begin
      root^:=next;
    end;
  end;
  if deleteall and (next<>NIL) then dispose(next,done(true));
  DeleteString(name);
end;

procedure TRegistry.Insert(ains:PRegister);
begin
  ains^.next:=Register;
  Register:=ains;
end;

procedure TRegistry.MakeMenuEntrys(var Menu:PMenuInfo;nr:byte;id:byte);
var buf:string[1];
    buf2:string;
    n:byte;
    activated:boolean;
    color:byte;
    sign:char;
begin
  if (id>MaxMenuEntryId) then begin
    writeln('Register : MakeMenuEntrys : wrong id ',id);
    delay(500);
    exit;
  end;
  if next<>nil then next^.MakeMenuEntrys(Menu,nr+1,id);
  if SetupDialog then begin
    str(nr:1,buf);
    for n:=1 to 80 do buf2[n]:=' ';
    for n:=1 to length(name^) do buf2[n]:=name^[n];
    buf2[60]:=buf[1];
    buf2[59]:='[';
    buf2[61]:=']';
    buf2[0]:=#62;
    if Register<>nil then activated:=Register^.activated
    else activated:=false;
    if activated then begin
      color:=green;
      sign:='';
    end else begin
      color:=yellow;
      sign:=' ';
    end;
    AddChoice(color,sign,yellow,Menu,buf2,name^,buf[1],mychoice[id]);
  end;
end;

procedure TRegistry.ExecuteMenuDialog(choice:integer;id:byte);
var Dialog:PDialogInfo;
    UpdateDialog : boolean;
begin
  if (id>MaxMenuEntryId) then begin
    writeln('Register : ExecuteMenuDialog : wrong id ',id);
    delay(500);
    exit;
  end;
  if choice<>mychoice[id] then begin
    if next<>nil then next^.ExecuteMenuDialog(choice,id);
  end else if Register<>nil then begin
    NewDialog(Dialog);
    Register^.MakeDialogEntry(Dialog);
    BackupVars(Dialog);
    DoDialog(Dialog,name^,UpdateDialog);
    UpdateProcedure;
  end;
end;

function TRegistry.ExecuteSelectMenu(choice:integer;id:byte):PRegister;
var Menu:PMenuInfo;
    iBack:integer;
    h:PRegister;
    nr:byte;
begin
  if (id>MaxMenuEntryId) then begin
    writeln('Register : ExecuteSelectMenu : wrong id ',id);
    delay(500);
    exit;
  end;
  if choice<>mychoice[id] then begin
    if next<>nil then ExecuteSelectMenu:=next^.ExecuteSelectMenu(choice,id)
    else ExecuteSelectMenu:=nil;
  end else if Register<>nil then begin
    NewMenu(Menu);
    h:=Register;
    nr:=0;
    while h<>nil do begin
      h^.MakeSelectionEntry(Menu);
      nr:=nr+1;
      h:=h^.next;
    end;
    choice:=-1;
    DoMenu(Menu,'Select Parameter',choice);
    ExecuteSelectMenu:=Register^.GetSelectionEntry(choice);
  end else ExecuteSelectMenu:=nil;
end;

procedure TRegistry.SaveVariables(aname:string);
var out,bak:text;
    i:byte;
    buf:string;
    D: DirStr;
    N: DOS.NameStr;
    E: ExtStr;
    bakname:string;
begin
  assign(out,aname);
  {$I-}
  reset(out);
  {$I+}
  if ioresult=0 then begin
    close(out);
    FSplit(aname, D, N, E);
    bakname:=D+N+'.BAK';
    assign(bak,bakname);
    {$I-}
    reset(bak);
    {$I+}
    if ioresult=0 then begin
      close(bak);
      erase(bak);
    end;
    rename(out,D+N+'.BAK');
  end;
  assign(out,aname);
  {$I-}
  rewrite(out);
  {$I+}
  if ioresult=0 then begin
    SaveVariablesInFile(out);
    close(out);
  end else writeln('Register : SaveVariables : Couldn''t save file ',aname);
end;

procedure TRegistry.SaveVariablesInFile(var out:text);
var h:pregister;
begin
  h:=Register;
  while h<>nil do begin
    h^.Save(out);
    h:=h^.next;
  end;
  if next<>NIL then next^.SaveVariablesInFile(out);
end;

procedure TRegistry.SendVariables(port:byte;onlychanged:boolean);
var h:PRegister;
begin
  {if next<>NIL then next^.SendVariables(port,onlychanged);}
  h:=Register;
  while h<>NIL do begin
    h^.Send(port,onlychanged);
    h:=h^.next;
  end;
end;

procedure TRegistry.LoadVariables(aname:string);
var inp:text;
begin
  assign(inp,aname);
  {$I-}
  reset(inp);
  {$I+}
  if ioresult=0 then begin
    LoadVariablesFromFile(inp);
    close(inp);
  end else begin
    writeln('Register : LoadVariables : Couldn''t load file ',aname,^G);
    delay(500);
  end;
end;

procedure TRegistry.LoadVariablesFromFile(var inp:text);
var h:PRegister;
begin
  if Register<>NIL then begin
    h:=Register;
    while h<>nil do begin
      h^.Load(inp);
      h:=h^.next;
    end;
  end;
  if Next<>NIL then next^.LoadVariablesFromFile(inp);
end;

function TRegistry.GetRegister(aname:string):PRegister;
var h:PRegister;
begin
  If Register<>nil then begin
    h:=Register^.GetRegister(aname);
    if h=nil then begin
      if next<>nil then GetRegister:=next^.GetRegister(aname)
      else GetRegister:=nil;
    end else GetRegister:=h;
  end else GetRegister:=nil;
end;

procedure TRegistry.SetValue(aname:string;value:string);
var h:PRegister;
begin
  h:=GetRegister(aname);
  if (h<>nil) then begin
    h^.ConvertFromString(value);
  end else begin
    writeln('Register : SetValue('+aname+','+value+') : '+aname+' not found');
    delay(500);
  end;
end;

procedure RegisterRegistry(aRegistry:PPRegistry);
begin
  inc(LastRegistry);
  RegistryList[lastRegistry]:=aRegistry;
end;

function MakeSelectMenu:PRegister;
var Menu:PMenuInfo;
    choice,iBack:integer;
    h:PRegister;
    n:byte;
begin
  h:=nil;
  choice:=-1;
  repeat
    NewMenu(Menu);
    for n:=1 to LastRegistry do if RegistryList[n]^<>nil
    then RegistryList[n]^^.MakeMenuEntrys(menu,0,1);
    AddChoice(yellow,' ',yellow,Menu,'Back                                         [B]',
	   'Back','B',iBack);
    DoMenu(Menu,' Select Parameter Main Menu ',Choice);
    if (choice<>iback) and (choice<>0) then begin
      n:=1;
      while (n<=LastRegistry) and (h=nil) and (RegistryList[n]^<>nil) do begin
        h:=RegistryList[n]^^.ExecuteSelectMenu(choice,1);
        inc(n);
      end;
    end;
  until (h<>nil) or (choice=0) or (choice=iBack);
  MakeSelectMenu:=h;
end;

const NewStarted:boolean=true;

procedure SendVariables(port:byte;SendAll:boolean;ShutDownSuccessfull:boolean);
var n:byte;
    onlychanged:boolean;
    h:PRegistry;
begin
 if (NewStarted and (not ShutDownSuccessfull)) or SendAll then writeln('Sending Parameters...');
  onlychanged:=not ((NewStarted and (not ShutDownSuccessfull)) or SendAll);
  for n:=1 to LastRegistry do if RegistryList[n]^<>nil then begin
    h:=RegistryList[n]^;
    while h<>NIL do begin
      h^.SendVariables(port,onlychanged);
      PutLine(port,'*T *** '+h^.name^+' *** = 0#');
      h:=h^.next;
    end;
  end;
  NewStarted:=false;
end;

end.