unit Register;
{
  This unit contains routines to manage (store, load) the parameters of the experiment.
}
interface
uses Menu,serial;

Type IntPtr=^Integer;
     WordPtr=^Word;
     DoublePtr=^Double;
     BoolPtr=^Boolean;
     StringPtr=^String;
     NameStr=string[30];
     QuestionStr=string[80];
     TitelStr=string[80];
     TProc=procedure;


type PRegister=^TRegister;
     TRegister=object
       next:PRegister;
       name:Namestr;
       Question:QuestionStr;
       Color:byte;
       constructor init(aname:Namestr;aquestion:QuestionStr;acolor:byte);
       destructor done;
       function GetRegister(var aname:Namestr):PRegister;
       procedure MakeDialogEntry(var Dialog:PDialogInfo);virtual;
       procedure MakeSelectionEntry(var Menu:PMenuInfo);virtual;
       function GetSelectionEntry(choice:integer):PRegister;virtual;
       procedure Load(var inp:text);virtual;
       procedure Save(var out:text);
       procedure GetString(var s:string);virtual;
       procedure Send(port:serialport);virtual;
       procedure ConvertFromString(s:string);virtual;
       function GetName:string;
     end;

     PRegInt=^TRegInt;
     TRegInt=object(TRegister)
       p:IntPtr;
       min,max:integer;
       constructor init(ap:IntPtr;aname:Namestr;ainit:integer;amin:integer;amax:integer;aquestion:QuestionStr;acolor:byte);
       destructor done;
       procedure MakeDialogEntry(var Dialog:PDialogInfo);virtual;
       procedure GetString(var s:string);virtual;
       procedure ConvertFromString(s:string);virtual;
     end;

     PRegDouble=^TRegDouble;
     TRegDouble=object(TRegister)
       p:DoublePtr;
       min,max:double;
       mychoice:integer;
       constructor init(ap:DoublePtr;aname:Namestr;ainit:double;amin:double;amax:double;aquestion:QuestionStr;acolor:byte);
       destructor done;
       function GetPointer:DoublePtr;
       procedure MakeDialogEntry(var Dialog:PDialogInfo);virtual;
       procedure MakeSelectionEntry(var Menu:PMenuInfo);virtual;
       function GetSelectionEntry(choice:integer):PRegister;virtual;
       procedure GetData(var aname:string;var amin:double;var amax:double;var aquestion:string);
       procedure GetString(var s:string);virtual;
       procedure ConvertFromString(s:string);virtual;
     end;

     PRegBoolean=^TRegBoolean;
     TRegBoolean=object(TRegister)
       p:BoolPtr;
       constructor init(ap:BoolPtr;aname:Namestr;ainit:Boolean;aquestion:QuestionStr;acolor:byte);
       destructor done;
       procedure MakeDialogEntry(var Dialog:PDialogInfo);virtual;
       procedure GetString(var s:string);virtual;
       procedure ConvertFromString(s:string);virtual;
     end;

     PRegString=^TRegString;
     TRegString=object(TRegister)
       p:StringPtr;
       constructor init(ap:StringPtr;aname:Namestr;ainit:String;aquestion:QuestionStr;acolor:byte);
       destructor done;
       procedure MakeDialogEntry(var Dialog:PDialogInfo);virtual;
       procedure GetString(var s:string);virtual;
       procedure ConvertFromString(s:string);virtual;
     end;
const MaxMenuEntryId=1;


type PPRegistry=^PRegistry;
     PRegistry=^TRegistry;
     TRegistry=object
       next:PRegistry;
       name:TitelStr;
       Register:PRegister;
       SetupDialog:boolean;
       mychoice:array[0..MaxMenuEntryId] of integer;
       UpdateProcedure:TProc;
       root:PPRegistry;
       constructor init(aroot:PPRegistry;anext:PRegistry;aSetupDialog:boolean;aname:TitelStr;aUpdateProcedure:TProc);
       destructor done(deleteall:boolean);
       procedure Insert(ains:PRegister);
       procedure MakeMenuEntrys(var Menu:PMenuInfo;nr:byte;id:byte);
       procedure ExecuteMenuDialog(choice:integer;id:byte);
       function ExecuteSelectMenu(choice:integer;id:byte):PRegister;
       procedure SaveVariables(aname:string);
       procedure SaveVariablesInFile(var out:text);
       procedure LoadVariables(aname:string);
       procedure SendVariables(port:serialport);
       procedure LoadVariablesFromFile(var inp:text);
       function GetRegister(aname:Namestr):PRegister;
       procedure SetValue(aname:string;value:string);
     end;

procedure RegisterRegistry(aRegistry:PPRegistry);
function MakeSelectMenu:PRegister;
procedure SendVariables(port:serialport);

implementation
uses crt,Dos;

const MaxRegistry=100;
var RegistryList:array[1..MaxRegistry] of PPRegistry;
const LastRegistry:byte=0;

procedure eatWhite(var buf:string);
begin
  while (buf[length(buf)]=' ') and (length(buf)>1) do buf:=copy(buf,1,length(buf)-1);
  while (buf[1]=' ') and (length(buf)>1) do buf:=copy(buf,2,length(buf)-1);
end;

{TRegister}

constructor TRegister.init(aname:Namestr;aquestion:QuestionStr;acolor:byte);
begin
  next:=NIL;
  name:=aname;
  Question:=aQuestion;
  Color:=aColor;
end;

destructor TRegister.done;
begin
  if next<>nil then dispose(next,done);
end;

{function TRegister.GetPointer(aname:Namestr):pointer;
begin
  if next<>nil then GetPointer:=next^.GetPointer(name)
  else GetPointer:=NIL;
end;}

function TRegister.GetName:string;
begin
  GetName:=name;
end;

function TRegister.GetRegister(var aname:Namestr):PRegister;
begin
  if aname=name then GetRegister:=@self
  else begin
    if next<>nil then GetRegister:=next^.GetRegister(aname)
    else GetRegister:=Nil;
  end;
end;

procedure TRegister.MakeDialogEntry(var Dialog:PDialogInfo);
begin
  if next<>nil then next^.MakeDialogEntry(Dialog);
end;

procedure TRegister.MakeSelectionEntry(var Menu:PMenuInfo);
begin
end;

function TRegister.GetSelectionEntry(choice:integer):PRegister;
begin
  if next<>nil then GetSelectionEntry:=next^.GetSelectionEntry(choice)
  else GetSelectionEntry:=nil;
end;

procedure TRegister.Load(var inp:text);
var aname,value,buf:string;
begin
  if eof(inp) then reset(inp);
  if not eof(inp) then begin
    readln(inp,buf);
    aname:=copy(buf,1,pos('=',buf)-1);
    value:=copy(buf,pos('=',buf)+1,pos(';',buf)-pos('=',buf)-1);
    eatWhite(value);
    eatWhite(aname);
    if not (name=aname) then begin
      reset(inp);
      while (not eof(inp)) and (name<>aname) do begin
        readln(inp,buf);
        aname:=copy(buf,1,pos('=',buf)-1);
        value:=copy(buf,pos('=',buf)+1,pos(';',buf)-pos('=',buf)-1);
        eatWhite(value);
        eatWhite(aname);
      end;
      if (name=aname) then ConvertFromString(value)
      else begin
        writeln('Register : '+name+' not found');
      end;
    end else ConvertFromString(value);
  end;
end;

procedure TRegister.ConvertFromString(s:string);
begin
end;

procedure TRegister.Save(var out:text);
var buf:string;
begin
  GetString(buf);
  write(out,name:15,'='+buf);
  writeln(out,' ; ',question);
end;

procedure TRegister.GetString(var s:string);
begin
  s:='';
end;

procedure PutLine (Pt:SerialPort;OutStr: string);
var
  i : integer;

begin
  for i:= 1 to length(OutStr) do PutChar (Pt,OutStr[i]);
end;

procedure TRegister.Send(port:serialport);
var buf:string;
begin
  GetString(buf);
  PutLine(port,'*'+name+'='+buf+'#');
end;

{TRegInt}

constructor TRegInt.init(ap:IntPtr;aname:Namestr;ainit:integer;amin:integer;amax:integer;aquestion:QuestionStr;acolor:byte);
begin
  TRegister.Init(aname,aquestion,acolor);
  p:=ap;
  min:=amin;
  max:=amax;
  p^:=ainit;
end;

destructor TRegInt.done;
begin
  TRegister.done;
end;

procedure TRegInt.MakeDialogEntry(var Dialog:PDialogInfo);
begin
  TRegister.MakeDialogEntry(Dialog);
  AddI(color,Dialog,p^,question,name,min,max);
end;

procedure TRegInt.GetString(var s:string);
begin
  str(p^:15,s);
end;

procedure TRegInt.ConvertFromString(s:string);
var err,hint:integer;
begin
  err:=0;
  val(s,hint,err);
  if err=0 then p^:=hint
  else writeln('Register : LoadVariables : could not convert ',s,' at position ',err,' Variable ',name);
end;

{TRegDouble}

constructor TRegDouble.init(ap:DoublePtr;aname:Namestr;ainit:double;amin:double;amax:double;aquestion:QuestionStr;acolor:byte);
begin
  TRegister.Init(aname,aquestion,acolor);
  p:=ap;
  min:=amin;
  max:=amax;
  p^:=ainit;
  mychoice:=0;
end;

destructor TRegDouble.done;
begin
  TRegister.done;
end;

function TRegDouble.GetPointer:DoublePtr;
begin
  GetPointer:=p;
end;

procedure TRegDouble.MakeDialogEntry(var Dialog:PDialogInfo);
begin
  TRegister.MakeDialogEntry(Dialog);
  AddR(color,Dialog,p^,question,name,min,max,1);
end;

procedure TRegDouble.MakeSelectionEntry(var Menu:PMenuInfo);
begin
  TRegister.MakeSelectionEntry(Menu);
  AddChoice(Menu,name,question,'*',mychoice);
end;

function TRegDouble.GetSelectionEntry(choice:integer):PRegister;
begin
  if (choice<>mychoice)
  then GetSelectionEntry:=TRegister.GetSelectionEntry(choice)
  else GetSelectionEntry:=@self;
end;

procedure TRegDouble.GetData(var aname:string;var amin:double;var amax:double;var aquestion:string);
begin
  aname:=name;
  amin:=min;
  amax:=max;
  aquestion:=question;
end;

procedure TRegDouble.GetString(var s:string);
begin
  str(p^:7:7,s);
end;

procedure TRegDouble.ConvertFromString(s:string);
var err:integer;
    hdoub:double;
begin
  err:=0;
  val(s,hdoub,err);
  if err=0 then p^:=hdoub
  else writeln('Register : LoadVariables : could not convert ',s,' at position ',err,' Variable ',name);
end;

{TRegBoolean}

constructor TRegBoolean.init(ap:BoolPtr;aname:Namestr;ainit:Boolean;aquestion:QuestionStr;acolor:byte);
begin
  TRegister.init(aname,aquestion,acolor);
  p:=ap;
  p^:=ainit;
end;

destructor TRegBoolean.done;
begin
  TRegister.done;
end;

procedure TRegBoolean.MakeDialogEntry(var Dialog:PDialogInfo);
begin
  TRegister.MakeDialogEntry(Dialog);
  AddB(color,Dialog,p^,question,name);
end;

procedure TRegBoolean.GetString(var s:string);
begin
  if p^ then s:='TRUE' else s:='FALSE';
end;

procedure TRegBoolean.ConvertFromString(s:string);
begin
  p^:=s='TRUE';
end;

{TRegString}

constructor TRegString.init(ap:StringPtr;aname:Namestr;ainit:String;aquestion:QuestionStr;acolor:byte);
begin
  TRegister.init(aname,aquestion,acolor);
  p:=ap;
  p^:=ainit;
end;

destructor TRegString.done;
begin
  TRegister.done;
end;

procedure TRegString.MakeDialogEntry(var Dialog:PDialogInfo);
begin
  TRegister.MakeDialogEntry(Dialog);
  AddS(color,Dialog,p^,question,name);
end;

procedure TRegString.GetString(var s:string);
begin
  s:=p^;
end;

procedure TRegString.ConvertFromString(s:string);
begin
  p^:=s;
end;

{TRegistry}

constructor TRegistry.init(aroot:PPRegistry;anext:PRegistry;aSetupDialog:boolean;aname:TitelStr;aUpdateProcedure:TProc);
var h:PRegistry;
    n:byte;
begin
  root:=aroot;
  if root^<>nil then begin
    if (root^=anext) then begin
      root^:=@self;
    end else begin
      h:=root^;
      while (h<>nil) and (h^.next<>anext) do h:=h^.next;
      if h^.next=anext then begin
        h^.next:=@self;
      end;
    end;
  end else root^:=@self;
  next:=anext;
  name:=aname;
  Register:=NIL;
  SetupDialog:=aSetupDialog;
  for n:=0 to MaxMenuEntryid do mychoice[n]:=0;
  UpdateProcedure:=aUpdateProcedure;
end;

destructor TRegistry.done(deleteall:boolean);
var h:PRegistry;
begin
  if root^<>nil then begin
    h:=root^;
    while (h<>nil) and (h^.next<>@self) do h:=h^.next;
    if h^.next=@self then begin
      h^.next:=next;
    end else if root^=@self then begin
      root^:=next;
    end;
  end;
  if deleteall and (next<>NIL) then dispose(next,done(true));
end;

procedure TRegistry.Insert(ains:PRegister);
begin
  ains^.next:=Register;
  Register:=ains;
end;

procedure TRegistry.MakeMenuEntrys(var Menu:PMenuInfo;nr:byte;id:byte);
var buf:string[1];
    buf2:string;
    n:byte;
begin
  if (id>MaxMenuEntryId) then begin
    writeln('Register : MakeMenuEntrys : wrong id ',id);
    delay(500);
    exit;
  end;
  if next<>nil then next^.MakeMenuEntrys(Menu,nr+1,id);
  if SetupDialog then begin
    str(nr:1,buf);
    for n:=1 to 80 do buf2[n]:=' ';
    for n:=1 to length(name) do buf2[n]:=name[n];
    buf2[60]:=buf[1];
    buf2[59]:='[';
    buf2[61]:=']';
    buf2[0]:=#62;
    AddChoice(Menu,buf2,name,buf[1],mychoice[id]);
  end;
end;

procedure TRegistry.ExecuteMenuDialog(choice:integer;id:byte);
var Dialog:PDialogInfo;
    UpdateDialog : boolean;
begin
  if (id>MaxMenuEntryId) then begin
    writeln('Register : ExecuteMenuDialog : wrong id ',id);
    delay(500);
    exit;
  end;
  if choice<>mychoice[id] then begin
    if next<>nil then next^.ExecuteMenuDialog(choice,id);
  end else if Register<>nil then begin
    NewDialog(Dialog);
    Register^.MakeDialogEntry(Dialog);
    BackupVars(Dialog);
    DoDialog(Dialog,name,UpdateDialog);
    UpdateProcedure;
  end;
end;

function TRegistry.ExecuteSelectMenu(choice:integer;id:byte):PRegister;
var Menu:PMenuInfo;
    iBack:integer;
    h:PRegister;
    nr:byte;
begin
  if (id>MaxMenuEntryId) then begin
    writeln('Register : ExecuteSelectMenu : wrong id ',id);
    delay(500);
    exit;
  end;
  if choice<>mychoice[id] then begin
    if next<>nil then ExecuteSelectMenu:=next^.ExecuteSelectMenu(choice,id)
    else ExecuteSelectMenu:=nil;
  end else if Register<>nil then begin
    NewMenu(Menu);
    h:=Register;
    nr:=0;
    while h<>nil do begin
      h^.MakeSelectionEntry(Menu);
      nr:=nr+1;
      h:=h^.next;
    end;
    DoMenu(Menu,'Select Parameter',choice);
    ExecuteSelectMenu:=Register^.GetSelectionEntry(choice);
  end else ExecuteSelectMenu:=nil;
end;

procedure TRegistry.SaveVariables(aname:string);
var out,bak:text;
    i:byte;
    buf:string;
    D: DirStr;
    N: DOS.NameStr;
    E: ExtStr;
    bakname:string;
begin
  assign(out,aname);
  {$I-}
  reset(out);
  {$I+}
  if ioresult=0 then begin
    close(out);
    FSplit(aname, D, N, E);
    bakname:=D+N+'.BAK';
    assign(bak,bakname);
    {$I-}
    reset(bak);
    {$I+}
    if ioresult=0 then begin
      close(bak);
      erase(bak);
    end;
    rename(out,D+N+'.BAK');
  end;
  assign(out,aname);
  {$I-}
  rewrite(out);
  {$I+}
  if ioresult=0 then begin
    SaveVariablesInFile(out);
    close(out);
  end else writeln('Register : SaveVariables : Couldn''t save file ',aname);
end;

procedure TRegistry.SaveVariablesInFile(var out:text);
var h:pregister;
begin
  h:=Register;
  while h<>nil do begin
    h^.Save(out);
    h:=h^.next;
  end;
  if next<>NIL then next^.SaveVariablesInFile(out);
end;

procedure TRegistry.SendVariables(port:serialport);
var h:PRegister;
begin
  if next<>NIL then next^.SendVariables(port);
  h:=Register;
  while h<>NIL do begin
    h^.Send(port);
    h:=h^.next;
  end;
end;

procedure TRegistry.LoadVariables(aname:string);
var inp:text;
begin
  assign(inp,aname);
  {$I-}
  reset(inp);
  {$I+}
  if ioresult=0 then begin
    LoadVariablesFromFile(inp);
    close(inp);
  end else begin
    writeln('Register : LoadVariables : Couldn''t load file ',aname,^G);
    delay(500);
  end;
end;

procedure TRegistry.LoadVariablesFromFile(var inp:text);
var h:PRegister;
begin
  if Register<>NIL then begin
    h:=Register;
    while h<>nil do begin
      h^.Load(inp);
      h:=h^.next;
    end;
  end;
  if Next<>NIL then next^.LoadVariablesFromFile(inp);
end;

function TRegistry.GetRegister(aname:Namestr):PRegister;
var h:PRegister;
begin
  If Register<>nil then begin
    h:=Register^.GetRegister(aname);
    if h=nil then begin
      if next<>nil then GetRegister:=next^.GetRegister(aname)
      else GetRegister:=nil;
    end else GetRegister:=h;
  end else GetRegister:=nil;
end;

procedure TRegistry.SetValue(aname:string;value:string);
var h:PRegister;
begin
  h:=GetRegister(aname);
  if (h<>nil) then begin
    h^.ConvertFromString(value);
  end else begin
    writeln('Register : SetValue('+aname+','+value+') : '+aname+' not found');
    delay(500);
  end;
end;

procedure RegisterRegistry(aRegistry:PPRegistry);
begin
  inc(LastRegistry);
  RegistryList[lastRegistry]:=aRegistry;
end;

function MakeSelectMenu:PRegister;
var Menu:PMenuInfo;
    choice,iBack:integer;
    h:PRegister;
    n:byte;
begin
  h:=nil;
  repeat
    NewMenu(Menu);
    for n:=1 to LastRegistry do if RegistryList[n]^<>nil
    then RegistryList[n]^^.MakeMenuEntrys(menu,0,1);
    AddChoice(Menu,'Back                                         [B]',
	   'Back','B',iBack);
    DoMenu(Menu,' Select Parameter Main Menu ',Choice);
    if (choice<>iback) and (choice<>0) then begin
      n:=1;
      while (n<=LastRegistry) and (h=nil) and (RegistryList[n]^<>nil) do begin
        h:=RegistryList[n]^^.ExecuteSelectMenu(choice,1);
        inc(n);
      end;
    end;
  until (h<>nil) or (choice=0) or (choice=iBack);
  MakeSelectMenu:=h;
end;

procedure SendVariables(port:serialport);
var n:byte;
begin
  for n:=1 to LastRegistry do if RegistryList[n]^<>nil
  then RegistryList[n]^^.SendVariables(port);
end;

end.