unit sequence;
{
This unit is used to do the reordering of the commands which
are transmitted to the output ports. This is necesarry for
example if a shutter has a delay time.
}

interface

const SequenceOut:boolean=true;

type TProcNoArg=procedure;
     TProcDoubleArg=procedure(d:double);
     TProcIntArg=procedure(i:integer);
     TProcByteArg=procedure(b:byte);
     TProcByteDoubleArg=procedure(b:byte;d:double);
     TProcByteDoubleDoubleDoubleDoubleArg=procedure(b:byte;d1:double;d2:double;d3:double;d4:double);
     TProcByteDoubleBooleanArg=procedure(b:byte;d:double;bo:boolean);
     TProcIntDoubleArg=procedure(i:integer;d:double);
     TProcDoubleDoubleArg=procedure(d1:double;d2:double);
     TProcDoubleByteByteArg=procedure(d:double;b1:byte;b2:byte);
     TProcBooleanArg=procedure(d:boolean);
     TProcByteBooleanArg=procedure(b:byte;d:boolean);
     TProcDoubleBooleanArg=procedure(d:double;d:boolean);
     TProcPointerArg=procedure(p:Pointer);

procedure StartSequence;
procedure StopSequence;
procedure DoSequence;
procedure PauseSequence;
procedure UnPauseSequence;
procedure SetSequenceWaitProc(proc:TProcDoubleArg);
procedure SequenceStartBlockNoArg(proc:TPRocNoArg);
procedure SequenceStartBlockByteArg(proc:TPRocByteArg;arg:byte);
procedure SequenceStartBlockPointerArg(proc:TPRocPointerArg;arg:pointer);
procedure SequenceEndBlock;
procedure SequenceWait(t:double); {time in ms}
function GetSequenceTime:double;
procedure SequenceLongProc(proc:TPRocNoArg;delay:double);
procedure SequenceLongProcPointerArg(proc:TProcPointerArg;aptr:Pointer;delay:double);
procedure SequenceNoArg(proc:TProcNoArg;delay:double);
procedure SequenceDoubleArg(proc:TProcDoubleArg;arg:double;delay:double);
procedure SequencePointerArg(proc:TProcPointerArg;arg:Pointer;delay:double);
procedure SequenceIntArg(proc:TProcIntArg;arg:integer;delay:double);
procedure SequenceByteArg(proc:TProcByteArg;arg:byte;delay:double);
procedure SequenceByteDoubleArg(proc:TProcByteDoubleArg;barg:byte;darg:double;delay:double);
procedure SequenceByteDoubleDoubleDoubleDoubleArg(proc:TProcByteDoubleDoubleDoubleDoubleArg;
  barg:byte;darg1:double;darg2:double;darg3:double;darg4:double;delay:double);
procedure SequenceByteDoubleBooleanArg(proc:TProcByteDoubleBooleanArg;barg:byte;darg:double;boarg:boolean;delay:double);
procedure SequenceIntDoubleArg(proc:TProcIntDoubleArg;iarg:integer;darg:double;delay:double);
procedure SequenceDoubleDoubleArg(proc:TProcDoubleDoubleArg;darg1:double;darg2:double;delay:double);
procedure SequenceDoubleByteByteArg(proc:TProcDoubleByteByteArg;darg:double;barg1:byte;barg2:byte;delay:double);
procedure SequenceBooleanArg(proc:TProcBooleanArg;arg:boolean;delay:double);
procedure SequenceByteBooleanArg(proc:TProcByteBooleanArg;arg1:byte;arg2:boolean;delay:double);
procedure SequenceDoubleBooleanArg(proc:TProcDoubleBooleanArg;arg1:double;arg2:boolean;delay:double);

implementation
uses crt,timing,portlist;

var aktTime:double;
    Totaltime:double;
    aktRealTime:double;

type PList=^TList;
     TList=object
       next:PList;
       time:double;
       constructor init(aTime:double;var aList:PList);
       destructor done;
       procedure doIt; virtual;
     end;

     PListNoArg=^TListNoArg;
     TListNoArg=object(TList)
       Proc:TProcNoArg;
       constructor init(aProc:TProcNoArg;aTime:double;var aList:PList);
       destructor done;
       procedure doIt; virtual;
     end;

     PListDoubleArg=^TListDoubleArg;
     TListDoubleArg=object(TList)
       Proc:TProcDoubleArg;
       arg:double;
       constructor init(aProc:TProcDoubleArg;aArg:double;aTime:double;var aList:PList);
       destructor done;
       procedure doIt; virtual;
     end;

     PListIntArg=^TListIntArg;
     TListIntArg=object(TList)
       Proc:TProcIntArg;
       arg:Integer;
       constructor init(aProc:TProcIntArg;aArg:Integer;aTime:double;var aList:PList);
       destructor done;
       procedure doIt; virtual;
     end;

     PListByteArg=^TListByteArg;
     TListByteArg=object(TList)
       Proc:TProcByteArg;
       arg:Byte;
       constructor init(aProc:TProcByteArg;aArg:Byte;aTime:double;var aList:PList);
       destructor done;
       procedure doIt; virtual;
     end;

     PListByteDoubleArg=^TListByteDoubleArg;
     TListByteDoubleArg=object(TList)
       Proc:TProcByteDoubleArg;
       darg:double;
       barg:Byte;
       constructor init(aProc:TProcByteDoubleArg;Abarg:Byte;Adarg:double;aTime:double;var aList:PList);
       destructor done;
       procedure doIt; virtual;
     end;

     PListByteDoubleDoubleDoubleDoubleArg=^TListByteDoubleDoubleDoubleDoubleArg;
     TListByteDoubleDoubleDoubleDoubleArg=object(TList)
       Proc:TProcByteDoubleDoubleDoubleDoubleArg;
       darg1:double;
       darg2:double;
       darg3:double;
       darg4:double;
       barg:Byte;
       constructor init(aProc:TProcByteDoubleDoubleDoubleDoubleArg;
         Abarg:Byte;Adarg1:double;Adarg2:double;Adarg3:double;Adarg4:double;aTime:double;var aList:PList);
       destructor done;
       procedure doIt; virtual;
     end;


     PListByteDoubleBooleanArg=^TListByteDoubleBooleanArg;
     TListByteDoubleBooleanArg=object(TList)
       Proc:TProcByteDoubleBooleanArg;
       darg:double;
       barg:Byte;
       boarg:boolean;
       constructor init(aProc:TProcByteDoubleBooleanArg;Abarg:Byte;Adarg:double;Aboarg:boolean;aTime:double;var aList:PList);
       destructor done;
       procedure doIt; virtual;
     end;

     PListIntDoubleArg=^TListIntDoubleArg;
     TListIntDoubleArg=object(TList)
       Proc:TProcIntDoubleArg;
       darg:double;
       iarg:integer;
       constructor init(aProc:TProcIntDoubleArg;Aiarg:integer;Adarg:double;aTime:double;var aList:PList);
       destructor done;
       procedure doIt; virtual;
     end;

     PListDoubleDoubleArg=^TListDoubleDoubleArg;
     TListDoubleDoubleArg=object(TList)
       Proc:TProcDoubleDoubleArg;
       darg1,darg2:double;
       constructor init(aProc:TProcDoubleDoubleArg;Adarg1:double;Adarg2:double;aTime:double;var aList:PList);
       destructor done;
       procedure doIt; virtual;
     end;

     PListDoubleByteByteArg=^TListDoubleByteByteArg;
     TListDoubleByteByteArg=object(TList)
       Proc:TProcDoubleByteByteArg;
       darg:double;
       barg1:byte;
       barg2:byte;
       constructor init(aProc:TProcDoubleByteByteArg;Adarg:double;Abarg1:byte;Abarg2:byte;aTime:double;var aList:PList);
       destructor done;
       procedure doIt; virtual;
     end;

     PListBooleanArg=^TListBooleanArg;
     TListBooleanArg=object(TList)
       Proc:TProcBooleanArg;
       arg:boolean;
       constructor init(aProc:TProcBooleanArg;aArg:boolean;aTime:double;var aList:PList);
       destructor done;
       procedure doIt; virtual;
     end;

     PListByteBooleanArg=^TListByteBooleanArg;
     TListByteBooleanArg=object(TList)
       Proc:TProcByteBooleanArg;
       arg1:byte;
       arg2:boolean;
       constructor init(aProc:TProcByteBooleanArg;aArg1:byte;aArg2:boolean;aTime:double;var aList:PList);
       destructor done;
       procedure doIt; virtual;
     end;

     PListDoubleBooleanArg=^TListDoubleBooleanArg;
     TListDoubleBooleanArg=object(TList)
       Proc:TProcDoubleBooleanArg;
       arg1:Double;
       arg2:boolean;
       constructor init(aProc:TProcDoubleBooleanArg;aArg1:Double;aArg2:boolean;aTime:double;var aList:PList);
       destructor done;
       procedure doIt; virtual;
     end;

     PListPointerArg=^TListPointerArg;
     TListPointerArg=object(TList)
       Proc:TProcPointerArg;
       arg:Pointer;
       constructor init(aProc:TProcPointerArg;aArg:Pointer;aTime:double;var aList:PList);
       destructor done;
       procedure doIt; virtual;
     end;

const Liste:PList=nil;
      SequenceBlock:boolean=false;
      InSequence:boolean=false;

procedure dummy(t:double);far;
begin
  writeln('Sequence : WaitProc not set');
end;

const SequenceWaitProc:TProcDoubleArg=dummy;

procedure SetSequenceWaitProc(proc:TProcDoubleArg);
begin
  SequenceWaitProc:=proc;
end;

constructor TList.init(aTime:double;var aList:PList);
var aktlist:PList;
begin
  CheckCounter;
  time:=atime;
  if (aList<>nil) then begin
    aktList:=aList;
    while (aktList^.next<>nil) and (aktList^.next^.time<=time) do aktList:=aktList^.next;
    next:=aktlist^.next;
    aktList^.next:=@self;
  end else begin
    next:=nil;
    aList:=@self;
  end;
end;

destructor TList.done;
begin
  if next<>nil then dispose(next,done);
  next:=nil;
end;

procedure TList.doIt;
begin
end;

constructor TListNoArg.init(aProc:TProcNoArg;aTime:double;var aList:PList);
begin
  TList.Init(aTime,aList);
  Proc:=aProc;
end;

destructor TListNoArg.done;
begin
  TList.done;
end;

constructor TListDoubleArg.init(aProc:TProcDoubleArg;aArg:double;aTime:double;var aList:PList);
begin
  TList.Init(aTime,aList);
  Proc:=aProc;
  Arg:=aArg;
end;

destructor TListDoubleArg.done;
begin
  TList.done;
end;

constructor TListPointerArg.init(aProc:TProcPointerArg;aArg:Pointer;aTime:double;var aList:PList);
begin
  TList.Init(aTime,aList);
  Proc:=aProc;
  Arg:=aArg;
end;

destructor TListPointerArg.done;
begin
  TList.done;
end;

constructor TListIntArg.init(aProc:TProcIntArg;aArg:Integer;aTime:double;var aList:PList);
begin
  TList.Init(aTime,aList);
  Proc:=aProc;
  Arg:=aArg;
end;

destructor TListIntArg.done;
begin
  TList.done;
end;

constructor TListByteArg.init(aProc:TProcByteArg;aArg:Byte;aTime:double;var aList:PList);
begin
  TList.Init(aTime,aList);
  Proc:=aProc;
  Arg:=aArg;
end;

destructor TListByteArg.done;
begin
  TList.done;
end;

constructor TListByteDoubleBooleanArg.init(aProc:TProcByteDoubleBooleanArg;
  Abarg:Byte;Adarg:double;Aboarg:boolean;aTime:double;var aList:PList);
begin
  TList.Init(aTime,aList);
  Proc:=aProc;
  dArg:=Adarg;
  bArg:=Abarg;
  boArg:=Aboarg;
end;

destructor TListByteDoubleBooleanArg.done;
begin
  TList.done;
end;

constructor TListByteDoubleArg.init(aProc:TProcByteDoubleArg;Abarg:Byte;Adarg:double;aTime:double;var aList:PList);
begin
  TList.Init(aTime,aList);
  Proc:=aProc;
  dArg:=Adarg;
  bArg:=Abarg;
end;

destructor TListByteDoubleArg.done;
begin
  TList.done;
end;

constructor TListByteDoubleDoubleDoubleDoubleArg.init(aProc:TProcByteDoubleDoubleDoubleDoubleArg;
  Abarg:Byte;Adarg1:double;Adarg2:double;Adarg3:double;Adarg4:double;aTime:double;var aList:PList);
begin
  TList.Init(aTime,aList);
  Proc:=aProc;
  dArg1:=Adarg1;
  dArg2:=Adarg2;
  dArg3:=Adarg3;
  dArg4:=Adarg4;
  bArg:=Abarg;
end;

destructor TListByteDoubleDoubleDoubleDoubleArg.done;
begin
  TList.done;
end;


constructor TListIntDoubleArg.init(aProc:TProcIntDoubleArg;Aiarg:integer;Adarg:double;aTime:double;var aList:PList);
begin
  TList.Init(aTime,aList);
  Proc:=aProc;
  dArg:=Adarg;
  iArg:=Aiarg;
end;

destructor TListIntDoubleArg.done;
begin
  TList.done;
end;

constructor TListDoubleDoubleArg.init(aProc:TProcDoubleDoubleArg;Adarg1:double;Adarg2:double;aTime:double;var aList:PList);
begin
  TList.Init(aTime,aList);
  Proc:=aProc;
  dArg1:=Adarg1;
  dArg2:=Adarg2;
end;

destructor TListDoubleDoubleArg.done;
begin
  TList.done;
end;

constructor TListDoubleByteByteArg.init(aProc:TProcDoubleByteByteArg;
  Adarg:double;Abarg1:byte;Abarg2:byte;aTime:double;var aList:PList);
begin
  TList.Init(aTime,aList);
  Proc:=aProc;
  dArg:=Adarg;
  bArg1:=Abarg1;
  bArg2:=Abarg2;
end;

destructor TListDoubleByteByteArg.done;
begin
  TList.done;
end;

constructor TListBooleanArg.init(aProc:TProcBooleanArg;aArg:boolean;aTime:double;var aList:PList);
begin
  TList.Init(aTime,aList);
  Proc:=aProc;
  Arg:=aArg;
end;

destructor TListBooleanArg.done;
begin
  TList.done;
end;

constructor TListByteBooleanArg.init(aProc:TProcByteBooleanArg;aArg1:byte;aArg2:boolean;aTime:double;var aList:PList);
begin
  TList.Init(aTime,aList);
  Proc:=aProc;
  Arg1:=aArg1;
  Arg2:=aArg2;
end;

destructor TListByteBooleanArg.done;
begin
  TList.done;
end;

procedure TListNoArg.doIt;
begin
  Proc;
end;

constructor TListDoubleBooleanArg.init(aProc:TProcDoubleBooleanArg;aArg1:Double;aArg2:boolean;aTime:double;var aList:PList);
begin
  TList.Init(aTime,aList);
  Proc:=aProc;
  Arg1:=aArg1;
  Arg2:=aArg2;
end;

destructor TListDoubleBooleanArg.done;
begin
  TList.done;
end;

procedure TListDoubleBooleanArg.doIt;
begin
  Proc(arg1,arg2);
end;

procedure TListDoubleArg.doIt;
begin
  Proc(arg);
end;

procedure TListPointerArg.doIt;
begin
  Proc(arg);
end;

procedure TListIntArg.doIt;
begin
  Proc(arg);
end;

procedure TListByteArg.doIt;
begin
  Proc(arg);
end;

procedure TListByteDoubleArg.doIt;
begin
  Proc(barg,darg);
end;

procedure TListByteDoubleDoubleDoubleDoubleArg.doIt;
begin
  Proc(barg,darg1,darg2,darg3,darg4);
end;


procedure TListByteDoubleBooleanArg.doIt;
begin
  Proc(barg,darg,boarg);
end;

procedure TListIntDoubleArg.doIt;
begin
  Proc(iarg,darg);
end;

procedure TListDoubleDoubleArg.doIt;
begin
  Proc(darg1,darg2);
end;

procedure TListDoubleByteByteArg.doIt;
begin
  Proc(darg,barg1,barg2);
end;

procedure TListBooleanArg.doIt;
begin
  Proc(arg);
end;

procedure TListByteBooleanArg.doIt;
begin
  Proc(arg1,arg2);
end;

procedure StartSequence;
begin
  SequenceBlock:=false;
  if liste<>nil then dispose(liste,done);
  liste:=nil;
  SequenceOut:=false;
  aktTime:=0;
  Totaltime:=0;
  InSequence:=true;
end;

procedure StopSequence;
begin
  SequenceBlock:=false;
  if Liste<>nil then DoSequence;
  SequenceOut:=true;
  InSequence:=false;
end;

procedure PauseSequence;
begin
  SequenceOut:=true;
end;

procedure UnPauseSequence;
begin
  SequenceOut:=false;
end;

procedure DoSequence;
var oldTime:double;
    hList:PList;
begin
  SequenceBlock:=false;
  InSequence:=false;
  if Liste=nil then begin
    writeln('Sequence : DoSequence : Sequence is empty');
  end else begin
    SequenceOut:=true;
    hList:=Liste;
    oldTime:=0;
    while (hList<>nil) do begin
      if (oldtime<>hList^.time) then SequenceWaitProc(hList^.time-oldtime);
      hList^.DoIt;
      oldTime:=hList^.time;
      hList:=hList^.next;
    end;
    if (oldTime<>aktTime) then SequenceWaitProc(aktTime-oldTime);
    StartSequence;
  end;
end;

procedure SequenceWait(t:double); {time in ms}
begin
  if not SequenceBlock then akttime:=akttime+t;
  TotalTime:=Totaltime+t
end;

function GetSequenceTime:double;
begin
  GetSequenceTime:=Totaltime;
end;

procedure SequenceStartBlockNoArg(proc:TPRocNoArg);
begin
  if InSequence then begin
    SequenceNoArg(proc,0);
    if SequenceBlock then begin
      writeln('Sequence: SequenceStartBlock : Block already started');
      delay(1000);
    end;
    SequenceBlock:=true;
  end;
end;

procedure SequenceStartBlockByteArg(proc:TPRocByteArg;arg:byte);
begin
  if InSequence then begin
    SequenceByteArg(proc,arg,0);
    if SequenceBlock then begin
      writeln('Sequence: SequenceStartBlockByteArg : Block already started');
      delay(1000);
    end;
    SequenceBlock:=true;
  end;
end;

procedure SequenceStartBlockPointerArg(proc:TPRocPointerArg;arg:pointer);
begin
  if InSequence then begin
    SequencePointerArg(proc,arg,0);
    if SequenceBlock then begin
      writeln('Sequence: SequenceStartBlockPointerArg : Block already started');
      delay(1000);
    end;
    SequenceBlock:=true;
  end;
end;

procedure SequenceEndBlock;
begin
  if InSequence then begin
    if not SequenceBlock then begin
      writeln('Sequence: SequenceEndBlock : Block not started');
      delay(1000);
    end;
    SequenceBlock:=false;
  end;
end;

procedure SequenceLongProc(proc:TProcNoArg;delay:double);
begin
  if not SequenceBlock then begin
    SequenceNoArg(proc,0);
    SequenceWait(delay);
  end;
end;

procedure SequenceLongProcPointerArg(proc:TProcPointerArg;aptr:Pointer;delay:double);
begin
  if not SequenceBlock then begin
    SequencePointerArg(proc,aptr,0);
    SequenceWait(delay);
  end;
end;

procedure SequenceNoArg(proc:TProcNoArg;delay:double);
var ListNoArg:PListNoArg;
begin
  if not SequenceBlock then begin
    if (maxavail>sizeof(TListNoArg))
    then new(ListNoArg,init(proc,aktTime+delay,Liste))
    else writeln('Sequence : NoArg : not sufficient memory');
  end;
end;

procedure SequenceDoubleArg(proc:TProcDoubleArg;arg:double;delay:double);
var ListDoubleArg:PListDoubleArg;
begin
  if not SequenceBlock then begin
    if (maxavail>sizeof(TListDoubleArg))
    then new(ListDoubleArg,init(proc,arg,aktTime+delay,Liste))
    else writeln('Sequence : DoubleArg : not sufficient memory');
  end;
end;

procedure SequencePointerArg(proc:TProcPointerArg;arg:Pointer;delay:double);
var ListPointerArg:PListPointerArg;
begin
  if not SequenceBlock then begin
    if (maxavail>sizeof(TListPointerArg))
    then new(ListPointerArg,init(proc,arg,aktTime+delay,Liste))
    else writeln('Sequence : PointerArg : not sufficient memory');
  end;
end;

procedure SequenceIntArg(proc:TProcIntArg;arg:integer;delay:double);
var ListIntArg:PListIntArg;
begin
  if not SequenceBlock then begin
    if (maxavail>sizeof(TListIntArg))
    then new(ListIntArg,init(proc,arg,aktTime+delay,Liste))
    else writeln('Sequence : IntArg : not sufficient memory');
  end;
end;

procedure SequenceByteArg(proc:TProcByteArg;arg:Byte;delay:double);
var ListByteArg:PListByteArg;
begin
  if not SequenceBlock then begin
    if (maxavail>sizeof(TListByteArg))
    then new(ListByteArg,init(proc,arg,aktTime+delay,Liste))
    else writeln('Sequence : ByteArg : not sufficient memory');
  end;
end;

procedure SequenceByteDoubleArg(proc:TProcByteDoubleArg;barg:byte;darg:double;delay:double);
var ListByteDoubleArg:PListByteDoubleArg;
begin
  if not SequenceBlock then begin
    if (maxavail>sizeof(TListByteDoubleArg))
    then new(ListByteDoubleArg,init(proc,barg,darg,aktTime+delay,Liste))
    else writeln('Sequence : ByteDoubleArg : not sufficient memory');
  end;
end;

procedure SequenceByteDoubleDoubleDoubleDoubleArg(proc:TProcByteDoubleDoubleDoubleDoubleArg;
  barg:byte;darg1:double;darg2:double;darg3:double;darg4:double;delay:double);
var ListByteDoubleDoubleDoubleDoubleArg:PListByteDoubleDoubleDoubleDoubleArg;
begin
  if not SequenceBlock then begin
    if (maxavail>sizeof(TListByteDoubleDoubleDoubleDoubleArg))
    then new(ListByteDoubleDoubleDoubleDoubleArg,init(proc,barg,darg1,darg2,darg3,darg4,aktTime+delay,Liste))
    else begin
      writeln('Sequence : ByteDoubleDoubleDoubleDoubleArg : not sufficient memory');
    end;
  end;
end;

procedure SequenceByteDoubleBooleanArg(proc:TProcByteDoubleBooleanArg;barg:byte;darg:double;boarg:boolean;delay:double);
var ListByteDoubleBooleanArg:PListByteDoubleBooleanArg;
begin
  if not SequenceBlock then begin
    if (maxavail>sizeof(TListByteDoubleBooleanArg))
    then new(ListByteDoubleBooleanArg,init(proc,barg,darg,boarg,aktTime+delay,Liste))
    else writeln('Sequence : ByteDoubleBooleanArg : not sufficient memory');
  end;
end;

procedure SequenceIntDoubleArg(proc:TProcIntDoubleArg;iarg:integer;darg:double;delay:double);
var ListIntDoubleArg:PListIntDoubleArg;
begin
  if not SequenceBlock then begin
    if (maxavail>sizeof(TListIntDoubleArg))
    then new(ListIntDoubleArg,init(proc,iarg,darg,aktTime+delay,Liste))
    else writeln('Sequence : IntDoubleArg : not sufficient memory');
  end;
end;

procedure SequenceDoubleDoubleArg(proc:TProcDoubleDoubleArg;darg1:double;darg2:double;delay:double);
var ListDoubleDoubleArg:PListDoubleDoubleArg;
begin
  if not SequenceBlock then begin
    if (maxavail>sizeof(TListDoubleDoubleArg))
    then new(ListDoubleDoubleArg,init(proc,darg1,darg2,aktTime+delay,Liste))
    else writeln('Sequence : DoubleDoubleArg : not sufficient memory');
  end;
end;

procedure SequenceDoubleByteByteArg(proc:TProcDoubleByteByteArg;darg:double;barg1:byte;barg2:byte;delay:double);
var ListDoubleByteByteArg:PListDoubleByteByteArg;
begin
  if not SequenceBlock then begin
    if (maxavail>sizeof(TListDoubleByteByteArg))
    then new(ListDoubleByteByteArg,init(proc,darg,barg1,barg2,aktTime+delay,Liste))
    else writeln('Sequence : DoubleByteByteArg : not sufficient memory');
  end;
end;

procedure SequenceBooleanArg(proc:TProcBooleanArg;arg:boolean;delay:double);
var ListBooleanArg:PListBooleanArg;
begin
  if not SequenceBlock then begin
    if (maxavail>sizeof(TListBooleanArg))
    then new(ListBooleanArg,init(proc,arg,aktTime+delay,Liste))
    else writeln('Sequence : BooleanArg : not sufficient memory');
  end;
end;

procedure SequenceByteBooleanArg(proc:TProcByteBooleanArg;arg1:byte;arg2:boolean;delay:double);
var ListByteBooleanArg:PListByteBooleanArg;
begin
  if not SequenceBlock then begin
    if (maxavail>sizeof(TListByteBooleanArg))
    then new(ListByteBooleanArg,init(proc,arg1,arg2,aktTime+delay,Liste))
    else writeln('Sequence : ByteBooleanArg : not sufficient memory');
  end;
end;

procedure SequenceDoubleBooleanArg(proc:TProcDoubleBooleanArg;arg1:Double;arg2:boolean;delay:double);
var ListDoubleBooleanArg:PListDoubleBooleanArg;
begin
  if not SequenceBlock then begin
    if (maxavail>sizeof(TListDoubleBooleanArg))
    then new(ListDoubleBooleanArg,init(proc,arg1,arg2,aktTime+delay,Liste))
    else writeln('Sequence : DoubleBooleanArg : not sufficient memory');
  end;
end;

end.
