unit Serial;
interface
type ParityType = (Podd,Peven,Pnone);

procedure OpenPort(PortNr:byte; Speed:longint);
procedure ClosePort(Pt:byte);
procedure WaitForSend(Pt:byte);
function PutChar(Pt:byte;C: char):boolean;
function GetChar(Pt:byte;var C: char):boolean;
procedure PutLine(Pt: byte; OutStr: string);
function GetLine(Pt : byte;endchar:char;maxlength:byte):string;
procedure ClearInQueue(Pt:byte);
procedure PrintBuffer(Pt:byte);

implementation
uses crt,debug;

const
  TxDataReg = 0;                { transmitter data register            }
  RxDataReg = 0;                { reciever data register               }
  DivLow = 0;                   { divisor latch, low byte              }
  DivHigh = 1;                  { divisor latch, high byte             }
  IntrEnable = 1;               { interrupt enable register            }
  IntrId = 2;                   { interrupt identification register    }
  FifoCtrl = 2;                 { first-in/first-out buffer controller }
  LineCtrl = 3;                 { line controll register               }
  ModemCtrl = 4;                { modem controll register              }
  LineStatus = 5;               { line status register                 }
  ModemStatus = 6;              { modem status register                }
  ScratchReg = 7;               { scratch pad (free useable)           }

const
  QueueLen =512;              { Length of a queue in bytes }
type
  { The TQueue type }
  TQueue = record
    Start: word;
    Stop: word;
    Content:array[0..QueueLen - 1] of char;
    end;

type
  { The TQuadString type }
  TQuadString = string[4];      { For the Hex-function }

const
  SpeedCount = 19;              { Number of valid speeds stored below }

const
  AllowedSpeeds: array[0..SpeedCount] of longint =
    (    50,      75,    110,    150,    300,    600,     900,    1200,
       1800,    2400,   3600,   4800,   7200,   9600,   14400,   19200,
      28800,   38400,  57600, 115200);

    { Just add any other speeds supported by your UART/Modem - don't }
    { forget to increase the SpeedCount                              }


{ Some global variables... }
Type TCOM= record
  { The two queues, one for incoming characters, the other for }
  { outgoing characters.                                       }
  PortBase: word;               { I/O base address of the used port         }
  Speed: longint;               { Speed in baud                             }
  UARTType: byte;               { UART type (one of the UART_xxxx constants }
  IRQ: byte;                { The number of the used IRQ                }
  IntSet:boolean;
  OldInterruptVec: pointer;
  InQueue: TQueue;              { The two queues: one for input buffering, }
  OutQueue: TQueue;             { the other for output buffering           }
end;

var COMData:array[1..4] of TCOM;

procedure EnableIRQ(IRQ: byte); assembler;
{ Enable a given IRQ from 0 to 7 }
asm
        mov     cl, IRQ
        mov     bl, 1
        shl     bl, cl
        mov     ah, 255
        sub     ah, bl
        in      al, 21h
        and     al, ah
        out     21h, al
end;

procedure DisableIRQ(IRQ: byte); assembler;
{ Disable a given IRQ from 0 to 7 }
asm
        mov     cl, IRQ
        mov     bl, 1
        shl     bl, cl
        in      al, 21h
        or      al, bl
        out     21h, al
end;

procedure SendEOI; assembler;
{ Send a EOI to the Interrupt Controller }
asm
        mov     al, 20h
        out     20h, al
end;

function GetPortBase(N: byte): word; assembler;
{ Read the I/O base address of the desired port from the BIOS data segment }
asm
        { Load the segment address of the BIOS data segment into ES }
        mov     ax, 0040h
        mov     es, ax
        { Calculate the offset of the I/O port base address }
        xor     ax, ax
        mov     al, N
        dec     al
        shl     ax, 1
        mov     si, ax
        { Read the desired value and return }
        mov     ax, es:[si]
end;

procedure SetIntVec(N: byte; P: pointer); assembler;
{ Set an interrupt vector to the given address }
asm
        push    ds
        { Just use the MS-DOS function 25h to set the vector }
        mov     ah, 25h
        mov     al, N
        { ds:dx contains the vector to set }
        lds     dx, P
        int     21h
        pop     ds
end;

function GetIntVec(N: byte): pointer; assembler;
{ Get the interrupt vector }
asm
        push    es
        { Use the MS-DOS function 35h to read the vector }
        mov     ah, 35h
        mov     al, N
        int     21h
        { Move the vector from es:bx to dx:ax }
        mov     ax, bx
        mov     dx, es
        pop     es
end;

procedure ResetQueue(var Q: TQueue);
{ Resets a Queue }
begin
  Q.Start := 0;
  Q.Stop := 0;
end;

function PutQueueByte(var Q: TQueue; B: byte;IRQ:byte;inout:byte):boolean;
{ Put a byte into the Queue }
var Length:word;
begin
  DisableIRQ(IRQ);
  if Q.Start<=Q.Stop then begin
    Length:=Q.Stop-Q.Start;
  end else begin
    Length:=QueueLen-Q.Start+Q.Stop;
  end;
  {if ((inout=1) and (Length>0)) then begin
    PutQueueByte:=false;
    EnableIRQ(IRQ);
    exit;
  end;}
  if (Length<QueueLen-2) then begin
    PutQueueByte:=true;
    { Put the byte into the Queue before incrementing the Queue end }
    Q.Content[Q.Stop] := char(B);

    {if inout=0 then TextColor(lightred) else TextColor(red);
    write(char(b));
    TextColor(Yellow);}

    { Now increment the Queue end position. When Q.Stop reaches QueueLen, }
    { be sure you don't increment Q.Stop but wrap it around to 0 again !  }
    if (Q.Stop = (QueueLen-1)) then
      Q.Stop := 0 else inc(Q.Stop);
  end else PutQueueByte:=false;
  EnableIRQ(IRQ);
end;

function GetQueueByte(var Q: TQueue; var B: byte;IRQ:byte;inout:byte): boolean;
{ Get a byte out of the Queue }
var Length:word;
begin
  DisableIRQ(IRQ);
  if Q.Start<=Q.Stop then begin
    Length:=Q.Stop-Q.Start;
  end else begin
    Length:=QueueLen-Q.Start+Q.Stop;
  end;
  if (Length=0) then
    { If the Queue is empty, just return false. Don't set B anyway. }
    GetQueueByte := false
  else begin
    { Queue is not empty: return true. }
    GetQueueByte := true;

    { Get the first byte out of the Queue and return it in B. }
    B := byte(Q.Content[Q.Start]);

    {if inout=0 then TextColor(lightgreen) else TextColor(green);
    write(char(b));
    TextColor(Yellow);}

    { Now increment the Queue position. Be sure to wrap it to zero }
    { if the Q.Start field reaches the QueueLen constant.         }

    if (Q.Start = (QueueLen-1)) then
      Q.Start := 0 else inc(Q.Start);
  end;
  EnableIRQ(IRQ);
end;

procedure SerialInterruptGeneral(pt:byte);
{ This procedure handles any incoming events from the UART. }
var
  Id:array[1..4] of byte;
  Trash:array[1..4] of byte;
begin

  DisableIRQ(COMData[Pt].IRQ);
  repeat
    { Now read the Interrupt Identification register }
    Id[Pt] := Port[COMData[Pt].PortBase + IntrId];


    { Check if there's any pending interrupt. }
    if ((Id[Pt] and 1) = 0) then
      begin

        { Now select the event. }
        case ((Id[Pt] and 6) shr 1) of
          $03:
            { The Line Status register changed. }
            begin
              { Just read the LSR to clear the event. }
              Trash[Pt] := Port[COMData[Pt].PortBase + LineStatus];
            end;
          $02:
            { Data arrived at the UART. }
            begin
              { Read out the data from the RxD register and store }
              { it in the incoming Queue.                         }
              Trash[Pt] := Port[COMData[Pt].PortBase + RxDataReg];
              PutQueueByte(COMData[Pt].InQueue, Trash[Pt],COMData[Pt].IRQ,0);
            end;
          $01:
            { The TxD register is empty. }
            begin
              { If there's any byte in the outgoing Queue, send it to the }
              { UART, else disable this interrupt.                        }
              if (GetQueueByte(COMData[Pt].OutQueue, Trash[Pt],COMData[Pt].IRQ,1)) then begin
                Port[COMData[Pt].PortBase + TxDataReg] := Trash[Pt];
              end else
                Port[COMData[Pt].PortBase + IntrEnable] := $0D;
            end;
          $00:
            { The Modem Status register changed. }
            begin
              { Just read the MSR to clear the event. }
              Trash[Pt] := Port[COMData[Pt].PortBase + ModemStatus];
            end;
          end;
        end;
    { Handle all interrupts ! Just check if there's }
    { one more interrupt pending.                   }
  until ((Id[Pt] and 1) = 1);
  { Now tell the PIC our interrupt handler has finished it's work. }
  SendEOI;
  EnableIRQ(COMData[Pt].IRQ);
end;

procedure SerialInterruptCOM1;interrupt;
begin
  SerialInterruptGeneral(1);
end;

procedure SerialInterruptCOM2; interrupt;
begin
  SerialInterruptGeneral(2);
end;

procedure SerialInterruptCOM3; interrupt;
begin
  SerialInterruptGeneral(3);
end;

procedure SerialInterruptCOM4; interrupt;
begin
  SerialInterruptGeneral(4);
end;

function PutChar(Pt:byte;C: char):boolean;
{ Send a char to the modem }
var timeout:word;
begin
  {PutChar:=PutQueueByte(COMData[Pt].OutQueue, byte(C));
  Port[PortBase + IntrEnable] := $0F;}
  timeout:=0;
  { Put the character into the outgoing queue. }
  while (not PutQueueByte(COMData[Pt].OutQueue, byte(C),COMData[Pt].IRQ,1)) and (timeout<1000) do begin
    inc(Timeout);
    delay(1);
  end;
  { Enable the "Transmitter register empty" interrupt }
  if (TimeOut<1000) then Port[COMData[Pt].PortBase + IntrEnable] := $0F
  else begin
    writeln('Serial:Putchar:timeout'+^G);
    Delay(1000);
  end;

  WaitForSend(Pt);

end;

const
  UART_Bad = 0;                 { Bad UART: not working or unidentifieable }
  UART_8250 = 1;                { Standart 8250 UART                       }
  UART_16450 = 2;               { 16450 UART (faster than 8250)            }
  UART_16550 = 3;               { 16550 UART (with buggy 16-byte FIFO)     }
  UART_16550A = 4;              { 16550A UART (with working FIFO)          }

function GetUARTType(Base: word): byte; assembler;
{ Check which UART type is assigned to the appropriate port }
asm
        { First difference: The 16450 has a scratch register which is }
        { readable and writeable. Check if it's there. If not, we've  }
        { got a 8250 UART.                                            }
        mov     dx, Base
        add     dx, ScratchReg
        mov     al, 0AAh
        out     dx, al
        in      al, dx
        cmp     al, 0AAh
        je      @@1
        mov     ax, UART_8250
        jmp     @@5
@@1:    { Now check out if the UART has got a FIFO. If it has none, it's  }
        { a 16450, if it has one but it's not working it's a 16550.       }
        { The UART will be identified as a 16550A if the FIFO is working. }
        mov     dx, Base
        add     dx, FifoCtrl
        mov     al, 01h
        out     dx, al
        nop
        mov     dx, Base
        add     dx, IntrId
        in      al, dx
        and     al, 0C0h
        cmp     al, 0C0h
        jne     @@2
        mov     al, UART_16550A
        jmp     @@5
@@2:    cmp     al, 80h
        jne     @@3
        mov     al, UART_16550
        jmp     @@5
@@3:    cmp     al, 0
        jne     @@4
        mov     al, UART_16450
        jmp     @@5
@@4:    mov     al, UART_Bad
@@5:    nop
end;

procedure UpCaseStr(var S: string); assembler;
{ Convert all chars in a string to uppercase letters }
asm
        les     di, S
        xor     cx, cx
        mov     cl, es:[di]
        inc     di
@@1:    mov     al, es:[di]
        cmp     al, 'a'
        jb      @@2
        cmp     al, 'z'
        ja      @@2
        { Chars between 'a' and 'z' will be uppercased here. }
        sub     al, 20h
@@2:    mov     es:[di], al
        inc     di
        loop    @@1
end;

function IsAllowedSpeed(Speed: longint): boolean;
{ Check if a speed is valid or not }
var
  I: byte;
begin
  { Return false by default. }
  IsAllowedSpeed := false;

  { Check if you can find the desired speed in the speeds table. }
  { If you found it, return "true".                              }
  for I := 0 to SpeedCount do
    if (AllowedSpeeds[I] = Speed) then
      begin
        IsAllowedSpeed := true;
        exit;
      end;

  { The default value ("false") will be returned if the speed wasn't }
  { found in the table.                                              }
end;

procedure SetupPort(Pt: byte);
{ Setup the UART and prepare for communication. }
var
  D: word;
  B: byte;
begin
  if COMData[Pt].IntSet then exit;
  { For startup, disable the IRQ for the UART. }
  DisableIRQ(COMData[Pt].IRQ);

  { Get the address of the old interrupt handler and set the vector }
  { to our won interrupt handling procedure ("SerialInterrupt")     }
  COMData[Pt].OldInterruptVec := GetIntVec($08 + COMData[Pt].IRQ);
  case Pt of
   1:SetIntVec($08 + COMData[Pt].IRQ, @SerialInterruptCOM1);
   2:SetIntVec($08 + COMData[Pt].IRQ, @SerialInterruptCOM2);
   3:SetIntVec($08 + COMData[Pt].IRQ, @SerialInterruptCOM3);
   4:SetIntVec($08 + COMData[Pt].IRQ, @SerialInterruptCOM4);
  end;
  COMData[Pt].IntSet:=true;

  { Enable the "Recieved Data avaliable" interrupt so we can }
  { read all data out of the UART's RxD register.            }
  Port[COMData[Pt].PortBase + IntrEnable] := $01;

  { Now clear all pending interrupts - if any }
  repeat
    { Read all important registers to clear any interrupt types, }
    { B is just used for temporary result storage                }
    B := Port[COMData[Pt].PortBase + RxDataReg];
    B := Port[COMData[Pt].PortBase + LineStatus];
    B := Port[COMData[Pt].PortBase + ModemStatus];

    { Just repeat until no more interrupts are pending. }
  until ((Port[COMData[Pt].PortBase + IntrId] and 1) = 1);

  { Enable the IRQ line for the UART after all pending interrupts }
  { have been cleared.                                            }
  EnableIRQ(COMData[Pt].IRQ);

  { Calculate the divisor latch contents for the desired baud rate }
  D := (115200 div COMData[Pt].Speed);

  { Set the DLAB bit to 1, then write the divisor latch low and high bytes }
  Port[COMData[Pt].PortBase + LineCtrl] := $80;
  Port[COMData[Pt].PortBase + DivLow] := Lo(D);
  Port[COMData[Pt].PortBase + DivHigh] := Hi(D);

  { Now set the divisor latch bit to 0 and write all other values }
  Port[COMData[Pt].PortBase + LineCtrl] := $03;
  Port[COMData[Pt].PortBase + ModemCtrl] := $0B;
  Port[COMData[Pt].PortBase + IntrEnable] := $0F;

  { Check if a 16550A UART is present... }
  if (COMData[Pt].UARTType = UART_16550A) then
    begin
      { Clear the FIFO queues }
      Port[COMData[Pt].PortBase + FifoCtrl] := $07;

      { Enable the FIFO queues }
      Port[COMData[Pt].PortBase + FifoCtrl] := $C1;

      { Print a message so the user recognizes the FIFO queues are on }
      writeln('Enabling 16550A FIFO queues...');
      delay(100);
    end;
end;

procedure ClosePort(Pt:byte);
{ Reset UART and reset interrupt vectors }
begin
  if ComData[Pt].IntSet then begin
    { Reset the DLAB bit. Clear all other registers. }
    Port[ComData[Pt].PortBase + LineCtrl] := Port[ComData[Pt].PortBase + LineCtrl] and $7F;
    Port[ComData[Pt].PortBase + ModemCtrl] := 0;
    Port[ComData[Pt].PortBase + IntrEnable] := 0;

    { Disable the IRQ for the UART, then restore the old interrupt vector. }
    DisableIRQ(ComData[Pt].IRQ);
    SetIntVec($08 + ComData[Pt].IRQ, ComData[Pt].OldInterruptVec);
    ComData[Pt].IntSet:=false;
  end;
end;

procedure WaitForSend(Pt:byte);
var timeout:longint;
    Length:word;
begin
  timeout:=0;
  with ComData[Pt].OutQueue do begin
    repeat
      if Start<=Stop then begin
        Length:=Stop-Start;
      end else begin
        Length:=QueueLen-Start+Stop;
      end;
      inc(timeout);
      if length>0 then delay(1);
    until (Length=0) or (Timeout>1000);
  end;
  if timeout>1000 then begin
    writeln('Serial:WaitForSend:Timeout',^G);
    delay(1000);
  end;
end;

procedure FlushSerial(Pt:byte);
begin
{ Reset both the incoming as well as the outgoing Queue. }
  ResetQueue(COMData[Pt].InQueue);
  ResetQueue(COMData[Pt].OutQueue);
end;

procedure ClearInQueue(Pt:byte);
begin
  {if COMData[Pt].InQueue.Start<>COMData[Pt].InQueue.Stop
  then begin
   writeln('err');
    Printbuffer(1);
    readln;
  end;}
  COMData[Pt].InQueue.Start:=COMData[Pt].InQueue.Stop;
end;

procedure OpenPort(PortNr:byte; Speed:longint);
begin
  if ComData[PortNr].IntSet then begin
    writeln('Serial:OpenPort: COM', PortNr, ': already used'+^G);
    delay(1000);
    exit;
  end;
  { Check for some correct parameters on the command line. }
  if (PortNr<=4) and (PortNr>0) then COMData[PortNr].PortBase := GetPortBase(PortNr);
  if (COMData[PortNr].PortBase = 0) or (PortNr>4) or (PortNr=0) then
    begin
      writeln('Serial:OpenPort: COM', PortNr, ': no such port'+^G);
      Delay(1000);
      exit;
    end;
  COMData[PortNr].IRQ := 4 - ((PortNr - 1) and 1);
  COMData[PortNr].UARTType := GetUARTType(COMData[PortNr].PortBase);

  FlushSerial(PortNr);

  { Setup the port, run the terminal and reset the port when ready. }
  COMData[PortNr].Speed:=Speed;
  SetupPort(PortNr);
end;

function GetChar(Pt:byte;var C: char):boolean;
begin
  GetChar:=GetQueueByte(COMData[Pt].InQueue, byte(c),COMData[Pt].IRQ,0);
end;

function PutCharToQueue(Pt:byte;C: char):boolean;
{ Send a char to the modem }
var timeout:word;
begin
  {PutChar:=PutQueueByte(COMData[Pt].OutQueue, byte(C));
  Port[PortBase + IntrEnable] := $0F;}
  timeout:=0;
  { Put the character into the outgoing queue. }
  while (not PutQueueByte(COMData[Pt].OutQueue, byte(C),COMData[Pt].IRQ,1)) and (timeout<1000) do begin
    inc(Timeout);
    delay(1);
  end;
  if (Timeout=1000) then begin
    writeln('Serial:PutcharToQueue:timeout'+^G);
    Delay(1000);
  end;
end;

procedure PutLine(Pt: byte; OutStr: string);
var i : integer;
begin
  if Debuging then writeln(DebugFile,'COM',Pt,': '+OutStr);
  for i:= 1 to length(OutStr) do PutChar(Pt,OutStr[i]);
  { Enable the "Transmitter register empty" interrupt
  Port[COMData[Pt].PortBase + IntrEnable] := $0F}
end;

function GetLine(Pt : byte;endchar:char;maxlength:byte):string;
var c : char;
    HilfStr : string;
    timeout:longint;
begin
  HilfStr := '';
  timeout:=0;
  repeat
    if GetChar(pt,c) then begin
      if (c<>endchar) then HilfSTR := HilfSTR + c;
    end else begin
      inc(timeout);
      delay(10);
    end;
  until (c = endchar) or (length(HilfStr) = maxlength) or (timeout>200);
  if (timeout>200) then begin
    writeln('Serial: GetLine: Timeout'+^G);
    delay(1000);
  end;
  if Debuging then writeln(DebugFile,'COM',Pt,': '+HilfStr);
  GetLine := HilfStr;
end;

procedure PrintBuffer(Pt:byte);
var n:word;
begin
  writeln;
  textcolor(green);
  writeln('Out:');
  for n:=0 to QueueLen-1 do begin
    if COMData[Pt].OutQueue.Stop=n then textcolor(lightgreen)
    else textcolor(green);
    if (COMData[Pt].OutQueue.Content[n]=' ') then write('.')
    else if (COMData[Pt].OutQueue.Content[n]>#13)
    then write(char(COMData[Pt].OutQueue.Content[n]))
    else write('');
  end;
  writeln;
  writeln;
  textcolor(red);
  writeln('In:');
  for n:=0 to QueueLen-1 do begin
    if COMData[Pt].InQueue.Stop=n then textcolor(lightred)
    else if COMData[Pt].InQueue.Start=n then textcolor(lightblue)
    else textcolor(red);
    if (COMData[Pt].InQueue.Content[n]=' ') then write('.')
    else if (COMData[Pt].InQueue.Content[n]>#13)
    then write(char(COMData[Pt].InQueue.Content[n]))
    else write('');
  end;
  writeln;
  writeln;
  textcolor(yellow);
end;

procedure MyExitProc;far;
var n:byte;
begin
  PrintBuffer(2);

  for n:= 1 to 4 do ClosePort(n);
end;

var OldExitProc:pointer;
    n:byte;
    m:word;
begin
  for n:=1 to 4 do begin
    ComData[n].IntSet:=false;
    for m:=0 to QueueLen-1 do begin
      COMData[n].InQueue.Content[m]:=' ';
      COMData[n].OutQueue.Content[m]:=' ';
    end;
  end;
  OldExitProc:=ExitProc;
  ExitProc:=@MyExitProc;
end.

