{A+,B-,D-,E-,F-,G-,I+,L-,N-,O-,R-,S-,V-,X+}

unit Serial;

interface
  uses      crt, Dos;

  const     PCOM1      = $3F8;       { port address of COM1 }
            PCOM2      = $2F8;       { port address of COM2 }


  type      SerialPort = (COM1, COM2);
            ParityType = (Podd,Peven,Pnone);



  function  AskCharNum (Pt: SerialPort): integer;
  procedure ClearBuf (Pt: SerialPort);
  procedure ResetCOM (Pt: SerialPort);
  procedure StopFlow (Pt: SerialPort);
  procedure StartFlow (Pt: SerialPort);
  function  GetChar (Pt: SerialPort): char;
  function  GetLine (Pt: SerialPort): string;
  function  GetReal (Pt: SerialPort): real;
  procedure InitCOM (Pt: SerialPort);
  procedure PutLine (Pt: SerialPort; OutStr: string);
  procedure PutReal (Pt: SerialPort; num: real; width,dec: integer);
  procedure PutChar (Pt: SerialPort; c: char);
  procedure SetProtocol (Pt: SerialPort; Baud,Leng,Stop: integer;
                         Par: ParityType; Hrdwire:Boolean);
  procedure SetTimeOut (Pt: SerialPort; time: word);


(******************************************************************)
implementation
(******************************************************************)
uses debug;
type
  ComDescr = array[COM1..COM2] of byte;

const
  BufSize            = 3000;              { size of receive buffer }
  BufLow             = 500;       { switch level for DTR handshake }
  IMA     : byte     = $21;       { address of 8259 interrupt mask }
  IMaskEn : ComDescr = ($EF,$F7);            { mask interrupt line }
  IVec    : ComDescr = ($0C,$0B);        { interrupt vector number }
  IER = 1;                        { UART interrupt enable register }
  LCR = 3;                            { UART line control register }
  MCR = 4;                           { UART modem control register }
  DLL = 0;                         { UART divisor latch - low byte }
  DLH = 1;                        { UART divisor latch - high byte }
  com1used : Boolean=false;
  com2used : Boolean=false;

type
  PortRec = record
              Buffer     : array[1..BufSize] of byte;  { RX buffer }
              Flow       : boolean;    { TRUE if handshake enabled }
              Hardwire   : boolean;    { TRUE if handshake wired   }
              InPointer  : integer;        { pointer to free place }
              OutPointer : integer;       { pointer to oldest char }
              Count      : integer;    { number of chars in buffer }
              VecBak     : pointer;         { old interrupt vector }
              TimeOut    : boolean;       { timeout during GetChar }
              TOCntMax   : word;      { delay until timeout (msec) }
            end;

var
  PortVar   : array[COM1..COM2] of PortRec;        { port variable }
  padr      : word;                     { physical address of port }
  cpt       : Serialport;                     { global port number }
  UARTStat  : array[COM1..COM2,1..6] of byte; { backup UART status }







(******************************************************************)
function CalcAdr (Pt: SerialPort): word;
(******************************************************************
ENTRY: pt ........ Port (COM1, COM2)

EXIT:  physical address of specified port
*******************************************************************)
begin
case pt of
  COM1 : CalcAdr := PCOM1;
  COM2 : CalcAdr := PCOM2;
end;
end;


(******************************************************************)
function  AskCharNum (Pt: SerialPort): integer;
(******************************************************************
ENTRY: pt ........ Port (COM1, COM2)

EXIT:  number of characters in buffer
*******************************************************************)

begin
AskCharNum := PortVar[pt].Count;
end;


(******************************************************************)
procedure SetTimeOut (Pt: SerialPort; time: word);
(******************************************************************
ENTRY: pt ........ Port (COM1, COM2)
       time ...... TimeOut-length (msec)

EXIT:  timeout is set to 'time' msec
*******************************************************************)

begin
PortVar[Pt].TOCntMax := time;
end;


(******************************************************************)
procedure StopFlow (Pt : SerialPort);
(******************************************************************
ENTRY: pt ........ Port (COM1, COM2)

EXIT:  handshake disables any further transmission of data
*******************************************************************)

begin
padr := CalcAdr (pt);
(*
port[padr+4] := Port[padr+4] and $FD;          { disable handshake RTS }
*)
port[padr+4] := Port[padr+4] and $FE;          { disable handshake DTR }
PortVar[Pt].Flow := False;
delay (2);
end;


(******************************************************************)
procedure StartFlow (Pt : SerialPort);
(******************************************************************
ENTRY: pt ........ Port (COM1, COM2)

EXIT:  handshake enables transmission of data
*******************************************************************)

begin
padr := CalcAdr (pt);
(*
port[padr+4] := Port[padr+4] or $02;            { enable handshake RTS }
*)
port[padr+4] := Port[padr+4] or $01;            { enable handshake DTR }
PortVar[Pt].Flow := True;
end;


(******************************************************************)
procedure ISRCOM1;
(******************************************************************
ENTRY: none

EXIT:  interrupt service routine COM1
*******************************************************************)

interrupt;

var
  InByte : integer;

begin
inline ($B0/$20/                  { MOV AL,20H   EOI command 8259  }
        $E6/$20/                  { OUT 20H,A                      }
        $FB);                     { STI                            }
InByte := port[PCOM1];                        { get byte from port }
with PortVar[COM1] do
if Count < BufSize then
  begin
  if Count > BufSize-100 then StopFlow(COM1);   { overflow occured }
  Buffer[InPointer] := InByte;                        { store byte }
  inc(Count);
  inc(InPointer);
  if InPointer > BufSize then InPointer := 1;  { close ring buffer }
  end;
end;


(******************************************************************)
procedure ISRCOM2;
(******************************************************************
ENTRY: none

EXIT:  interrupt service routine COM2
*******************************************************************)

interrupt;

var
  InByte : integer;

begin
inline ($B0/$20/                  { MOV AL,20H   EOI command 8259  }
        $E6/$20/                  { OUT 20H,A                      }
        $FB);                     { STI                            }
InByte := port[PCOM2];                        { get byte from port }
with PortVar[COM2] do
if Count < BufSize then
  begin
  if Count > BufSize-100 then StopFlow(COM2);   { overflow occured }
  Buffer[InPointer] := InByte;                        { store byte }
  inc(Count);
  inc(InPointer);
  if InPointer > BufSize then InPointer := 1;  { close ring buffer }
  end;
end;


(******************************************************************)
procedure InitCOM (Pt : SerialPort);
(******************************************************************
ENTRY: pt ........ Port (COM1, COM2)

EXIT:  specified port is initialized
*******************************************************************)

var
  test : byte;

begin
  case pt of com1:if com1used then exit;
             com2:if com2used then exit;
  end;
padr := CalcAdr (pt);
test := port[padr];                     { clear pending interrupts }
test := port[padr+5];
UARTSTat [pt,1] := port[padr+IER];              { save UART status }
UARTSTat [pt,2] := port[padr+LCR];
UARTSTat [pt,3] := port[padr+MCR];
port[padr+3] := port[padr+3] or $80;
UARTSTat [pt,4] := port[padr+DLL];
UARTSTat [pt,5] := port[padr+DLH];
port[padr+3] := port[padr+3] and $7F;
UARTSTat [pt,6] := port[IMA] and not(IMaskEn[pt]); { get int. mask }
with PortVar[pt] do
  begin
  InPointer := 1;                         { init buffer management }
  OutPointer := 1;
  Count := 0;
  GetIntVec (IVec[Pt],VecBak);
  case Pt of
    COM1: begin
            SetIntVec (IVec[pt],addr(ISRCOM1));   { interrupt vector }
            com1used:=true;
          end;
    COM2: begin
            SetIntVec (IVec[pt],addr(ISRCOM2));   { interrupt vector }
            com2used:=true;
          end;
  end;
  port[IMA] := IMaskEn[Pt] and port[IMA]; { interrupt mask on 8259 }
  port[padr+4] := $0F;                     { & set handshake lines }
  Flow := True;
  port[padr+3] := port[padr+3] and $7F;               { reset DLAB }
  port[padr+1] := $01;         { enable receiver interrupt on 8250 }
  end;
end;


(******************************************************************)
procedure ClearBuf (Pt : SerialPort);
(******************************************************************
ENTRY: pt ........ Port (COM1, COM2)

EXIT:  receive buffer of specified port is cleared
*******************************************************************)

begin
port[IMA] := not (IMaskEn[pt]) or port[IMA];     { reset interrupt }
with PortVar[Pt] do
  begin
  Count := 0;
  InPointer := 1;
  OutPointer := InPointer;
  end;
port[IMA] := IMaskEn[pt] and port[IMA]; { enable interrupt on 8259 }
end;


(******************************************************************)
procedure ResetCOM (Pt : SerialPort);
(******************************************************************
ENTRY: pt ........ Port (COM1, COM2)

EXIT:  specified port is reset
*******************************************************************)


begin
padr := CalcAdr (pt);
port[padr+1] := $00;          { disable receiver interrupt on 8250 }
port[IMA] := not(IMaskEn[pt]) or port[IMA]; { reset interrupt mask }
SetIntVec (IVec[pt],PortVar[pt].VecBak);  { reset interrupt vector }
port[padr+LCR] := UARTSTat [pt,2];           { restore UART status }
port[padr+MCR] := UARTSTat [pt,3];
port[padr+3] := port[padr+3] or $80;
port[padr+DLL] := UARTSTat [pt,4];
port[padr+DLH] := UARTSTat [pt,5];
port[padr+3] := port[padr+3] and $7F;
port[padr+IER] := UARTSTat [pt,1];
port[IMA] := (port[IMA] or UARTStat[pt,6]) and
             (UARTStat[pt,6] or IMaskEn[pt]);
case pt of
  com1:com1used:=false;
  com2:com2used:=false;
end;
end;


(******************************************************************)
function GetChar (Pt : SerialPort): char;
(******************************************************************
ENTRY: pt ........ Port (COM1, COM2)

EXIT:  returns character read from specified port buffer
*******************************************************************)


var
  TimeOutCnt  : integer;

begin
with PortVar[pt] do
  begin
  TimeOut := false;
  TimeOutCnt := TOCntMax;
  if COunt < BufLow then StartFlow(pt);
  repeat
    Delay (1);
    dec (TimeOutCnt);
  until (Count > 0) or (TimeOutCnt = 0);
  if TimeOutCnt = 0
    then begin
         GetChar := chr(0);
         timeOut := True;
         end
    else begin
         port[IMA] := not (IMaskEn[pt]) or port[IMA];
         GetChar := chr(Buffer[OutPOinter]);
         OutPointer := OutPointer + 1;
         Count := COunt - 1;
         if OutPointer > BufSize then OutPointer := 1;
         port[IMA] := IMaskEn[pt] and port[IMA];
         end;
  end;
end;


(******************************************************************)
function GetLine (Pt : SerialPort): string;
(******************************************************************
ENTRY: pt ........ Port (COM1, COM2)

EXIT:  returns line read from specified port buffer
*******************************************************************)


var
  c : char;
  HilfStr : string;

begin
HilfStr := '';
repeat
  c := GetChar(pt);
  if c >= ' ' then HilfSTR := HilfSTR + c;
until (c = chr(13)) or (length(HilfStr) = 255);
GetLine := HilfStr;
end;


(******************************************************************)
function GetReal (Pt : SerialPort): real;
(******************************************************************
ENTRY: pt ........ Port (COM1, COM2)

EXIT:  returns real number read from specified port buffer
*******************************************************************)


var
  c        : char;
  HilfStr  : string;
  Status   : integer;
  HilfReal : real;

begin
HilfStr := '';
with PortVar[pt] do
  begin
  repeat
    c := UpCase(GetChar(pt));
  until (not TimeOut) and (c in ['0'..'9','+','-','.','E']);
  HilfStr := HilfStr + c;
  repeat
    c := UpCase(GetChar(pt));
  until Not TimeOut;
  while (c in ['0'..'9','+','-','.','E']) do
    begin
    HilfSTR := HilfSTR + c;
    repeat
      c := UpCase(GetChar(pt));
    until Not TimeOut;
    end;
  val (HilfStr,HilfReal,status);
  GetReal := HilfReal;
  end;
end;


(******************************************************************)
procedure SetProtocol (Pt : SerialPort;
                  Baud, Leng, Stop: integer; Par:ParityType; Hrdwire:boolean);
(******************************************************************
ENTRY: pt ........ Port (COM1, COM2)
       Baud ...... Baudrate (110,150,300,600,1200,2400,4800,9600)
       Leng ...... number of data bits (7,8)
       Stop ...... number of stop bits (1,2)
       Par ....... parity bit (Pnone, Podd, Peven)

EXIT:  protocol of specified port is set accordingly
*******************************************************************)

var
  BaudBit, LengBit, ParBit, StopBit : byte;
  regs     : registers;
  test     : byte;

begin
padr := CalcAdr (pt);
test := port[padr+1];                    (* save interrupt status *)
case Baud of
  110 : BaudBit := 0;
  150 : BaudBit := 1;
  300 : BaudBit := 2;
  600 : BaudBit := 3;
 1200 : BaudBit := 4;
 2400 : BaudBit := 5;
 4800 : BaudBit := 6;
 9600 : BaudBit := 7
else BaudBit := 7
end;
case Leng of
    7 : LengBit := 0;
    8 : LengBit := 1
else LengBit := 1
end;
case Par of
  Pnone : ParBit := 0;
  Podd  : Parbit := 1;
  Peven : ParBit := 3;
else ParBit := 0
end;
case Stop of
    1 : StopBit := 0;
    2 : StopBit := 1
else StopBit := 0
end;
with Regs do
  begin
  DX := ord(pt);
  AX := 32*BaudBit + 8*ParBit + 4*StopBit + 2 + LengBit;
  Intr($14,Regs);
  end;
port[padr+1] := test;                 (* restore interrupt status *)
PortVar[Pt].Hardwire:=Hrdwire;
end;


(******************************************************************)
procedure PutChar (Pt : SerialPort; c : char);
(******************************************************************
ENTRY: pt ........ Port (COM1, COM2)
       c ......... character to be sent

EXIT:  character 'c' is sent on port 'pt'
*******************************************************************)

var n:word;
begin
padr := CalcAdr (pt);
delay(1);
n:=0;
if PortVar[Pt].Hardwire then repeat inc(n);{ wait until CTS }
  until (((port[padr+5] and 32) <> 0)  and ((port[padr+6] and 16) <> 0)) or (n>60000)
else repeat inc(n); until ((port[padr+5] and 32) <> 0) or (n>60000);
port [padr] := ord(c);
if (n>10000) then writeln('Serial : PutChar : no CTS');
end;


(******************************************************************)
procedure PutLine (Pt: SerialPort; OutStr: string);
(******************************************************************
ENTRY: pt ........ Port (COM1, COM2)
       Outstr .... line to be sent

EXIT:  line 'OutStr' is sent on port 'pt'
*******************************************************************)


var
  i : integer;

begin
  if Debuging then begin
    case Pt of
      COM1: writeln(DebugFile,'COM1: '+OutStr);
      COM2: writeln(DebugFile,'COM2: '+OutStr);
    end;
  end;
for i:= 1 to length(OutStr) do
  PutChar (Pt,OutStr[i]);
end;


(******************************************************************)
procedure PutReal (Pt: SerialPort; num: real; width,dec: integer);
(******************************************************************
ENTRY: pt ........ Port (COM1, COM2)
       num ....... real number to be sent
       width ..... format parameter - width
       dec ....... format parameter - decimal places

EXIT:  real number is sent on port 'pt'
*******************************************************************)



var
  hilfstr : string;

begin
str (num:width:dec,Hilfstr);
PutLine (Pt,HilfStr);
end;


(******************************************************************)
(*                          I N I T                               *)
(******************************************************************)


var ExitSave : Pointer;
{$F+}Procedure MyExit; {$F-}
Begin
  ExitProc:=ExitSave;
  if com1used then ResetCom(Com1);
  if com2used then ResetCom(Com2);
End;

begin
  ExitSave:=ExitProc;
  ExitProc:=@MyExit;

  for cpt := COM1 to COM2 do with PortVar[cpt] do begin
    GetIntVec (IVec[cpt],VecBak);
    TOCntMax := 5000;
    InPointer := 1;
    OutPointer := 1;
    Count := 0;
  end;
end.

