unit SigGen;
interface
uses params;

const on=true;
      off=false;
      MaxRFLevel=19;
      MinRFLevel=-136;
      MaxSynth=2;
      LastFrequency:array[0..MaxSynth] of double=(0,0,0);
      Marconi=0;
      old=1;
      HP=2;
      Rohde=3;
      SRS=4;
      GPIBAdress:array[0..MaxSynth] of byte=(2,0,18);
      SynthType:array[0..MaxSynth] of byte=(Rohde,Marconi,SRS);
      SynthGPIBCommandTime:array[0..MaxSynth] of double=(MarconiGPIBCommandTime,MarconiGPIBCommandTime,MarconiGPIBCommandTime);

procedure SwitchRFGPIB(Synth:byte;Level:double;onoff:boolean);
procedure SetFrequency(Synth:byte;f:double);
procedure PrepareSweep(Synth:byte;StartFrequency:double;StopFrequency:double;StepTime:double;Steps:double);
procedure InitSweep(Synth:byte);
procedure InitMarconi(Synth:byte);
function RFOn(Synth:byte):boolean;
procedure SwitchRFOff(Synth:byte);
procedure PreparePuls(Synth:byte;width:double);
procedure PrepareVCORFSweep(Synth:byte;width:double);
procedure ModulationOff(Synth:byte);

implementation
uses crt,IEEE,sequence,ExpIO;

procedure eatWhite(var buf:string);
begin
  while (buf[length(buf)]=' ') and (length(buf)>1) do buf:=copy(buf,1,length(buf)-1);
  while (buf[1]=' ') and (length(buf)>1) do buf:=copy(buf,2,length(buf)-1);
end;

const RFison:array[0..MaxSynth] of boolean=(false,false,false);

function RFOn(Synth:byte):boolean;
begin
  RFOn:=RFison[Synth];
end;

procedure SwitchRFOff(Synth:byte);
var status:integer;
begin
  if SequenceOut then begin
    case SynthType[Synth] of
      Marconi:Send(GPIBAdress[Synth],'RFLV:VALUE -144.0DBM;OFF;',status);
      old:Send(GPIBAdress[Synth],'LEVEL -140.0DBM;LEVEL:OFF;',status);
      HP:begin
        Send(GPIBAdress[Synth],':POW -136.0DBM',status);
        Send(GPIBAdress[Synth],':OUTP OFF',status);
      end;
      Rohde:Send(GPIBAdress[Synth],':POW -144;OUTP:STAT OFF',status);
      SRS:Send(GPIBAdress[Synth],'AMPL 0VP',status);
    end;
    SetRFSweepCopyFrequency(Synth,0);
  end else SequenceByteArg(SwitchRFOff,Synth,0);
  RFison[Synth]:=off;
end;

procedure SwitchRFGPIB(Synth:byte;Level:double;onoff:boolean);
var status:integer;
    buf:string;
begin
  RFison[Synth]:=onoff;
  if SequenceOut then begin
    str(level:5:1,buf);
    EatWhite(buf);
    if onoff then begin
      case SynthType[Synth] of
        Marconi:Send(GPIBAdress[Synth],'RFLV:VALUE '+buf+'DBM;ON;',status);
        old:Send(GPIBAdress[Synth],'LEVEL '+buf+'DBM; LEVEL:ON',status);
        HP:begin
          Send(GPIBAdress[Synth],':POW '+buf+'DBM',status);
          Send(GPIBAdress[Synth],':OUTP ON',status);
        end;
        Rohde:Send(GPIBAdress[Synth],':POW '+buf+';OUTP:STAT ON',status);
        SRS:Send(GPIBAdress[Synth],'AMPL '+buf+'DB',status);
      end;
      SetRFSweepCopyFrequency(Synth,LastFrequency[Synth]);
    end else begin
      case SynthType[Synth] of
        Marconi:Send(GPIBAdress[Synth],'RFLV:VALUE '+buf+'DBM;OFF;',status);
        old:Send(GPIBAdress[Synth],'LEVEL '+buf+'DBM; LEVEL:OFF',status);
        HP:begin
          Send(GPIBAdress[Synth],':POW '+buf+'DBM',status);
          Send(GPIBAdress[Synth],':OUTP OFF',status);
        end;
        Rohde:Send(GPIBAdress[Synth],':POW '+buf+';OUTP:STAT OFF',status);
        SRS:Send(GPIBAdress[Synth],'AMPL 0VP',status);
      end;
      SetRFSweepCopyFrequency(Synth,0);
    end;
  end else SequenceBytedoublebooleanArg(SwitchRFGPIB,Synth,Level,onoff,0);
end;

procedure SetFrequency(Synth:byte;f:double);
var buf:string;
    buf2:string;
    status:integer;
begin
  if SequenceOut then begin
    str(f:4:4,buf);
    str(f*1000000:9:0,buf2);
    EatWhite(buf);
    EatWhite(buf2);
    case SynthType[Synth] of
      Marconi:Send(GPIBAdress[Synth],'CFRQ '+buf+'MHZ',status);
      old:Send(GPIBAdress[Synth],'RF '+buf+'MHZ',status);
      HP:Send(GPIBAdress[Synth],':FREQ '+buf+'MHZ',status);
      Rohde:Send(GPIBAdress[Synth],'FREQ '+buf+'MHZ',status);
      SRS:Send(GPIBAdress[Synth],'FREQ '+buf2,status);
    end;
    SetRFSweepCopyFrequency(Synth,f);
    LastFrequency[Synth]:=f;
  end else SequenceByteDoubleArg(Setfrequency,Synth,f,0);
end;

procedure PrepareSweep(Synth:byte;StartFrequency:double;StopFrequency:double;StepTime:double;Steps:double);
var bufstart:string;
    bufstop:string;
    buftime:string;
    bufnrstep:string;
    status,n:integer;
begin
  if SequenceOut then begin
    str(StartFrequency:4:4,bufstart);
    str(StopFrequency:4:4,bufstop);
    str(StepTime:4:4,buftime);
    str(Steps:4:0,bufnrstep);
    EatWhite(bufstart);
    EatWhite(bufstop);
    EatWhite(buftime);
    EatWhite(bufnrstep);
    case SynthType[Synth] of
      Marconi:begin
        Send(GPIBAdress[Synth],'IMODE SWEEPER',status);
        Send(GPIBAdress[Synth],'SWEEP:TYPE CFRQ',status);
        Send(GPIBAdress[Synth],'SWEEP:CFRQ:START '+bufstart+
          'MHZ',status);
        Send(GPIBAdress[Synth],'SWEEP:CFRQ:STOP '+bufstop+'MHZ',status);
        Send(GPIBAdress[Synth],'SWEEP:CFRQ:STEP '+bufnrstep,status);
        Send(GPIBAdress[Synth],'SWEEP:CFRQ:TIME '+buftime+'MS',status);
        Send(GPIBAdress[Synth],'SWEEP:MODE SNGL',status);
        Send(GPIBAdress[Synth],'SWEEP:GO',status);
      end;
      HP:begin
        Send(GPIBAdress[Synth],':FREQ:MODE LIST',status);
        Send(GPIBAdress[Synth],':LIST:TYPE STEP',status);
        Send(GPIBAdress[Synth],':FREQ:START '+bufstart+' MHZ',status);
        Send(GPIBAdress[Synth],':FREQ:STOP '+bufstop+' MHZ',status);
        Send(GPIBAdress[Synth],':SWE:POIN '+bufnrstep,status);
        Send(GPIBAdress[Synth],':SWE:DWEL '+buftime+' MS',status);
      end;
    end;
    {LastFrequency[Synth]:=StopFrequency;}
  end else SequenceByteDoubleDoubleDoubleDoubleArg(PrepareSweep,Synth,StartFrequency,StopFrequency,StepTime,Steps,0);
end;

procedure InitSweep(Synth:byte);
var status:integer;
begin
  if SequenceOut then begin
    case SynthType[Synth] of
      Marconi:begin
        Send(GPIBAdress[Synth],'IMODE SWEEPER',status);
        Send(GPIBAdress[Synth],'SWEEP:GO',status);
      end;
      HP:begin
        Send(GPIBAdress[Synth],':FREQ:MODE LIST',status);
        Send(GPIBAdress[Synth],':INIT',status);
      end;
    end;
  end else SequenceByteArg(InitSweep,Synth,0);
end;

procedure InitMarconi(Synth:byte);
var status:integer;
begin
  if SequenceOut then begin
    case SynthType[Synth] of
      Marconi: begin
        Send(GPIBAdress[Synth],'*RST',status);
        if (status<>0) then begin
          writeln('Marconi: Error talking to Synthesizer Nr ',Synth);
          delay(500);
        end else begin
          Send(GPIBAdress[Synth],'MOD:OFF',status);
          {Send(GPIBAdress[Synth],'SWEEP:SNGL',status);}
          if (status<>0) then begin
            writeln('SigGen:Marconi: Error in command SWEEP:SNGL Synth Nr ',Synth);
            delay(500);
          end;
          SwitchRFGPIB(Synth,MinRFLevel,off);
        end;
      end;
      old:;
      HP: begin
        Send(GPIBAdress[Synth],'*RST',status);
        if (status<>0) then begin
          writeln('SigGen:HP: Error talking to Synthesizer Nr ',Synth);
          delay(500);
        end else begin
          Send(GPIBAdress[Synth],':OUTP:MOD OFF',status);
          SwitchRFGPIB(Synth,MinRFLevel,off);
        end;
      end;
      Rohde: begin
        Send(GPIBAdress[Synth],'*RST',status);
        if (status<>0) then begin
          writeln('SigGen:Rohde&Schwarz: Error talking to Synthesizer Nr ',Synth);
          delay(500);
        end else begin
          {Send(GPIBAdress[Synth],'MOD:OFF',status);}
          {Send(GPIBAdress[Synth],'SWEEP:SNGL',status);}
          Send(GPIBAdress[Synth],':PULM:SOUR INT; STAT OFF',status);
          if (status<>0) then begin
            writeln('Marconi: Error in command SWEEP:SNGL Synth Nr ',Synth);
            delay(500);
          end;
          SwitchRFGPIB(Synth,MinRFLevel,off);
        end;
      end;
      SRS:begin
        Send(GPIBAdress[Synth],'*RST',status);
        if (status<>0) then begin
          writeln('SigGen:SRS: Error talking to Synthesizer Nr ',Synth);
          delay(500);
        end else begin
          Send(GPIBAdress[Synth],'AMPL 0VP',status);
          Send(GPIBAdress[Synth],'FUNC 0',status);
          Send(GPIBAdress[Synth],'FREQ 0',status);
          Send(GPIBAdress[Synth],'OFFS 0',status);
          Send(GPIBAdress[Synth],'MDWF 6',status);
          Send(GPIBAdress[Synth],'MENA 0',status);
        end;
      end;
    end;
  end else SequenceByteArg(InitMarconi,Synth,0);
end;

procedure PreparePuls(Synth:byte;width:double);
var buf:string;
    status:integer;
begin
  if SequenceOut then begin
    case SynthType[Synth] of
      Rohde: begin
        Send(GPIBAdress[Synth],':PULM:SOUR INT; STAT ON',status);
        Send(GPIBAdress[Synth],':PULM:POL NORM',status);
        Send(GPIBAdress[Synth],':PULM:EXT:IMP 10E3',status);
        Send(GPIBAdress[Synth],':PULS:DOUB OFF',status);
        Send(GPIBAdress[Synth],':PULS:DEL 1us',status);
        Send(GPIBAdress[Synth],':PULS:PER 10000us',status);
        str(width*1000:5:1,buf);
        EatWhite(buf);
        Send(GPIBAdress[Synth],':PULS:WIDT '+buf+'us',status);
        Send(GPIBAdress[Synth],':TRIG:PULS:SOUR EXT',status);
        Send(GPIBAdress[Synth],':TRIG:PULS:SLOPE POS',status);
      end;
    end;
  end else SequenceByteDoubleArg(PreparePuls,Synth,width,0);
end;

procedure PrepareVCORFSweep(Synth:byte;width:double);
var buf:string;
    status:integer;
begin
  if SequenceOut then begin
    str(width*1000:4:3,buf);
    EatWhite(buf);
    case SynthType[Synth] of
      Rohde: begin
        Send(GPIBAdress[Synth],':SOUR:FM1:DEV '+buf+'kHz',status);
        Send(GPIBAdress[Synth],':SOUR:FM1:SOUR EXT1',status);
        Send(GPIBAdress[Synth],':SOUR:FM1:EXT1:COUP DC',status);
        Send(GPIBAdress[Synth],':SOUR:FM:STAT ON',status);
      end;
      Marconi: begin
        Send(GPIBAdress[Synth],'MODE:FM1',status);
        Send(GPIBAdress[Synth],'MOD:ON',status);
        Send(GPIBAdress[Synth],'FM1:DEVN '+buf+'kHz;EXT1DC;ON',status);
      end;
    end;
  end else SequenceByteDoubleArg(PrepareVCORFSweep,Synth,width,0);
end;


procedure ModulationOff(Synth:byte);
var status:integer;
begin
  if SequenceOut then begin
    case SynthType[Synth] of
      Rohde: begin
        Send(GPIBAdress[Synth],':PULM:SOUR INT; STAT OFF',status);
      end;
    end;
  end else SequenceByteArg(ModulationOff,Synth,0);
end;

begin
  InitMarconi(0);
  InitMarconi(1);
  InitMarconi(2);
end.