unit Timing;

{
  This Unit implements a good delay routine using the DAS 1601 Counter 0.
  The computer is synchronized to the counter to increase the reliability.
  startCounter has to be called shortly (less than the first wait-time and
  less than half a second) before the first Wait command.
}

interface
{$R-}
var DAS1601Present:boolean;
procedure startCounter;
procedure TimingWait(time:real); {time in ms}
procedure SimpleWait(time:real); {time in ms}
procedure CheckCounter;
function GetTimeSinceLastWait:double;
function GetTimingError:double;
const ID=0.000001;
      IDfrac=100;

implementation
uses crt,dos,debug;

CONST PCCalibration=1193.180;
      TimingError:double=0;

CONST
  {internal timer registers}
  PCtimer0=$40;
  PCtimer1=$41;
  PCtimer2=$42;
  PCtimerctl=$43;
  PCtimergate=$61;
  {DAS-1601 registers:}
  adr=$310;
  ad0=adr+$0;
  ad1=adr+$1;
  mux=adr+$2;
  dio=adr+$3;
  da0lb=adr+$4;
  da0hb=adr+$5;
  da1lb=adr+$6;
  da1hb=adr+$7;
  statusa=adr+$8;
  ctrl=adr+$9;
  cnten=adr+$A;
  gain=adr+$B;
  DAS1601timer0=adr+$C;
  DAS1601timer1=adr+$D;
  DAS1601timer2=adr+$E;
  DAS1601timerctl=adr+$F;
  piopa=adr+$400;
  piopb=adr+$401;
  piopc=adr+$402;
  pioctl=adr+$403;
  disarm=adr+$404;
  bmden=adr+$405;
  mden1600=adr+$406;
  statusb=adr+$407;

CONST
  CNT0=0;
  CNT1=64;
  CNT2=128;
  Readback=128+64;

  CNTLatch=0;
  LowByte=16;
  HighByte=32;
  BothBytes=16+32;

  PulseOnTerminalCount=0;
  ProgrammableOneShot=2;
  RateGenerator=4;
  SquareWaveGenerator=2+4;
  SoftwareTriggeredStrobe=8;
  HardwareTriggeredStrobe=8+2;

  Binary=0;
  BCD=1;

  NoCNTLatch=32;
  DontReadStatus=16;
  ReadStatus=0;
  C0=2;
  C1=4;
  C2=8;

  ExternalClock=0;
  InternalClock=2;

  OldCounter:word=$ffff;
  CounterChecked:boolean=false;
  CounterSum:double=0;

var C:byte;
    CounterNR:byte;
    timerctl:word;
    timer:word;
    counts_ms:double;
    GoalDistance:word;
var OldTimerInt,oldexitproc:pointer;
    OldNow,Zaehler:longint;
    ZaehlerNow:word;
    Littlewait:double;
    zaehlerueberlaufabgearbeitet:boolean;
    WaitNr:longint;

procedure StartCounter;
begin
  TimingError:=0;
  WaitNr:=0;
  CounterSum:=0;
  CounterChecked:=false;
  if DAS1601Present then begin
    port[cnten]:=InternalClock;       {enable counter 0}
    Port[timerctl]:=CounterNR or BothBytes or PulseOnTerminalCount or Binary;
    Port[timer]:=$0;
    Port[timer]:=$0;    {counter 0 counting down form 65535 with 100kHz}
    delay(1);
    Port[timerctl]:=ReadBack or DontReadStatus or CNTLatch or C; {read counter}
    OldCounter:=Port[timer]*256+Port[timer];
  end else begin
    Port[PCtimerctl]:=ReadBack or DontReadStatus or CNTLatch or C0;
    OldCounter:=Port[PCtimer0]*256+Port[PCtimer0];
  end;
end;

procedure CheckCounter;
var now:word;
begin
  CounterChecked:=true;
  Port[timerctl]:=ReadBack or DontReadStatus or CNTLatch or C; {read counter}
  now:=Port[timer]*256+Port[timer];
  if now<=Oldcounter then begin
    CounterSum:=CounterSum+(OldCounter-now)/Counts_ms;
    OldCounter:=now;
  end else begin
    CounterSum:=CounterSum+OldCounter/Counts_ms;
    CounterSum:=CounterSum+(65535-now)/Counts_ms;
    OldCounter:=now;
  end;
end;

function GetTimeSinceLastWait:double;
begin
  CheckCounter;
  GetTimeSinceLastWait:=CounterSum;
end;

procedure TimingWait(time:real); {time in ms}
var goal,now,futurenow,help,n,highcount:word;
    aktZaehler,CounterNow,PCNow,Goalnow,cnt:longint;
    first:boolean;
begin
  if time<0.01 then exit;
  inc(WaitNr);
  if Debuging or DebugingAsOszilloskop then begin
    if DebugingAsOszilloskop then DebugWait(time)
    else writeln(DebugFile,'Wait(',time:2:2,' ms)   Wait Nr=',WaitNr,' WaitID:',frac(Time*IDfrac)/(IDfrac*ID):3:2);
    if time<2 then exit;
  end;
  if (time>1000000) then begin
    writeln('Timing : Wait : You surely don''t want to wait that long',
      ' Wait Nr:',WaitNr,' WaitID:',frac(Time*IDfrac)/(IDfrac*ID):3:2);
    TimingError:=-1;
    time:=10000;
  end;
{  if DAS1601Present then begin}
  if CounterChecked then begin
    CounterChecked:=false;
    if CounterSum>time then begin
      Writeln('Timing : CounterSum ',Countersum:2:2,' bigger than delay ',
        time:2:2,' Wait Nr:',WaitNr,' WaitID:',frac(Time*IDfrac)/(IDfrac*ID):3:2);
      TimingError:=-2;
      CounterSum:=0;
      exit;
    end else time:=time-CounterSum;
    {Writeln(CounterSum:2:2);}
    CounterSum:=0;
  end;
    if (time<60000/counts_ms) then cnt:=round(time*counts_ms) else begin
      highcount:=trunc(time/LittleWait);
      cnt:=round((time-highcount*LittleWait)*counts_ms);
      if (cnt<2*GoalDistance) then repeat
        cnt:=cnt+round(counts_ms*LittleWait);
        if highcount>0 then highcount:=highcount-1
        else begin
          writeln('Timing : Wait : GoalDistance error',' Wait Nr:',WaitNr,' WaitID:',frac(Time*IDfrac)/(IDfrac*ID):3:2);
          TimingError:=-3;
        end;
      until (cnt>=2*GoalDistance);
      for n:=1 to highcount do TimingWait(LittleWait);
    end;
{    if (cnt<GoalDistance/3) then writeln('Timing : Wait : delay too short');}
    {$R-} goal:=Oldcounter-cnt;
    Port[timerctl]:=ReadBack or DontReadStatus or CNTLatch or C; {read counter}
    now:=Port[timer]*256+Port[timer];
    futurenow:=now-1;
    if ((goal<Oldcounter) and ((futurenow>OldCounter) or (futurenow<goal))) or
       ((goal>Oldcounter) and ((futurenow>OldCounter) and (futurenow<goal)))
    then begin
      writeln('Timing : Wait : delay ',time:3:2,' ms too short',' Wait Nr:',
        WaitNr,' WaitID:',frac(Time*IDfrac)/(IDfrac*ID):3:2);
      if TimingError=0 then TimingError:=WaitNr;
      Oldcounter:=now;
      exit;
    end;
    if (Oldcounter<cnt) then begin
      {the counter has to jump from 0 to $FFFF}
      if (65535-goal>GoalDistance) then begin
        {the goal is not to near to the boarder, so we can wait until he jumped}
        repeat
          Port[timerctl]:=ReadBack or DontReadStatus or CNTLatch or C; {read counter}
          now:=Port[timer]*256+Port[timer];
        until (now>Oldcounter);
      end else begin
        {the goal is very near the boarder, so we wait until the counter is near the boarder
         or jumped over it}
        repeat
          Port[timerctl]:=ReadBack or DontReadStatus or CNTLatch or C;
          now:=Port[timer]*256+Port[timer];
        until (now<GoalDistance) or (now>65535-GoalDistance);
        {we do not want to be disturbed, so switch interrupts off}
        asm cli end;
        {wait until goal is reached}
        repeat
          Port[timerctl]:=ReadBack or DontReadStatus or CNTLatch or C;
          now:=Port[timer]*256+Port[timer];
        until (now>65000) and (now<=goal);
        asm sti end;  {switch interrupts on again}
        Oldcounter:=goal;  {store last goal}
        exit;         {we have reached the goal}
      end;
    end;
    Oldcounter:=goal; {store last goal}
    {wait until the counter is near the goal}
    repeat
      Port[timerctl]:=ReadBack or DontReadStatus or CNTLatch or C;
      now:=Port[timer]*256+Port[timer];
    until (now-goal<GoalDistance);
    asm cli end; {switch interrupts off}
    {wait until the counter reaches the goal}
    repeat
      Port[timerctl]:=ReadBack or DontReadStatus or CNTLatch or C;
      now:=Port[timer]*256+Port[timer];
    until (now<=goal);
    asm sti end; {switch interrupts on}
end;

procedure SimpleWait(time:real); {time in ms}
var cnt:word;
    status:byte;
begin
  StartCounter;
  TimingWait(time);
end;

function checkDAS1601:boolean;
begin
  checkDAS1601:=DAS1601present;
end;

procedure InitTiming;
var n,now,oldnow,cnt:word;
begin
  port[cnten]:=InternalClock;       {enable counter 0}
  Port[DAS1601timerctl]:=CNT0 or BothBytes or PulseOnTerminalCount or Binary;
  cnt:=round(5*100); {program counter 5ms}
  Port[DAS1601timer0]:=Lo(cnt);
  Port[DAS1601timer0]:=Hi(cnt);
  Port[DAS1601timerctl]:=ReadBack or DontReadStatus or CNTLatch or C0;
  oldnow:=Port[DAS1601timer0]*256+Port[DAS1601timer0];
  n:=0;
  repeat
    delay(1);
    Port[DAS1601timerctl]:=ReadBack or DontReadStatus or CNTLatch or C0;
    now:=Port[DAS1601timer0]*256+Port[DAS1601timer0];
    n:=n+1;
  until (now<oldnow) or (n>10);
  DAS1601present:=(n<10) or (now<oldnow);

  {DAS1601present:=false;}

  if DAS1601present then begin
    GoalDistance:=10;
    timerctl:=DAS1601timerctl;
    timer:=DAS1601timer0;
    CounterNR:=CNT0;
    C:=C0;
    counts_ms:=100;
    Littlewait:=400;
  end else begin
    GoalDistance:=200;
    timerctl:=PCtimerctl;
    timer:=PCtimer0;
    CounterNR:=CNT0;
    C:=C0;
    counts_ms:=PCCalibration*2;
    Zaehler:=0;
    Littlewait:=10;
  end;
end;

function GetTimingError:double;
begin
  GetTimingError:=TimingError;
end;

begin
  InitTiming;
end.