unit Timing;
{
  This Unit implements a good delay routine using the PC Timer Counter 0.
  The computer is synchronized to the counter to increase the reliability.
  startCounter has to be called shortly (less than the first wait-time and
  less than half a second) before the first Wait command.
}

interface
{$R-}
procedure startCounter;
procedure TimingWait(time:real); {time in ms}
procedure SimpleWait(time:real); {time in ms}

implementation
uses crt;

CONST PCCalibration=1193.180;

CONST
  {internal timer registers}
  PCtimer0=$40;
  PCtimer1=$41;
  PCtimer2=$42;
  PCtimerctl=$43;
  PCtimergate=$61;
CONST
  CNT0=0;
  CNT1=64;
  CNT2=128;
  Readback=128+64;

  CNTLatch=0;
  LowByte=16;
  HighByte=32;
  BothBytes=16+32;

  PulseOnTerminalCount=0;
  ProgrammableOneShot=2;
  RateGenerator=4;
  SquareWaveGenerator=2+4;
  SoftwareTriggeredStrobe=8;
  HardwareTriggeredStrobe=8+2;

  Binary=0;
  BCD=1;

  NoCNTLatch=32;
  DontReadStatus=16;
  ReadStatus=0;
  C0=2;
  C1=4;
  C2=8;

  ExternalClock=0;
  InternalClock=2;

  OldCounter:word=$ffff;

    C:byte=C0;
    CounterNR:byte=CNT0;
    timerctl:word=PCtimerctl;
    timer:word=PCtimer0;
    counts_ms:double=PCCalibration*2;
    GoalDistance:word=200;

procedure StartCounter;
begin
  Port[PCtimerctl]:=ReadBack or DontReadStatus or CNTLatch or C0;
  OldCounter:=Port[PCtimer0]*256+Port[PCtimer0];
end;

procedure TimingWait(time:real); {time in ms}
var goal,now,n,highcount:word;
    cnt:longint;
begin
  if (time>100000) then begin
    writeln('Timing : Wait : You surely don''t want to wait that long');
    time:=10000;
  end;
  if (time<60000/counts_ms) then cnt:=round(time*counts_ms) else begin
    highcount:=trunc(time/10);
    cnt:=round((time-highcount*10)*counts_ms);
    if (cnt<2*GoalDistance) then repeat
      cnt:=cnt+round(counts_ms*10);
      if highcount>0 then highcount:=highcount-1
      else writeln('Timing : Wait : GoalDistance error');
    until (cnt>=2*GoalDistance);
    for n:=1 to highcount do TimingWait(10);
  end;
{    if (cnt<GoalDistance/3) then writeln('Timing : Wait : delay too short');}
  {$R-} goal:=Oldcounter-cnt;
  Port[timerctl]:=ReadBack or DontReadStatus or CNTLatch or C; {read counter}
  now:=Port[timer]*256+Port[timer];
  if ((goal<Oldcounter) and ((now>OldCounter) or (now<goal))) or
     ((goal>Oldcounter) and ((now>OldCounter) and (now<goal)))
  then begin
    writeln('Timing : Wait : delay ',time:3:2,' ms too short');
    Oldcounter:=goal;
    exit;
  end;
  if (Oldcounter<cnt) then begin
    {the counter has to jump from 0 to $FFFF}
    if (65535-goal>GoalDistance) then begin
      {the goal is not to near to the boarder, so we can wait until he jumped}
      repeat
        Port[timerctl]:=ReadBack or DontReadStatus or CNTLatch or C; {read counter}
        now:=Port[timer]*256+Port[timer];
      until (now>Oldcounter);
    end else begin
      {the goal is very near the boarder, so we wait until the counter is near the boarder
       or jumped over it}
      repeat
        Port[timerctl]:=ReadBack or DontReadStatus or CNTLatch or C;
        now:=Port[timer]*256+Port[timer];
      until (now<GoalDistance) or (now>65535-GoalDistance);
      {we do not want to be disturbed, so switch interrupts off}
      asm cli end;
      {wait until goal is reached}
      repeat
        Port[timerctl]:=ReadBack or DontReadStatus or CNTLatch or C;
        now:=Port[timer]*256+Port[timer];
      until (now>65000) and (now<=goal);
      asm sti end;  {switch interrupts on again}
      Oldcounter:=goal;  {store last goal}
      exit;         {we have reached the goal}
    end;
  end;
  Oldcounter:=goal; {store last goal}
  {wait until the counter is near the goal}
  repeat
    Port[timerctl]:=ReadBack or DontReadStatus or CNTLatch or C;
    now:=Port[timer]*256+Port[timer];
  until (now-goal<GoalDistance);
  asm cli end; {switch interrupts off}
  {wait until the counter reaches the goal}
  repeat
    Port[timerctl]:=ReadBack or DontReadStatus or CNTLatch or C;
    now:=Port[timer]*256+Port[timer];
  until (now<=goal);
  asm sti end; {switch interrupts on}
end;

procedure SimpleWait(time:real); {time in ms}
begin
  StartCounter;
  TimingWait(time);
end;

end.