unit Vision;
interface
uses Unimess,params;

procedure VisionSetComPort(aPort:byte);
function VisionCommand(comand:string):boolean;
procedure VisionMessage(Message:string);
function VisionTakeAbsorptionPicture(PictureTyp:char;
  TwoPictures:boolean;Li6ExpansionTime:double;Li7ExpansionTime:double;NoisePictureMode:integer;
  Calibration:double;TriggerDelay:double;TriggerDelay2:double;FinalTrap:string):boolean;
procedure VisionSendPictureData(MOTFluorescence:double;RecapFluo:double;
          FinalTrapType:TFinalTrap;
          BarCurrent:double;TransferCurrent:double;MOTCurrent:double;
          Bias:double;Gradiant:double;Curbature:double;
          LoadingTime:double;TimingError:double;Measured7LiOffsetFrequency:double;
          RampEndFrequency:double);
function VisionStartSerie(commentary:string;ParamNr:integer;ParamNames:string;
  regParams:boolean;UniMeasure:PUniMess):boolean;
function VisionStopSerie:boolean;
function VisionReady:boolean;
function VisionCameraReady(Camera:byte):boolean;
procedure VisionReset;
procedure VisionRegisterInformation1(i1:double);
procedure VisionRegisterInformation2(i1:double;i2:double);
procedure VisionRegisterInformation3(i1:double;i2:double;i3:double);
procedure VisionRegisterInformation(nr:byte;i:array of double);
procedure VisionRegisterParameters(UniMeasure:PUniMess);
procedure VisionNumberFit;
procedure VisionSimpleNumberFit;
procedure VisionTemperatureFit;
procedure VisionChargingTimeFit;
procedure VisionLifeTimeFit;
function VisionTakeProbePicture:boolean;
function VisionPictureOk:boolean;
procedure VisionCheckNumber;
procedure VisionSetCameraParameters(Camera:byte;BiningX:integer;
  BiningY:integer;StartX:integer;StartY:integer;Width:integer;Height:integer;
  CalX:double;CalY:double;halfpictureMode:integer;Temperature:double;
  HorizontalShiftSpeed:integer;FastKineticSeries:boolean;
  FKSExposedRows:integer;FKSImages:integer;FKSShiftSpeed:integer;FKSExposureTime:double;
  PictureSplitting:boolean;AndorFKSPictureDistance:integer;AndorFKSPictureOffset:integer;
  AndorFKSPictureHeight:integer;AndorFKSSoftBining:integer;
  AndorFKSTriggerMode:boolean;Gain:double;Offset:double);

var VisionInSerie:boolean;
    VisionSerieFirstPicture:boolean;

implementation
uses crt,serial,register,debug,ShutDown;

var Port:byte;
const initialized:boolean=false;

procedure VisionSetComPort(aPort:byte);
begin
  if not Initialized then begin
    {InitCOM(Port);
    SetProtocol (Port,9600,8,1,Pnone,true,true);
    SetTimeOut (Port,1000);}
    Port:=aPort;
    OpenPort(Port,115200);{9600 or  57600 115200}
  end;
  Initialized:=true;
end;

procedure WriteDouble(d:double);
var buf:string;
begin
  str(d:8:5,buf);
  PutLine(port,'*'+buf+'#');
end;

procedure WriteInteger(i:integer);
var buf:string;
begin
  str(i:8,buf);
  PutLine(port,'*'+buf+'#');
end;

procedure WriteBoolean(b:boolean);
begin
  if b then PutLine(port,'*TRUE#')
  else PutLine(port,'*FALSE#');
end;

procedure WriteString(s:string);
begin
  PutLine(port,'*'+s+'#');
end;

procedure WriteChar(c:char);
begin
  PutLine(port,'*'+c+'#');
end;

function GetCommand(var command:string):boolean;
var timeout:word;
    key,ein:char;
    ready:boolean;
begin
  ready:=false;
  timeout:=0;
  while keypressed do key:=readkey;
  key:='@';
  ein:='@';
  repeat
    {if AskCharNum(port)>0 then ein:=GetChar(port)}
    if not GetChar(port,ein) then
    begin
      delay(1);
      timeout:=timeout+1;
      while keypressed do key:=readkey;
      if key=#13 then begin
        PrintBuffer(port);
        key:='@';
      end;
    end;
  until (timeout>40000) or (ein='*') or (key=' ');
  if (ein='*') then begin
    command:='';
    repeat
      {if AskCharNum(port)>0 then begin ein:=GetChar(port);}
      if GetChar(port,ein) then begin
        if ein<>'#' then command:=command+ein;
      end else begin
        delay(1);
        timeout:=timeout+1;
      end;
    until (timeout>30000) or (ein='#');
    ready:=ein='#';
  end;
  if ready and Debuging then writeln(DebugFile,'Vision: ',command);
  GetCommand:=ready;
end;

function VisionCommand(comand:string):boolean;
begin
  {VisionReset;}
  PutLine(port,'*'+comand+'#');
  if not VisionReady then begin
    writeln('Vision not Ready! (Command: ',comand,')  Press any key to continue',^G);
    VisionCommand:=false;
    repeat until keypressed;
  end else VisionCommand:=true;
end;

procedure VisionMessage(Message:string);
begin
  if VisionCommand('VisionMessage') then begin
    WriteString(Message);
  end;
end;

function VisionTakeAbsorptionPicture(PictureTyp:char;
  TwoPictures:boolean;Li6ExpansionTime:double;Li7ExpansionTime:double;NoisePictureMode:Integer;
  Calibration:double;TriggerDelay:double;TriggerDelay2:double;FinalTrap:string):boolean;
var ok:boolean;
    buf:string;
begin
  ok:=VisionCommand('VisionTakeAbsorptionPicture');
  if ok then begin
    WriteInteger(CameraPosition);
    WriteInteger(CameraTyp[CameraPosition]);
    WriteChar(CameraCode[CameraPosition]);
    WriteChar(PictureTyp);
    if DetectIsotop=6 then WriteInteger(6) else WriteInteger(7);
    WriteDouble(3/2);
    if PumpToF_2 then WriteDouble(2) else WriteDouble(1);
    WriteDouble(Probe6PrincipalDetuning);
    WriteDouble(Probe7PrincipalDetuning);
    WriteDouble(Li6ExpansionTime);
    WriteDouble(Li7ExpansionTime);
    WriteDouble(Calibration);
    WriteDouble(TriggerDelay);
    WriteDouble(TriggerDelay2);
    if TwoPictures then WriteInteger(1) else WriteInteger(0);
    WriteInteger(NoisePictureMode);
    FillChar(buf,250,' ');
    buf[0]:=#0;
    if length(FinalTrap)>60 then begin
      buf:=copy(FinalTrap,length(FinalTrap)-59,60);
    end else buf:=FinalTrap;
    {if length(buf)>200 then begin
      writeln('Vision : TakeAbsorptionPicture : FinalTrap string too long');
    end else}
    buf[0]:=#60;
    {PutLine(port,buf);}
    WriteString(buf);
  end;
  VisionTakeAbsorptionPicture:=ok;
end;

function VisionTakeProbePicture:boolean;
begin
  VisionTakeProbePicture:=VisionCommand('VisionTakeProbePicture');
end;


procedure SendParamNames(ParamNr:integer;ParamNames:string);
var buf:string;
    i:integer;
    h:integer;
begin
  str(ParamNr,buf);
  PutLine(port,'*'+buf+'#');
  for i:=1 to ParamNr do begin
    h:=pos(' ',ParamNames);
    if h>0 then begin
      buf:=copy(ParamNames,1,h);
      ParamNames:=copy(ParamNames,h+1,255);
    end else buf:=ParamNames;
    PutLine(port,'*'+buf+'#');
  end;
end;

function VisionStartSerie(commentary:string;ParamNr:integer;ParamNames:string;
  regParams:boolean;UniMeasure:PUniMess):boolean;
begin
  VisionStartSerie:=VisionCommand('VisionStartSerie');
  SendParamNames(ParamNr,ParamNames);
  PutLine(port,'*'+commentary+'#');
  if regParams then VisionRegisterParameters(UniMeasure);
  VisionInSerie:=true;
  VisionSerieFirstPicture:=true;
end;

function VisionStopSerie:boolean;
begin
  VisionStopSerie:=VisionCommand('VisionStopSerie');
  VisionInSerie:=false;
end;

function VisionReady:boolean;
var buf:string;
begin
 WaitForSend(port);
 buf:='';
 if GetCommand(buf) then
   VisionReady:=buf='Ready'
 else VisionReady:=false;
end;

procedure VisionReset;
begin
  {ClearBuf(port);}
  {FlushSerial(Port);}
end;

procedure VisionRegisterInformation1(i1:double);
var buf:string;
begin
  if VisionCommand('VisionRegisterInformation') then WriteDouble(i1);
end;

procedure VisionRegisterInformation2(i1:double;i2:double);
var buf:string;
begin
  if VisionCommand('VisionRegisterInformation') then begin
    WriteDouble(i1);
    WriteDouble(i2);
  end;
end;

procedure VisionRegisterInformation3(i1:double;i2:double;i3:double);
var buf:string;
begin
  if VisionCommand('VisionRegisterInformation') then begin
    WriteDouble(i1);
    WriteDouble(i2);
    WriteDouble(i3);
  end;
end;

procedure VisionRegisterInformation(nr:byte;i:array of double);
var n:byte;
begin
  if VisionCommand('VisionRegisterInformation') then begin
    for n:=0 to nr-1 do WriteDouble(i[n]);
  end;
end;

procedure VisionRegisterBigInformation(nr:byte;i:array of double);
var n:byte;
begin
  if VisionCommand('VisionRegisterBigInformation') then begin
    for n:=0 to nr-1 do WriteDouble(i[n]);
  end;
end;

procedure VisionRegisterBigInformation1(i1:double);
var buf:string;
begin
  if VisionCommand('VisionRegisterBigInformation') then WriteDouble(i1);
end;

procedure VisionRegisterBigInformation2(i1:double;i2:double);
var buf:string;
begin
  if VisionCommand('VisionRegisterBigInformation') then begin
    WriteDouble(i1);
    WriteDouble(i2);
  end;
end;

procedure VisionRegisterBigInformation3(i1:double;i2:double;i3:double);
var buf:string;
begin
  if VisionCommand('VisionRegisterBigInformation') then begin
	 WriteDouble(i1);
	 WriteDouble(i2);
	 WriteDouble(i3);
  end;
end;

procedure VisionRegisterBigInformation4(i1:double;i2:double;i3:double;i4:double);
var buf:string;
begin
  if VisionCommand('VisionRegisterBigInformation') then begin
    WriteDouble(i1);
    WriteDouble(i2);
    WriteDouble(i3);
    WriteDouble(i4);
  end;
end;

procedure VisionSynchronizeParameters;
var buf:string;
    SendAll:boolean;
begin
  if VisionCommand('VisionSynchronizeParameters') then begin
    GetCommand(buf);
    SendVariables(port,buf='SendAll',LastShutdownSuccessfull);
    VisionCommand('VisionSynchronizeParametersEnd')
  end;
end;

procedure VisionRegisterParameters(UniMeasure:PUniMess);
begin
  VisionSynchronizeParameters;
  if VisionCommand('VisionRegisterParameters') then begin
    if (uniMeasure<>NIL) then UniMeasure^.SendVariables(port);
    VisionCommand('VisionRegisterParametersEnd')
  end;
end;

procedure VisionNumberFit;
begin
  VisionCommand('VisionNumberFit');
end;

procedure VisionSimpleNumberFit;
begin
  VisionCommand('VisionSimpleNumberFit');
end;

procedure VisionTemperatureFit;
begin
  VisionCommand('VisionTemperatureFit');
end;

procedure VisionChargingTimeFit;
begin
  VisionCommand('VisionChargingTimeFit');
end;

procedure VisionLifeTimeFit;
begin
  VisionCommand('VisionLifetimeFit');
end;

procedure VisionCheckNumber;
begin
  VisionCommand('VisionCheckNumber');
end;

function VisionPictureOk:boolean;
var buf:string;
    ein:char;
    TimeOut:longint;
begin
  if GetCommand(buf) then
    VisionPictureOk:=buf='VisionPictureOk'
  else begin
    writeln('Vision : VisionPictureOk : no response from Vision');
    writeln('Is the picture ok (Y/N)?');
    VisionMessage('No response from Vision, is the picture ok (Y/N)?');
    timeout:=0;
    ein:='Y';
    repeat
        repeat
        delay(1);
        inc(timeout);
      until keypressed or (timeout>10000);
      while keypressed do ein:=upcase(readkey);
    until (ein in ['Y','N']) or (Timeout>10000);
    VisionPictureOk:=ein='Y';
  end;
end;

procedure VisionSetCameraParameters(Camera:byte;BiningX:integer;
  BiningY:integer;StartX:integer;StartY:integer;Width:integer;Height:integer;
  CalX:double;CalY:double;halfpictureMode:integer;Temperature:double;
  HorizontalShiftSpeed:integer;FastKineticSeries:boolean;
  FKSExposedRows:integer;FKSImages:integer;FKSShiftSpeed:integer;FKSExposureTime:double;
  PictureSplitting:boolean;AndorFKSPictureDistance:integer;AndorFKSPictureOffset:integer;
  AndorFKSPictureHeight:integer;AndorFKSSoftBining:integer;
  AndorFKSTriggerMode:boolean;
  Gain:double;Offset:double);
begin
  if VisionCommand('VisionSetCameraParameters') then begin
    WriteInteger(Camera);
    WriteDouble(CalX);
    WriteDouble(CalY);
    WriteInteger(StartX);
    WriteInteger(StartY);
    WriteInteger(Width);
    WriteInteger(Height);
    WriteInteger(BiningX);
    WriteInteger(BiningY);
    if PictureSplitting then WriteInteger(1) else WriteInteger(0);
    case Camera of
      0:begin
          WriteInteger(halfpictureMode);
          WriteDouble(Gain);
          WriteDouble(Offset);
        end;
      1:begin
          WriteInteger(round(Temperature));
          WriteInteger(HorizontalShiftSpeed);
          WriteBoolean(FastKineticSeries);
          WriteInteger(FKSShiftSpeed);
          WriteInteger(FKSExposedRows);
          WriteInteger(FKSImages);
          WriteDouble(FKSExposureTime/1000);
          WriteInteger(AndorFKSPictureDistance);
          WriteInteger(AndorFKSPictureOffset);
          WriteInteger(AndorFKSPictureHeight);
          WriteInteger(AndorFKSSoftBining);
          if AndorFKSTriggerMode then WriteInteger(1) else WriteInteger(0);
        end;
    end;
  end;
end;

function VisionCameraReady(Camera:byte):boolean;
var buf:string;
begin
  if VisionCommand('VisionCameraReady') then begin
    WriteInteger(Camera);
    if GetCommand(buf) then VisionCameraReady:=buf='VisionCameraReady'
    else begin
      writeln('Vision : VisionCameraReady : no response from Vision');
      VisionCameraReady:=false;
    end;
  end else begin
    writeln('Vision : VisionCameraReady : no response from Vision');
    VisionCameraReady:=false;
  end;
end;

procedure VisionSendPictureData(MOTFluorescence:double;RecapFluo:double;
          FinalTrapType:TFinalTrap;
          BarCurrent:double;TransferCurrent:double;MOTCurrent:double;
          Bias:double;Gradiant:double;Curbature:double;
          LoadingTime:double;TimingError:double;Measured7LiOffsetFrequency:double;
          RampEndFrequency:double);
begin
  VisionSynchronizeParameters;
  WriteDouble(MOTFluorescence);
  WriteDouble(RecapFluo);
  case FinalTrapType of
    MOT:WriteChar('M');
    QP:WriteChar('Q');
    IoffePritchard:WriteChar('I');
  end;
  WriteDouble(BarCurrent);
  WriteDouble(TransferCurrent);
  WriteDouble(MOTCurrent);
  WriteDouble(Bias);
  WriteDouble(Gradiant);
  WriteDouble(Curbature);
  WriteDouble(LoadingTime);
  WriteDouble(TimingError);
  WriteDouble(Measured7LiOffsetFrequency);
  WriteDouble(RampEndFrequency);
end;

begin
  Port:=1;
  VisionInSerie:=false;
  VisionSerieFirstPicture:=true;
end.